@@ -124,7 +124,7 @@ public class ClassType extends ObjectType
           {
             return reflectClass.getSimpleName();
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           { /* ... fall thorugh ... */ }
       }
     /* #endif */
@@ -264,7 +264,7 @@ public class ClassType extends ObjectType
 
   /** Note that this class needs an other link ("this$0") field.
    * This is only allowed if !isExisting().
-   * Adjust any existing "<init>" methods to take the extra
+   * Adjust any existing {@code "<init>"} methods to take the extra
    * implicit parameter.
    * @param outer the outer class
    */
@@ -617,7 +617,7 @@ public class ClassType extends ObjectType
   }
 
   /** Use reflection to add all the declared fields of this class.
-   * Does not add private or package-private fields.
+   * Does not add private fields.
    * Does not check for duplicate (already-known) fields.
    * Is not thread-safe if another thread may access this ClassType. */
   public synchronized void addFields()
@@ -638,8 +638,13 @@ public class ClassType extends ObjectType
         java.lang.reflect.Field field = fields[i];
         if ("this$0".equals(field.getName()))
           flags |= HAS_OUTER_LINK;
-        addField(field.getName(), Type.make(field.getType(), field.getGenericType()),
-                 field.getModifiers());
+        int mods = field.getModifiers();
+        if ((mods & Access.PRIVATE) == 0) {
+            Field fld = addField(field.getName(),
+                                 null, // lazy type lookup
+                                 mods);
+            fld.rfield = field;
+        }
       }
     flags |= ADD_FIELDS_DONE;
   }
@@ -708,7 +713,9 @@ public class ClassType extends ObjectType
     while (--j >= 0)
 	args[j] = Type.make(paramTypes[j], gparamTypes[j]);
     Type rtype = Type.make(method.getReturnType(), method.getGenericReturnType());
-    return addMethod(method.getName(), modifiers, args, rtype);
+    Method meth = addMethod(method.getName(), modifiers, args, rtype);
+    meth.rmethod = method;
+    return meth;
   }
 
   public Method addMethod (java.lang.reflect.Constructor method)
@@ -719,7 +726,9 @@ public class ClassType extends ObjectType
     Type[] args = new Type[j];
     while (--j >= 0)
       args[j] = Type.make(paramTypes[j]);
-    return addMethod("<init>", modifiers, args, Type.voidType);
+    Method meth = addMethod("<init>", modifiers, args, Type.voidType);
+    meth.rmethod = method;
+    return meth;
   }
 
   public Method addMethod (String name,  String signature, int flags)
@@ -1309,8 +1318,7 @@ public class ClassType extends ObjectType
   {
     if (other.isInterface())
       return implementsInterface(other);
-    if ((this == toStringType && other == javalangStringType)
-	|| (this == javalangStringType && other == toStringType))
+    if ((this == javalangStringType && other == toStringType))
       return true;
     if (other == Type.javalangObjectType)
       return true;
@@ -1324,6 +1332,23 @@ public class ClassType extends ObjectType
     return false;
   }
 
+    @Override
+    public int isCompatibleWithValue(Type valueType) {
+        if (this == objectType && valueType instanceof ObjectType)
+            return 2;
+        int comp = compare(valueType);
+        if (comp >= 0) {
+            if (valueType instanceof ClassType
+                || valueType instanceof ParameterizedType
+                || valueType instanceof TypeVariable)
+                return 2;
+            else
+                return 1;
+        } else
+            return comp == -3 ? -1 : 0;
+    }
+
+
   public int compare(Type other)
   {
     if (other == nullType)
@@ -1338,10 +1363,6 @@ public class ClassType extends ObjectType
       return -1;
     if (cother.isSubclass(this))
       return 1;
-    if (this == toStringType)
-      return cother == Type.javalangObjectType ? -1 : 1;
-    if (cother == toStringType)
-      return this == Type.javalangObjectType ? 1 : -1;
     if (this.isInterface())
       return cother.isAnnotation() || cother.isFinal() ? -3
         : cother == Type.javalangObjectType ? -1 : -2;
