@@ -960,6 +960,14 @@ public class CodeAttr extends Attribute implements AttrContainer
     pushType(Type.javalangClassType);
   }
 
+    /** Push a MethodHandle, using an appropriate constant pool entry.
+     * This is only supported by JDK 1.6 and later.
+     */
+    public final void emitPushMethodHandle(Method method) {
+        emitPushConstant(getConstants().addMethodHandle(method));
+        pushType(Type.javalanginvokeMethodHandleType);
+    }
+
   public void emitPushNull ()
   {
     reserve(1);
@@ -1198,7 +1206,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   /** Compile code to allocate a new array.
    * The size should have been already pushed on the stack.
    * @param element_type type of the array elements
-   * @params dims number of dimensions - more than 1 is untested
+   * @param dims number of dimensions - more than 1 is untested
    */
   public void emitNewArray (Type element_type, int dims)
   {
@@ -1445,7 +1453,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public final void emitGetStatic(Field field)
   {
-    pushType(field.type);
+    pushType(field.getType());
     emitFieldop (field, 178);  // getstatic
   }
 
@@ -1455,7 +1463,7 @@ public class CodeAttr extends Attribute implements AttrContainer
   public final void emitGetField(Field field)
   {
     popType();
-    pushType(field.type);
+    pushType(field.getType());
     emitFieldop(field, 180);  // getfield
   }
 
@@ -1531,8 +1539,8 @@ public class CodeAttr extends Attribute implements AttrContainer
         for (int i = used == null ? 0 : used.length;  --i >= 0; )
           {
             Variable var = used[i];
-            if (var != null && var.type == t)
-              var.type = ctype;
+            if (var != null && var.getType() == t)
+                var.setType(ctype);
           }
         for (int i = local_types == null ? 0 : local_types.length;  --i >= 0; )
           if (local_types[i] == t)
@@ -1657,6 +1665,10 @@ public class CodeAttr extends Attribute implements AttrContainer
   { emitGotoIfCompare1(label, 157); }
   public final void emitGotoIfIntLeZero(Label label)
   { emitGotoIfCompare1(label, 158); }
+  public final void emitGotoIfNull(Label label)
+  { emitGotoIfCompare1(label, 198); }
+  public final void emitGotoIfNonNull(Label label)
+  { emitGotoIfCompare1(label, 199); }
 
   public final void emitGotoIfCompare2 (Label label, int logop)
   { 
@@ -1723,7 +1735,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The value of <var>x</var> must already have been pushed. */
   public final void emitIfCompare1 (int opcode)
   {
-    IfState new_if = new IfState(this);
+    IfState new_if = pushIfState();
     if (popType().promote() != Type.intType)
       throw new Error ("non-int type to emitIfCompare1");
     reserve(3);
@@ -1756,7 +1768,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * reference type. */
   public final void emitIfRefCompare1 (int opcode)
   {
-    IfState new_if = new IfState(this);
+    IfState new_if = pushIfState();
     if (! (popType() instanceof ObjectType))
       throw new Error ("non-ref type to emitIfRefCompare1");
     reserve(3);
@@ -1780,7 +1792,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The value of x and y must already have been pushed. */
   public final void emitIfIntCompare(int opcode)
   {
-    IfState new_if = new IfState(this);
+    IfState new_if = pushIfState();
     popType();
     popType();
     reserve(3);
@@ -1798,7 +1810,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfNEq ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfEq(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1807,7 +1819,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfEq ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfNE(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1816,7 +1828,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfLt ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfGe(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1825,7 +1837,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfGe ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfLt(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1834,7 +1846,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfGt ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfLe(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1843,7 +1855,7 @@ public class CodeAttr extends Attribute implements AttrContainer
    * The values of x and y must already have been pushed. */
   public final void emitIfLe ()
   {
-    IfState new_if = new IfState (this);
+    IfState new_if = pushIfState();
     emitGotoIfGt(new_if.end_label);
     new_if.start_stack_size = SP;
   }
@@ -1942,6 +1954,31 @@ public class CodeAttr extends Attribute implements AttrContainer
     if_stack = if_stack.previous;
   }
 
+    /** Convenience for compiling {@code if P1 && P2 then S1 else S2}.
+     * Compile that as:
+     * <pre>
+     * compile P1, including an appropriate emitIfXxx
+     * emitAndThen()
+     * compile P2, including an appropriate emitIfXxx
+     * compile S1
+     * emitElse
+     * compile S2
+     * emitFi
+     * </pre>
+     */
+    public void emitAndThen() {
+        if (if_stack==null||if_stack.andThenSet) throw new InternalError();
+        if_stack.andThenSet = true;
+    }
+
+    private IfState pushIfState() {
+        if (if_stack!=null && if_stack.andThenSet) {
+            if_stack.andThenSet = false;
+            return if_stack;
+        }
+        return new IfState(this);
+    }
+
   public final void emitConvert (Type from, Type to)
   {
     String to_sig = to.getSignature();
@@ -2025,8 +2062,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 
   public static boolean castNeeded (Type top, Type required)
   {
-    if (top instanceof UninitializedType)
-      top = ((UninitializedType) top).getRawType();
+    top = top.getRawType();
     for (;;)
       {
         if (top == required)
@@ -2506,19 +2542,27 @@ public class CodeAttr extends Attribute implements AttrContainer
     emitTailCall(pop_args, scope.start);
   }
 
-  public void processFixups ()
-  {
-    if (fixup_count <= 0)
-      return;
+    public void processFixups() {
+        if (fixup_count <= 0)
+            return;
 
-    // For each label, set it to its maximum limit, assuming all
-    // fixups causes the code the be expanded.  We need a prepass
-    // for this, since FIXUP_MOVEs can cause code to be reordered.
-    // Also, convert each FIXUP_MOVE_TO_END to FIXUP_MOVE.
+        // For each label, set it to its maximum limit, assuming all
+        // fixups causes the code the be expanded.  We need a prepass
+        // for this, since FIXUP_MOVEs can cause code to be reordered.
+        // Also, convert each FIXUP_MOVE_TO_END to FIXUP_MOVE.
 
-    int delta = 0;
-    int instruction_tail = fixup_count;
-    fixupAdd(CodeAttr.FIXUP_MOVE, 0, null);
+        int delta = 0;
+        int instruction_tail = fixup_count;
+        fixupAdd(CodeAttr.FIXUP_MOVE, 0, null);
+
+        /* DEBUGGING:
+        System.err.println("processFixups for "+getMethod());
+        ClassTypeWriter writer =
+            new ClassTypeWriter(getMethod().getDeclaringClass(), System.err, 0);
+        writer.println("processFixups for "+getMethod());
+        disAssembleWithFixups(writer);
+        writer.flush();
+        */
 
   loop1:
    for (int i = 0;  ;  )
@@ -2591,7 +2635,7 @@ public class CodeAttr extends Attribute implements AttrContainer
 
     // Number of bytes to be inserted or (if negative) removed, so far.
     int new_size = PC;
-    // Current delta between final PC and offset in generate code array.
+    // Current delta between final PC and offset in generated code array.
     delta = 0;
   loop2:
     for (int i = 0;  i < fixup_count;  )
@@ -2908,88 +2952,90 @@ public class CodeAttr extends Attribute implements AttrContainer
     dst.printAttributes(this);
   }
 
-  /* DEBUGGING:
-  public void disAssembleWithFixups (ClassTypeWriter dst)
-  {
-    if (fixup_count <= 0)
-      {
-	disAssemble(dst, 0, PC);
-	return;
-      }
-    int prev_pc = 0;
-    for (int i = 0;  i < fixup_count; )
-      {
-	int offset = fixup_offsets[i];
-	int kind = offset & 15;
-	Label label = fixup_labels[i];
-	offset = offset >> 4;
-	int pc = offset;
-	if (kind == FIXUP_MOVE || kind == FIXUP_MOVE_TO_END)
-	  pc = (i+1 >= fixup_count) ? PC : fixup_offsets[i+1] >> 4;
-	disAssemble(dst, prev_pc, offset);
-	prev_pc = pc;
-	dst.print("fixup#");  dst.print(i);
-	dst.print(" @");  dst.print(offset);
-	switch (kind)
-	  {
-	  case FIXUP_NONE:
-	    dst.println(" NONE");
-	    break;
-	  case FIXUP_DEFINE:
-	    dst.print(" DEFINE ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_SWITCH:
-	    dst.println(" SWITCH");
-	    break;
-	  case FIXUP_CASE:
-	    dst.println(" CASE");
-	    break;
-	  case FIXUP_GOTO:
-	    dst.print(" GOTO ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_TRANSFER:
-	    dst.print(" TRANSFER ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_TRANSFER2:
-	    dst.print(" TRANSFER2 ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_DELETE3:
-	    dst.println(" DELETE3");
-	    break;
-	  case FIXUP_MOVE:
-	    dst.print(" MOVE ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_MOVE_TO_END:
-	    dst.print(" MOVE_TO_END ");
-	    dst.println(label);
-	    break;
-	  case FIXUP_TRY:
-	    dst.print(" TRY start: ");
-	    dst.println(label);
-	    i++;
-	    dst.print(" - end: ");
-	    dst.print(fixup_labels[i]);
-	    dst.print(" type: ");
-	    dst.println(fixup_offsets[i] >> 4);
+    /* DEBUGGING:
+    public void disAssembleWithFixups(ClassTypeWriter dst) {
+        if (fixup_count <= 0) {
+            disAssemble(dst, 0, PC);
+            return;
+        }
+        int prev_pc = 0;
+        for (int i = 0;  i < fixup_count; ) {
+            int offset = fixup_offsets[i];
+            int kind = offset & 15;
+            Label label = fixup_labels[i];
+            offset = offset >> 4;
+            int pc = offset;
+            if (kind == FIXUP_MOVE || kind == FIXUP_MOVE_TO_END)
+                pc = (i+1 >= fixup_count) ? PC : fixup_offsets[i+1] >> 4;
+            else if (kind == FIXUP_CASE)
+                pc = prev_pc;
+            disAssemble(dst, prev_pc, pc);
+
+            dst.print("fixup#");  dst.print(i);
+            dst.print(" @");  dst.print(offset);
+            prev_pc = pc;
+            switch (kind) {
+            case FIXUP_NONE:
+                dst.println(" NONE");
+                break;
+            case FIXUP_DEFINE:
+                dst.print(" DEFINE ");
+                dst.println(label);
+                break;
+            case FIXUP_SWITCH:
+                dst.println(" SWITCH");
+                break;
+            case FIXUP_CASE:
+                dst.print(" CASE ");
+                dst.println(label);
+                break;
+            case FIXUP_GOTO:
+                dst.print(" GOTO ");
+                dst.println(label);
+                break;
+            case FIXUP_TRANSFER:
+                dst.print(" TRANSFER ");
+                dst.println(label);
+                break;
+            case FIXUP_TRANSFER2:
+                dst.print(" TRANSFER2 ");
+                dst.println(label);
+                break;
+            case FIXUP_DELETE3:
+                dst.println(" DELETE3");
+                break;
+            case FIXUP_MOVE:
+                dst.print(" MOVE ");
+                dst.println(label);
+                break;
+            case FIXUP_MOVE_TO_END:
+                dst.print(" MOVE_TO_END ");
+                dst.println(label);
+                break;
+            case FIXUP_TRY:
+                dst.print(" TRY start: ");
+                dst.println(label);
+                i++;
+                dst.print(" - end: ");
+                dst.print(fixup_labels[i]);
+                dst.print(" type: ");
+                dst.println(fixup_offsets[i] >> 4);
+                i++;
+                break;
+            case FIXUP_LINE_PC:
+                dst.print(" LINE ");
+                i++;
+                dst.println(fixup_offsets[i] >> 4);
+                break;
+            default:
+                dst.println(" kind:"+fixupKind(i)+" offset:"+fixupOffset(i)
+                            +" "+fixup_labels[i]);
+            }
             i++;
-	    break;
-	  case FIXUP_LINE_PC:
-	    dst.print(" LINE ");
-	    i++;
-	    dst.println(fixup_offsets[i] >> 4);
-	    break;
-	  default:
-	    dst.println(" kind:"+fixupKind(i)+" offset:"+fixupOffset(i)+" "+fixup_labels[i]);
-	  }
-	i++;
-      }
-  }
-  */
+        }
+        disAssemble(dst, prev_pc, PC);
+    }
+    */
 
   public void disAssemble (ClassTypeWriter dst, int start, int limit)
   {
