@@ -441,10 +441,16 @@ public abstract class Type
   }
 
     /** If this is the target type, is a given source type compatible?
-     * Return -1 if no; 1 if yes; 0 if need to check at run-time. */
+     * Return -1 if no; 1 if yes; 0 if need to check at run-time.
+     * @return -1 if not compatible; 0 if need to check at run-time;
+     *   1 if compatible; 2 if compatible and no conversion or cast needed.
+     */
     public int isCompatibleWithValue(Type valueType) {
         if (this == toStringType)
-            return 1;
+            return valueType == javalangStringType ? 2 : 1;
+        // Hack for LangPrimType.charType.
+        if (this == charType && valueType.getImplementationType()== this)
+            return 2;
         int comp = compare(valueType);
         return comp >= 0 ? 1 : comp == -3 ? -1 : 0;
     }
@@ -545,14 +551,21 @@ public abstract class Type
     code.emitInstanceof(this);
   }
 
-  /** Convert an object to a value of this Type.
-   * Throw a ClassCastException when this is not possible. */
-  public abstract Object coerceFromObject (Object obj);
+    /** Convert an object to a value of this Type.
+     * The result is actually of the implementation type, boxed as appropriate,
+     * so it is suitable for standard reflective operations.
+     * Throw a ClassCastException when this is not possible.
+     */
+    public abstract Object coerceFromObject (Object obj);
 
-  public Object coerceToObject (Object obj)
-  {
-    return obj;
-  }
+    /** Given a raw JVM value convert it to an object of this type.
+     * I.e. the argument is an object of the type returned by
+     * {@code getRawType()}, boxed as needed.  The result may be
+     * a language-specific (boxed) value. Generally a no-op.
+     */
+    public Object coerceToObject(Object obj) {
+        return obj;
+    }
 
   /** Convert from stackType (usually PrimType) to this type.
    * However, we might only convert part-way, to some object type.
@@ -628,21 +641,13 @@ public abstract class Type
     public static final Type neverReturnsType
         = ClassType.make("gnu.bytecode.Type$NeverReturns");
 
-  /** The magic type of null. */
-  public static final ObjectType nullType = new ObjectType("(type of null)");
-
-  public static final ObjectType errorType = new ClassType("(error type)");
-
-  static public ClassType javalangStringType = ClassType.make("java.lang.String");
-  /* The String type. but coercion is handled by toString. */
-  public static final ClassType toStringType
-    = new ClassType("java.lang.String");
-
   public static final ClassType javalangObjectType
     = ClassType.make("java.lang.Object");
   public static final ClassType objectType = javalangObjectType;
   public static final ClassType javalangBooleanType
     = ClassType.make("java.lang.Boolean");
+  public static final ClassType javalangCharacterType
+    = ClassType.make("java.lang.Character");
   public static final ClassType javalangThrowableType
     = ClassType.make("java.lang.Throwable");
   public static final ClassType javalangannotationAnnotationType
@@ -671,12 +676,28 @@ public abstract class Type
 			      booleanType, Access.PUBLIC);
   public static final ClassType javalangClassType
     = ClassType.make("java.lang.Class");
+
+    public static final ClassType javalanginvokeMethodHandleType
+        = ClassType.make("java.lang.invoke.MethodHandle");
+
+    /** The magic type of null. */
+    public static final ObjectType nullType
+    //= new ObjectType("(type of null)");
+    = new SpecialObjectType("(type of null)", objectType);
+    public static final ObjectType errorType = new ObjectType("(error type)");
+
+    static public ClassType javalangStringType = ClassType.make("java.lang.String");
+    /* The String type. but coercion is handled by toString. */
+    public static final ObjectType toStringType
+        = new SpecialObjectType("String", javalangStringType);
+    //= new ClassType("java.lang.String");
+
   /** @deprecated */
   public static final ClassType pointer_type = javalangObjectType;
   /** @deprecated */
   public static final ClassType string_type = javalangStringType;
   /** @deprecated */
-  public static final ClassType tostring_type = toStringType;
+  public static final ObjectType tostring_type = toStringType;
   /** @deprecated */
   public static final ClassType java_lang_Class_type = javalangClassType;
   /** @deprecated */
