@@ -7,6 +7,7 @@ import gnu.lists.*;
 import gnu.expr.*;
 import gnu.text.Char;
 import kawa.standard.Scheme;
+import gnu.kawa.functions.*;
 import gnu.bytecode.Type;
 import gnu.kawa.lispexpr.LangPrimType;
 import gnu.kawa.functions.DisplayFormat;
@@ -64,7 +65,9 @@ public class CommonLisp extends Lisp2
   public static final NumberCompare numLss;
   public static final NumberCompare numLEq;
 
-  public static final gnu.kawa.functions.IsEq isEq;
+  public static final Not not;
+  public static final IsEq isEq;
+  public static final IsEqv isEqv;
   
   /** Package location symbols. */
   public static final Symbol internalKeyword = Keyword.make("INTERNAL");
@@ -77,6 +80,7 @@ public class CommonLisp extends Lisp2
 
     instance.define("t", TRUE);
     instance.define("nil", FALSE);
+    not = new Not(instance, "not");
     numEqu = NumberCompare.make(instance, "=",
                                 NumberCompare.TRUE_IF_EQU);
     numGrt = NumberCompare.make(instance, ">",
@@ -87,7 +91,9 @@ public class CommonLisp extends Lisp2
                                 NumberCompare.TRUE_IF_LSS);
     numLEq = NumberCompare.make(instance, "<=",
                                 NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
-    isEq = new gnu.kawa.functions.IsEq(instance, "eq?");
+    isEq = new IsEq(instance, "eq?");
+    isEqv = new IsEqv(instance, "eqv?", isEq);
+    
     Environment saveEnv = Environment.setSaveCurrent(clispEnvironment);
     try
       {
@@ -146,23 +152,80 @@ public class CommonLisp extends Lisp2
     defun("prog2", prog1.prog2);
     defun("progn", new kawa.standard.begin());
     defun("unwind-protect", new gnu.commonlisp.lang.UnwindProtect());
-    Procedure not = new gnu.kawa.functions.Not(this);
-    defun("not", not);
     defun("null", not);
-    defun("eq", new gnu.kawa.functions.IsEq(this, "eq"));
-    defun("equal", new gnu.kawa.functions.IsEqual(this, "equal"));
+    defun("eq", new IsEq(this, "eq"));
+    defun("equal", new IsEqual(this, "equal"));
     defun("typep", new gnu.kawa.reflect.InstanceOf(this));
+    defProcStFld("the", "gnu.kawa.functions.Convert", "as");
     defun("%flet", new kawa.standard.let("flet", true));
     defProcStFld("princ", "gnu.commonlisp.lisp.PrimOps");
     defProcStFld("prin1", "gnu.commonlisp.lisp.PrimOps");
 
+    defAliasStFld("*package*", "gnu.kawa.lispexpr.LispPackage", "currentPackage");
     defProcStFld("=", "gnu.commonlisp.lang.CommonLisp", "numEqu");
     defProcStFld("<", "gnu.commonlisp.lang.CommonLisp", "numLss");
     defProcStFld(">", "gnu.commonlisp.lang.CommonLisp", "numGrt");
     defProcStFld("<=", "gnu.commonlisp.lang.CommonLisp", "numLEq");
     defProcStFld(">=", "gnu.commonlisp.lang.CommonLisp", "numGEq");
-
+    defProcStFld("not", "gnu.commonlisp.lang.CommonLisp");
+    defProcStFld("eq?", "gnu.commonlisp.lang.CommonLisp", "isEq");
+    defProcStFld("eqv?", "gnu.commonlisp.lang.CommonLisp", "isEqv");
     defProcStFld("functionp", "gnu.commonlisp.lisp.PrimOps");
+    defProcStFld("car", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("first", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("cdr", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("caar", "kawa.lib.lists");
+    defProcStFld("cadr", "kawa.lib.lists");
+    defProcStFld("cdar", "kawa.lib.lists");
+    defProcStFld("cddr", "kawa.lib.lists");
+    defProcStFld("caaar", "kawa.lib.lists");
+    defProcStFld("caadr", "kawa.lib.lists");
+    defProcStFld("cadar", "kawa.lib.lists");
+    defProcStFld("caddr", "kawa.lib.lists");
+    defProcStFld("cdaar", "kawa.lib.lists");
+    defProcStFld("cdadr", "kawa.lib.lists");
+    defProcStFld("cddar", "kawa.lib.lists");
+    defProcStFld("cdddr", "kawa.lib.lists");
+    defProcStFld("caaaar", "kawa.lib.lists");
+    defProcStFld("caaadr", "kawa.lib.lists");
+    defProcStFld("caadar", "kawa.lib.lists");
+    defProcStFld("caaddr", "kawa.lib.lists");
+    defProcStFld("cadaar", "kawa.lib.lists");
+    defProcStFld("cadadr", "kawa.lib.lists");
+    defProcStFld("caddar", "kawa.lib.lists");
+    defProcStFld("cadddr", "kawa.lib.lists");
+    defProcStFld("cdaaar", "kawa.lib.lists");
+    defProcStFld("cdaadr", "kawa.lib.lists");
+    defProcStFld("cdadar", "kawa.lib.lists");
+    defProcStFld("cdaddr", "kawa.lib.lists");
+    defProcStFld("cddaar", "kawa.lib.lists");
+    defProcStFld("cddadr", "kawa.lib.lists");
+    defProcStFld("cdddar", "kawa.lib.lists");
+    defProcStFld("cddddr", "kawa.lib.lists");
+    defProcStFld("rest", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("second", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("third", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("nthcdr", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("nth", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("1-", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("1+", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("acons", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("listp", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("numberp", "gnu.commonlisp.lisp.primitives");
+    defProcStFldAs("zerop", "kawa.lib.numbers", "zero?");
+    defProcStFldAs("consp", "kawa.lib.lists", "pair?");
+    defProcStFld("atom", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("eql", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("member", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("complement", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("apply", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("funcall", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("minusp", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("plusp", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("flet", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("labels", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("multiple-value-bind", "gnu.commonlisp.lisp.primitives");
+    defProcStFld("floor", "gnu.commonlisp.lisp.primitives");
   }
 
   public static CommonLisp getInstance()
