@@ -7,11 +7,20 @@ import gnu.mapping.*;
 import java.util.*;
 import java.io.*;
 import kawa.Shell;
-import gnu.text.*;
 import java.util.zip.*;
 import java.util.Stack;
 import gnu.kawa.functions.Convert;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.kawa.lispexpr.LangPrimType;
 import gnu.kawa.reflect.LazyType;
+import gnu.lists.Pair;
+import gnu.text.Char;
+import gnu.text.Lexer;
+import gnu.text.Options;
+import gnu.text.SourceLocator;
+import gnu.text.SourceMessages;
+import kawa.lang.Translator.FormStack;
 
 /** State for a single expression or module.
  * For each top-level thing (expression or file) we compile or evaluate
@@ -70,11 +79,11 @@ public class Compilation implements SourceLocator
   public static final int CLASS_WRITTEN = 16;
   public static final int ERROR_SEEN = 100;
 
-  public ModuleInfo minfo;
   public Lexer lexer;
 
-  boolean pedantic;
-  public boolean isPedantic() { return pedantic; }
+    private boolean pedantic;
+    public boolean isPedantic() { return pedantic; }
+    public void setPedantic(boolean value) { pedantic = value; }
 
   /** Used to access the "main" instance.
    * This is used for two different purposes, which may be confusing:
@@ -94,7 +103,7 @@ public class Compilation implements SourceLocator
   /** Stack of quads of (ModuleInfo, ScopeExp, position, formSize). */
   public java.util.Stack<Object> pendingImports;
 
-  public void pushPendingImport (ModuleInfo info, ScopeExp defs, int formSize)
+  public void pushPendingImport(ModuleInfo info, ScopeExp defs, FormStack forms)
   {
     if (pendingImports == null)
       pendingImports = new java.util.Stack<Object>();
@@ -103,9 +112,14 @@ public class Compilation implements SourceLocator
     Expression posExp = new ReferenceExp((Object) null);
     posExp.setLine(this);
     pendingImports.push(posExp);
-    pendingImports.push(Integer.valueOf(formSize));
+    pendingImports.push(forms.lastPair());
   }
 
+    /* Write out implicitly-compiled classes.
+     * Compare javac's -implicit:class flag.  Current not set.
+     */
+    public static boolean writeImplicitClasses = false;
+    
   /** If true, print out expressions after parsing and before optimizations. */
   public static boolean debugPrintExpr = false;
 
@@ -202,10 +216,10 @@ public class Compilation implements SourceLocator
     // ClassType.JDK_1_8_VERSION
     /* #else */
     /* #ifdef JAVA7 */
-    // ClassType.JDK_1_7_VERSION
+    ClassType.JDK_1_7_VERSION
     /* #else */
     /* #ifdef JAVA6 */
-    ClassType.JDK_1_6_VERSION
+    // ClassType.JDK_1_6_VERSION
     /* #else */
     /* #ifdef JAVA5 */
     // ClassType.JDK_1_5_VERSION
@@ -316,39 +330,17 @@ public class Compilation implements SourceLocator
     = ClassType.make("gnu.kawa.reflect.StaticFieldLocation");
   static public ClassType typeSymbol
     = ClassType.make("gnu.mapping.Symbol");
-  static public final Method getSymbolValueMethod
-    = typeLanguage.getDeclaredMethod("getSymbolValue", 1);
-  static public final Method getSymbolProcedureMethod
-    = typeLanguage.getDeclaredMethod("getSymbolProcedure", 1);
-  static public final Method getLocationMethod
-    = typeLocation.addMethod("get", Type.typeArray0,
-			    Type.objectType, Access.PUBLIC);
-  static public final Method getProcedureBindingMethod
-    = typeSymbol.addMethod("getProcedure", Type.typeArray0,
-			    typeProcedure, Access.PUBLIC);
   static public final Field trueConstant
     = scmBooleanType.getDeclaredField("TRUE"); 
   static public final Field falseConstant
     = scmBooleanType.getDeclaredField("FALSE");
 
-  static final Method setNameMethod
-    = typeProcedure.getDeclaredMethod("setName", 1);
   static Method makeListMethod;
   
   public static final Type[] int1Args = { Type.intType  };
   public static final Type[] string1Arg = { javaStringType };
   public static final Type[] sym1Arg = string1Arg;
 
-  static public final Method getLocation1EnvironmentMethod
-  = typeEnvironment.getDeclaredMethod("getLocation", 1);
-  static public final Method getLocation2EnvironmentMethod;
-  static {
-    Type[] args = { typeSymbol, Type.objectType    };
-    getLocation2EnvironmentMethod
-      = typeEnvironment.addMethod("getLocation", args,
-				  typeLocation, Access.PUBLIC|Access.FINAL);
-  }
-
   static {
     Type[] makeListArgs = { objArrayType, Type.intType  };
     makeListMethod = scmListType.addMethod ("makeList",
@@ -544,7 +536,7 @@ public class Compilation implements SourceLocator
      * methods getting too big.
      */
     static boolean avoidInline(LambdaExp proc) {
-        return proc.outer instanceof ModuleExp && proc.nameDecl != null;
+        return proc.getOuter() instanceof ModuleExp && proc.nameDecl != null;
     }
 
   public boolean isApplyFunction (Expression exp)
@@ -564,18 +556,16 @@ public class Compilation implements SourceLocator
   {
     if (target instanceof IgnoreTarget)
       return;
-    if (value instanceof Values)
+    if (value instanceof Values && target instanceof ConsumerTarget)
       {
-	Object[] values = ((Values) value).getValues();
+        Object[] values = ((Values) value).getValues();
         int len = values.length;
-        if (target instanceof ConsumerTarget)
+        for (int i = 0;  i < len;  i++)
           {
-            for (int i = 0;  i < len;  i++)
-              {
-                compileConstant(values[i], target);
-              } 
-            return;
-          }
+            compileConstant(values[i],
+                            ((ConsumerTarget) target).getSingleTarget());
+          } 
+        return;
       }
     if (target instanceof ConditionalTarget)
       {
@@ -602,6 +592,7 @@ public class Compilation implements SourceLocator
 		    Number num = (Number) value;
 		    switch (sig1)
 		      {
+                      case 'C':
 		      case 'I':
 			code.emitPushInt(num.intValue());
 			return;
@@ -622,6 +613,28 @@ public class Compilation implements SourceLocator
 			return;
 		      }
 		  }
+                // FIXME we should move this into a new method
+                // PrimType#pushValue(Object value), with LangPrimType override.
+                if (type == LangPrimType.characterType
+                    || type == LangPrimType.characterOrEofType)
+                  {
+                    if (value instanceof Char)
+                      {
+                        code.emitPushInt(((Char) value).intValue());
+                        return;
+                      }
+                    if (value instanceof Character)
+                      {
+                        code.emitPushInt(((Character) value).charValue());
+                        return;
+                      }
+                    if (value == gnu.lists.Sequence.eofValue
+                        && type == LangPrimType.characterOrEofType)
+                      {
+                        code.emitPushInt(-1);
+                        return;
+                     }
+                  }
 		if (sig1 == 'C')
 		  {
 		    code.emitPushInt((int) ((PrimType) type).charValue(value));
@@ -692,12 +705,15 @@ public class Compilation implements SourceLocator
     code.emitFi();
   }
 
-  private void dumpInitializers (Initializer inits)
-  {
-    for (Initializer init = Initializer.reverse(inits);
-         init != null;  init = init.next)
-      init.emit(this);
-  }
+    boolean dumpingInitializers;
+
+    private void dumpInitializers(Initializer inits) {
+        dumpingInitializers = true;
+        for (Initializer init = Initializer.reverse(inits);
+             init != null;  init = init.next)
+            init.emit(this);
+        dumpingInitializers = false;
+    }
 
   /** Search this Compilation for a ClassType with a given name.
    * @param name the name of the class desired
@@ -972,7 +988,7 @@ public class Compilation implements SourceLocator
 	  new File(parent).mkdirs();
 	clas.writeToFile(out_name);
       }
-    minfo.cleanupAfterCompilation();
+    getMinfo().cleanupAfterCompilation();
   }
 
   public void cleanupAfterCompilation ()
@@ -980,7 +996,9 @@ public class Compilation implements SourceLocator
     for (int iClass = 0;  iClass < numClasses;  iClass++)
       classes[iClass].cleanupAfterCompilation();
     classes = null;
-    minfo.comp = null;
+    ModuleInfo minfo = getMinfo();
+    minfo.className = mainClass.getName(); // In case it hasn't been set yet.
+    minfo.setCompilation(null);
     // We don't clear minfo.exp itself, since it might be re-required.
     if (minfo.exp != null)
       minfo.exp.body = null;
@@ -1063,11 +1081,12 @@ public class Compilation implements SourceLocator
 
   public void addClass (ClassType new_class)
   {
-    if (mainLambda.filename != null)
+    String fname = getModule().filename;
+    if (fname != null)
       {
 	if (emitSourceDebugExtAttr)
 	  new_class.setStratum(getLanguage().getName());
-	new_class.setSourceFile(mainLambda.filename);
+	new_class.setSourceFile(fname);
       }
     registerClass(new_class);
     new_class.setClassfileVersion(defaultClassFileVersion);
@@ -1158,7 +1177,7 @@ public class Compilation implements SourceLocator
     if (curClass == mainClass
         // Optimization: No point in calling ModuleInfo.register if we aren't
         // compiling a named module.
-        && minfo != null && minfo.sourcePath != null
+        && getMinfo() != null && getMinfo().sourcePath != null
         && ! getModule().getFlag(ModuleExp.USE_DEFINED_CLASS))
       {
 	code.emitPushThis();
@@ -1173,7 +1192,7 @@ public class Compilation implements SourceLocator
 	LambdaExp save = curLambda;
 	curLambda = new LambdaExp();
 	curLambda.closureEnv = code.getArg(0);
-	curLambda.outer = save;
+	curLambda.setOuter(save);
         Initializer init;
 	while ((init = lexp.initChain) != null)
 	  {
@@ -1238,7 +1257,7 @@ public class Compilation implements SourceLocator
                 clas = ((ClassType)
                         Type.make(Class.forName(clas.getName() + "$class")));
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
                 return;
               }
@@ -1907,7 +1926,8 @@ public class Compilation implements SourceLocator
             setState(BODY_PARSED-1);
             language.parse(this, 0);
             mexp.classFor(this);
-            lexer.close();
+            if (lexer != null)
+                lexer.close();
             lexer = null;
             setState(messages.seenErrors() ? ERROR_SEEN : BODY_PARSED);
             if (pendingImports != null)
@@ -1927,9 +1947,9 @@ public class Compilation implements SourceLocator
 
         // Avoid writing class needlessly.
         if (! explicit && ! immediate
-            && minfo.checkCurrent(ModuleManager.getInstance(), System.currentTimeMillis()))
+            && getMinfo().checkCurrent(ModuleManager.getInstance(), System.currentTimeMillis()))
           {
-            minfo.cleanupAfterCompilation();
+            getMinfo().cleanupAfterCompilation();
             setState(CLASS_WRITTEN);
           }
 
@@ -1974,8 +1994,10 @@ public class Compilation implements SourceLocator
             setState(messages.seenErrors() ? ERROR_SEEN : COMPILED);
           }
         if (wantedState >= CLASS_WRITTEN && getState() < CLASS_WRITTEN)
-          { 
-            outputClass(ModuleManager.getInstance().getCompilationDirectory());
+          {
+            if (! (mexp.getFlag(ModuleExp.HAS_SUB_MODULE)
+                   && mexp.body == QuoteExp.voidExp && mexp.firstDecl() == null))
+                outputClass(ModuleManager.getInstance().getCompilationDirectory());
             setState(CLASS_WRITTEN);
           }
       }
@@ -2232,7 +2254,7 @@ public class Compilation implements SourceLocator
             else
               litTable.emit();
 	  }
-	catch (Throwable ex)
+	catch (Exception ex)
 	  {
 	    error('e', "Literals: Internal error:" + ex);
 	  }
@@ -2275,7 +2297,7 @@ public class Compilation implements SourceLocator
       }
 
     String uri;
-    if (minfo != null && (uri = minfo.getNamespaceUri()) != null)
+    if (getMinfo() != null && (uri = getMinfo().getNamespaceUri()) != null)
       {
         // Need to generate a ModuleSet for this class, so XQuery can find
         // this module and other modules in the same namespace.
@@ -2297,7 +2319,7 @@ public class Compilation implements SourceLocator
               {
                 // Do nothing.
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
                 error('e', "error loading map for "+mainPackage+" - "+ex);
               }
@@ -2351,7 +2373,7 @@ public class Compilation implements SourceLocator
                       Path.toURL(mi.getSourceAbsPathname()).toString();
                   moduleSource = Path.relativize(sourcePath, path);
                 }
-              catch (Throwable ex)
+              catch (Exception ex)
                 {
                   throw new WrappedException("exception while fixing up '"
                                              +moduleSource+'\'',
@@ -2463,13 +2485,13 @@ public class Compilation implements SourceLocator
     ScopeExp sc = scope;
     while (scope_nesting > current_nesting)
       {
-	sc = sc.outer;
+	sc = sc.getOuter();
 	scope_nesting--;
       }
     while (sc != current_scope)
       {
 	pop(current_scope);
-	sc = sc.outer;
+        sc = sc.getOuter();
       }
     pushChain(scope, sc);
   }
@@ -2490,7 +2512,7 @@ public class Compilation implements SourceLocator
   {
     if (scope != limit)
       {
-	pushChain(scope.outer, limit);
+        pushChain(scope.getOuter(), limit);
         pushScope(scope);
         lexical.push(scope);
       }
@@ -2509,12 +2531,12 @@ public class Compilation implements SourceLocator
       module.setFile(filename);
     if (generatingApplet() || generatingServlet())
       module.setFlag(ModuleExp.SUPERTYPE_SPECIFIED);
+    mainLambda = module;
     if (immediate)
       {
         module.setFlag(ModuleExp.IMMEDIATE);
         new ModuleInfo().setCompilation(this);
       }
-    mainLambda = module;
     push(module);
     return module;
   }
@@ -2535,14 +2557,14 @@ public class Compilation implements SourceLocator
                 && scope instanceof LambdaExp
                 && ! (scope instanceof ModuleExp))))
       mustCompileHere();
-    scope.outer = current_scope;
+    scope.setOuter(current_scope);
     current_scope = scope;
   }
 
   public void pop (ScopeExp scope)
   {
     lexical.pop(scope);
-    current_scope = scope.outer;
+    current_scope = scope.getOuter();
   }
 
   public final void pop ()
@@ -2566,15 +2588,22 @@ public class Compilation implements SourceLocator
    * It is used to ensure that we can inherit from classes defined when in
    * immediate mode (in Scheme using define-class or similar).
    */
-  public void usedClass (Type type)
-  {
-    while (type instanceof ArrayType)
-      type = ((ArrayType) type).getComponentType();
-    if (immediate && type instanceof ClassType)
-      loader.addClass((ClassType) type);
-  }
+    public void usedClass(Type type) {
+        while (type instanceof ArrayType)
+            type = ((ArrayType) type).getComponentType();
+        if (immediate && type instanceof ClassType) {
+            ClassType cl = (ClassType) type;
+            for (;;) {
+                loader.addClass(cl);
+                ClassType enc = cl.getDeclaringClass();
+                if (enc == null)
+                    break;
+                cl = enc;
+            }
+        }
+    }
 
-    /** Set moudle name - which sets name of generated class. */
+    /** Set module name - which sets name of generated class. */
     public void setModuleName(String name) {
         getModule().setName(name);
     }
@@ -2647,7 +2676,7 @@ public class Compilation implements SourceLocator
    * @param message an error message to print out
    * @return an ErrorExp
    */
-  public Expression syntaxError (String message)
+  public ErrorExp syntaxError (String message)
   {
     error('e', message);
     return new ErrorExp (message);
@@ -2727,91 +2756,101 @@ public class Compilation implements SourceLocator
     return let;
   }
 
-  private void checkLoop()
-  {
-    if (((LambdaExp) current_scope).getName() != "%do%loop")
-      throw new Error("internal error - bad loop state");
-  }
+    private void checkLoop() {
+        if (((LambdaExp) current_scope).getName() != "%do%loop")
+            throw new Error("internal error - bad loop state");
+    }
 
-  /** Start a new loop.
-   * (We could make this implied by the first loopVaribale call ???) */
-  public void loopStart()
-  {
-    if (exprStack == null)
-      exprStack = new Stack<Expression>();
-    LambdaExp loopLambda = new LambdaExp();
-    LetExp let = new LetExp();
-    String fname = "%do%loop";
-    Declaration fdecl = let.addDeclaration(fname);
-    fdecl.setInitValue(loopLambda);
-    fdecl.noteValueFromLet(let);
-    loopLambda.setName(fname);
-    let.outer = current_scope;
-    loopLambda.outer = let;
-    current_scope = loopLambda;
-  }
+    /** Start a new loop.
+     * This provides the functionality of Scheme 'named let'. 
+     */
+    public LambdaExp loopStart() {
+        if (exprStack == null)
+            exprStack = new Stack<Expression>();
+        LambdaExp loopLambda = new LambdaExp();
+        LetExp let = new LetExp();
+        String fname = "%do%loop";
+        Declaration fdecl = let.addDeclaration(fname);
+        fdecl.setInitValue(loopLambda);
+        fdecl.noteValueFromLet(let);
+        loopLambda.setName(fname);
+        let.setOuter(current_scope);
+        loopLambda.setOuter(let);
+        current_scope = loopLambda;
+        return loopLambda;
+    }
 
-  public Declaration loopVariable(Object name, Type type, Expression init)
-  {
-    checkLoop();
-    LambdaExp loopLambda = (LambdaExp) current_scope;
-    Declaration decl = loopLambda.addDeclaration(name, type);
-    exprStack.push(init);
-    loopLambda.min_args++;
-    return decl;
-  }
+    /** Add a new loop variable, with initializer. */
+    public Declaration loopVariable(Object name, Type type, Expression init) {
+        checkLoop();
+        LambdaExp loopLambda = (LambdaExp) current_scope;
+        Declaration decl = loopLambda.addDeclaration(name, type);
+        exprStack.push(init);
+        loopLambda.min_args++;
+        return decl;
+    }
 
-  /** Done handling loop variables, and pushes them into the lexical scope.
-   * Ready to parse the loop condition. */ 
-  public void loopEnter ()
-  {
-    checkLoop();
-    LambdaExp loopLambda = (LambdaExp) current_scope;
-    int ninits = loopLambda.min_args;
-    loopLambda.max_args = ninits;
-    Expression[] inits = new Expression[ninits];
-    for (int i = ninits;  --i >= 0; )
-      inits[i] = (Expression) exprStack.pop();
-    LetExp let = (LetExp) loopLambda.outer;
-    Declaration fdecl = let.firstDecl();  // The decls for loopLambda.
-    let.setBody(new ApplyExp(new ReferenceExp(fdecl), inits));
-    lexical.push(loopLambda);
-  }
-  public void loopCond(Expression cond)
-  {
-    checkLoop();
-    exprStack.push(cond);
-  }
-  public void loopBody(Expression body)
-  {
-    LambdaExp loopLambda = (LambdaExp) current_scope;
-    loopLambda.body = body;
-  }
-  public Expression loopRepeat(Expression[] exps)
-  {
-    LambdaExp loopLambda = (LambdaExp) current_scope;
-    ScopeExp let = loopLambda.outer;
-    Declaration fdecl = let.firstDecl();  // The decls for loopLambda.
-    Expression cond = (Expression) exprStack.pop();
-    Expression recurse = new ApplyExp(new ReferenceExp(fdecl), exps);
-    loopLambda.body = new IfExp(cond,
-				new BeginExp(loopLambda.body, recurse),
-				QuoteExp.voidExp);
-    lexical.pop(loopLambda);
-    current_scope = let.outer;
-    return let;
-  }
+    /** Done handling loop variables, and pushes them into the lexical scope.
+     * Ready to parse the loop condition.
+     */ 
+    public void loopEnter() {
+        checkLoop();
+        LambdaExp loopLambda = (LambdaExp) current_scope;
+        int ninits = loopLambda.min_args;
+        loopLambda.max_args = ninits;
+        Expression[] inits = new Expression[ninits];
+        for (int i = ninits;  --i >= 0; )
+            inits[i] = (Expression) exprStack.pop();
+        LetExp let = (LetExp) loopLambda.getOuter();
+        Declaration fdecl = let.firstDecl();  // The decls for loopLambda.
+        let.setBody(new ApplyExp(new ReferenceExp(fdecl), inits));
+        lexical.push(loopLambda);
+    }
 
-  public Expression loopRepeat ()
-  {
-    return loopRepeat(Expression.noExpressions);
-  }
+    @Deprecated
+    public void loopCond(Expression cond) {
+        checkLoop();
+        exprStack.push(cond);
+    }
 
-  public Expression loopRepeat (Expression exp)
-  {
-    Expression[] args = { exp };
-    return loopRepeat(args);
-  }
+    @Deprecated
+    public void loopBody(Expression body) {
+        LambdaExp loopLambda = (LambdaExp) current_scope;
+        loopLambda.body = body;
+    }
+
+    /** Recurse to next iteration of specified loop. */
+    public Expression loopRepeat(LambdaExp loop, Expression... exps) {
+        ScopeExp let = loop.getOuter();
+        Declaration fdecl = let.firstDecl();  // The decls for loopLambda.
+        return new ApplyExp(new ReferenceExp(fdecl), exps);
+    }
+
+    /** Finish building a loop and return resulting expression. */
+    public Expression loopDone(Expression body) {
+        LambdaExp loopLambda = (LambdaExp) current_scope;
+        ScopeExp let = loopLambda.getOuter();
+        loopLambda.body = body;
+        lexical.pop(loopLambda);
+        current_scope = let.getOuter();
+        return let;
+    }
+
+    /** Combine loopRepeat and loopDone.
+     * Assume loopCond and loopBody have been called.
+     */
+    public Expression loopRepeatDone(Expression... exps) {
+        LambdaExp loopLambda = (LambdaExp) current_scope;
+        ScopeExp let = loopLambda.getOuter();
+        Expression cond = (Expression) exprStack.pop();
+        Expression recurse = loopRepeat(loopLambda, exps);
+        loopLambda.body = new IfExp(cond,
+                                    new BeginExp(loopLambda.body, recurse),
+                                    QuoteExp.voidExp);
+        lexical.pop(loopLambda);
+        current_scope = let.getOuter();
+        return let;
+    }
 
   public QuoteExp makeQuoteExp (Object value)
   {
@@ -2829,20 +2868,19 @@ public class Compilation implements SourceLocator
     Expression[] exps = new Expression[2];
     exps[0] = type;
     exps[1] = value;
-    QuoteExp c = new QuoteExp(Convert.getInstance());
+    QuoteExp c = new QuoteExp(Convert.cast);
     return new ApplyExp(c, exps);
   }
 
-  /**
-   * Convenience method to make an Expression that coerces a value.
-   * @param value to be coerced
-   * @param type to coerce value to
-   * @return expression that coerces value to type
-   */
-  public static Expression makeCoercion(Expression value, Type type)
-  {
-    return makeCoercion(value, new QuoteExp(type));
-  }
+    /**
+     * Convenience method to make an Expression that coerces a value.
+     * @param value to be coerced
+     * @param type to coerce value to
+     * @return expression that coerces value to type
+     */
+    public static ApplyExp makeCoercion(Expression value, Type type) {
+        return makeCoercion(value, new QuoteExp(type));
+    }
 
   /** If non-null, a helper method generated by getForNameHelper. */
   Method forNameHelper;
@@ -2906,9 +2944,11 @@ public class Compilation implements SourceLocator
     return forNameHelper;
   }
 
+    public Environment getGlobalEnvironment() { return Environment.getCurrent(); }
+
   public Object resolve(Object name, boolean function)
   {
-    Environment env = Environment.getCurrent();
+    Environment env = getGlobalEnvironment();
     Symbol symbol;
     if (name instanceof String)
       symbol = env.defaultNamespace().lookup((String) name);
@@ -2963,7 +3003,7 @@ public class Compilation implements SourceLocator
       }
     catch (Throwable ex)
       {
-        throw new WrappedException("internal error", ex);
+        WrappedException.rethrow(ex);
       }
   }
 
@@ -3038,4 +3078,8 @@ public class Compilation implements SourceLocator
   {
     return "<compilation "+mainLambda+">";
   }
+
+    public ModuleInfo getMinfo() {
+        return mainLambda.info;
+    }
 }
