@@ -2,7 +2,13 @@ package gnu.expr;
 import javax.script.*;
 import java.io.*;
 import gnu.mapping.*;
-import gnu.text.*;
+import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.text.SourceError;
+import gnu.text.SourceMessages;
+import gnu.text.SyntaxException;
 
 public class KawaScriptEngine extends AbstractScriptEngine
   implements Compilable //, Invocable
@@ -105,6 +111,8 @@ public class KawaScriptEngine extends AbstractScriptEngine
         java.net.URL url;
         if (filename != null)
           url = Path.toURL(filename);
+        else if (port instanceof CharArrayInPort)
+          url = null;
         else
           {
             Path portpath = port.getPath();
