@@ -8,11 +8,16 @@ import gnu.mapping.Location;
 import gnu.lists.*;
 import gnu.text.Lexer;
 import gnu.text.SourceMessages;
+import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.OutPort;
 import gnu.kawa.reflect.*;
 import java.io.*;
+import gnu.kawa.functions.GetNamedPart;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import gnu.kawa.lispexpr.ClassNamespace; // FIXME
+import kawa.SourceType;
 
 /**
  * Contains various language-dependent methods.
@@ -354,10 +359,14 @@ public abstract class Language
    *   final field whose type extends gnu.mapping.Procedure.
    */
   protected void defProcStFld(String name, String cname, String fname)
+  {
+    defProcStFld(getSymbol(name), cname, fname);
+  }
+
+  protected void defProcStFld(Symbol sym, String cname, String fname)
   {
     Object property = (hasSeparateFunctionNamespace() ? EnvironmentKey.FUNCTION
 		       : null);
-    Symbol sym = getSymbol(name);
     StaticFieldLocation loc
       = StaticFieldLocation.define(environ, sym, property, cname, fname);
     loc.setProcedure();
@@ -372,6 +381,17 @@ public abstract class Language
   {
     defProcStFld(name, cname, mangleNameIfNeeded(name));
   }
+  
+  /**
+   * Declare by name in the current environment a procedure bound to a static field.
+   * 
+   * @param asName The procedure's source-level name.
+   * @param cname The name of the class containing the field.
+   * @param mname The name of the (mangled) field in {@code cname}
+   */
+  protected void defProcStFldAs(String asName, String cname, String mname) {
+    defProcStFld(asName, cname, mangleNameIfNeeded(mname));
+  }
 
   /** Enter a named function into the current environment. */
   public final void defineFunction(Named proc)
@@ -637,6 +657,7 @@ public abstract class Language
   /** Compilation explicitly requested, not just because of an import. */
   public static final int PARSE_EXPLICIT = 64;
   public static final int PARSE_INTERACTIVE_MODULE = 128;
+  public static final int PARSE_EMIT_MAIN = 256;
 
   public static boolean requirePedantic;
 
@@ -682,11 +703,13 @@ public abstract class Language
     boolean immediate = (options & PARSE_IMMEDIATE) != 0;
     Compilation tr = getCompilation(messages, lexical);
     if (requirePedantic)
-      tr.pedantic = true;
+      tr.setPedantic(true);
     if (! immediate)
       tr.mustCompile = true;
     tr.immediate = immediate;
     tr.langOptions = options;
+    if ((options & PARSE_EMIT_MAIN) != 0)
+      tr.currentOptions.set("main", null);
     if ((options & PARSE_EXPLICIT) != 0)
       tr.explicit = true;
     if ((options & PARSE_PROLOG) != 0)
@@ -790,6 +813,71 @@ public abstract class Language
     return null;
   }
 
+    /** Encode this type as a parseable string.
+     * Stored in SourceType or SourceMethodType annotations.
+     * @return A string suitable for {@link #decodeType},
+     * or null if the type is "uninteresting" in the sense we can
+     * infer it from the Java typing.
+     */
+    public String encodeType(Type type) {
+        if (type instanceof TypeValue)
+            return ((TypeValue) type).encodeType(this);
+        if (type instanceof ArrayType) {
+            String el = encodeType(((ArrayType) type).getComponentType());
+            if (el != null)
+                return el+"[]";
+        }
+        return null;
+    }
+
+    public Type decodeType(Type javaType, String annotType,
+                           ParameterizedType parameterizedType) {
+        if (annotType != null && annotType.length() > 0)
+            return getTypeFor(annotType);
+        return getLangTypeFor
+            (resolveTypeVariables(javaType, parameterizedType));
+    }
+
+    static Type resolveTypeVariables(Type langType, ParameterizedType parameterizedType) {
+        if (langType instanceof TypeVariable)
+            return resolveTypeVariable((TypeVariable) langType, parameterizedType);
+        if (langType instanceof ParameterizedType) {
+            ParameterizedType ptype = (ParameterizedType) langType;
+            Type[] paramTypes = ptype.getTypeArgumentTypes();
+            int nparams = paramTypes.length;
+            Type[] resolvedTypes = new Type[nparams];
+            boolean changed = false;
+            for (int i = 0; i < nparams;  i++) {
+                Type t0 = paramTypes[i];
+                char bound = ptype.getTypeArgumentBound(i);
+                // FIXME Don't support wildcards here yet.
+                if (bound != '\0')
+                    return langType.getRawType();
+                Type t1 = resolveTypeVariables(t0, parameterizedType);
+                resolvedTypes[i] = t1;
+                if (t0 != t1)
+                    changed = true;
+            }
+            if (changed) {
+                return new ParameterizedType(ptype.getRawType(), resolvedTypes);
+            }
+        }
+        return langType;
+    }
+
+    static Type resolveTypeVariable(TypeVariable tvar, ParameterizedType parameterizedType) {
+	if (parameterizedType != null) {
+	    TypeVariable[] tparams = parameterizedType.getRawType().getTypeParameters();
+	    int nparams = tparams.length;
+	    for (int i = 0;  i < nparams;  i++) {
+		if (tvar.getName().equals(tparams[i].getName())) {
+		    return parameterizedType.getTypeArgumentType(i);
+		}
+	    }
+	}
+	return tvar.getRawType();
+    }
+
   /** "Coerce" a language-specific "type specifier" object to a Type. */
   public final Type asType(Object spec)
   {
@@ -852,6 +940,19 @@ public abstract class Language
 		    name = ((Named) loc).getName();
 		  }
 	      }
+            else if (exp instanceof ApplyExp) {
+                ApplyExp aexp = (ApplyExp) exp;
+                Expression afunc = aexp.getFunction();
+                Object func = afunc.valueIfConstant();
+                Expression[] args = aexp.getArgs();
+                int start = 0;
+                if (decl.isAlias()
+                    && func == kawa.standard.location.getMakeProcLocProc()) {
+                    afunc = args[0];
+                    start = 1;
+                }
+                return getTypeFor(afunc, args, start);
+            }
 	    else if (! decl.getFlag(Declaration.IS_UNKNOWN))
 	      return getTypeFor(exp, lenient);
 	  }
@@ -865,6 +966,10 @@ public abstract class Language
             && name.charAt(len-1) == '>')
           return getTypeFor(name.substring(1, len-1));
       }
+    else if (exp instanceof ApplyExp) {
+        ApplyExp aexp = (ApplyExp) exp;
+        return getTypeFor(aexp.getFunction(), aexp.getArgs(), 0);
+    }
     else if ((exp instanceof ClassExp && ! (exp instanceof ObjectExp))
              || exp instanceof ModuleExp)
       {
@@ -873,6 +978,37 @@ public abstract class Language
     return null;
   }
 
+    private Type getTypeFor(Expression afunc, Expression[] args, int start) {
+        Object func = afunc.valueIfConstant();
+        try {
+            if (func == SlotGet.staticField
+                && args.length == 2+start) {
+                Object arg1 = args[start+0].valueIfConstant();
+                Object arg2 = args[start+1].valueIfConstant();
+                if (arg1 != null && arg2 != null) {
+                    Object fld = SlotGet.staticField.apply2(arg1, arg2);
+                    if (fld instanceof Type)
+                        return (Type) fld;
+                }
+            }
+            if (func == GetNamedPart.getNamedPart
+                && args.length == 2+start) {
+                Object arg1 = args[start+0].valueIfConstant();
+                Object arg2 = args[start+1].valueIfConstant();
+                if (arg2 instanceof Symbol) {
+                    Object comb = GetNamedPart.getNamedPart(arg1, (Symbol) arg2);
+                    if (comb instanceof Class)
+                        return Type.make((Class) comb);
+                    if (comb instanceof Type)
+                        return (Type) comb;
+                }
+            }
+        } catch (Exception ex) {
+            ;
+        }
+        return null;
+    }
+
   public static/* for now */ Type unionType (Type t1, Type t2)
   {
     if (t1 == Type.neverReturnsType)
@@ -894,83 +1030,82 @@ public abstract class Language
     return Type.objectType;
   }
 
-  public Declaration declFromField (ModuleExp mod, Object fvalue, Field fld)
-  {
-    String fname = fld.getName();
-    Type ftype = fld.getType();
-    boolean isAlias = ftype.isSubtype(Compilation.typeLocation);
-    Object fdname;
-    // FIXME if fvalue is FieldLocation, and field is final,
-    // get name from value of field.
-    boolean isImportedInstance;
-    boolean externalAccess = false;
-    boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
-    if ((isImportedInstance = fname.endsWith("$instance")))
-      fdname = fname;
-    else if (isFinal && ftype == Compilation.typeModuleMethod
-             && fvalue instanceof Named /* should always be true */)
-      fdname = ((Named) fvalue).getSymbol();
-    else
-      {
-	// FIXME move this to demangleName
-	if (fname.startsWith(Declaration.PRIVATE_PREFIX))
-          {
-            externalAccess = true;
-            fname = fname.substring(Declaration.PRIVATE_PREFIX.length());
-          }
-	fdname = Compilation.demangleName(fname, true).intern();
-      }
-    try
-      {
-        SourceName sourceName = fld.getReflectField().getAnnotation(SourceName.class);
-        if (sourceName != null)
-          {
-            fdname = Symbol.valueOf(sourceName.name(), sourceName.uri(), sourceName.prefix());
-          }
-      }
-    catch (Throwable ex)
-      {
-      }
-    if (fdname instanceof String)
-      {
-        String uri = mod.getNamespaceUri();
-        String sname = (String) fdname;
-        Symbol sym;
-        if (uri == null)
-          fdname = SimpleSymbol.valueOf(sname);
-        else
-          fdname = Symbol.make(uri, sname);
-      }
-    Type dtype = isAlias ? Type.objectType
-	: getLangTypeFor(ftype);
-    Declaration fdecl = mod.addDeclaration(fdname, dtype);
-    boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
-    if (isAlias)
-      {
-        fdecl.setIndirectBinding(true);
-        if (ftype instanceof ClassType
-            && ((ClassType) ftype).isSubclass("gnu.mapping.ThreadLocation"))
-          fdecl.setFlag(Declaration.IS_DYNAMIC);
-      }
-    else if (isFinal && ftype instanceof ClassType)
-      {
-        if (ftype.isSubtype(Compilation.typeProcedure))
-          fdecl.setProcedureDecl(true);
-        else if (((ClassType) ftype).isSubclass("gnu.mapping.Namespace"))
-          fdecl.setFlag(Declaration.IS_NAMESPACE_PREFIX);
-      }
-    if (isStatic)
-      fdecl.setFlag(Declaration.STATIC_SPECIFIED);
-    fdecl.field = fld; 
-    if (isFinal && ! isAlias) // FIXME? ok for location?
-      fdecl.setFlag(Declaration.IS_CONSTANT);
-    if (isImportedInstance)
-      fdecl.setFlag(Declaration.MODULE_REFERENCE);
-    fdecl.setSimple(false);
-    if (externalAccess)
-      fdecl.setFlag(Declaration.EXTERNAL_ACCESS|Declaration.PRIVATE);
-    return fdecl;
-  }
+    public Declaration declFromField(ModuleExp mod, Object fvalue, Field fld) {
+        String fname = fld.getName();
+        Type ftype = fld.getType();
+        boolean isAlias = ftype.isSubtype(Compilation.typeLocation);
+        Object fdname;
+        // FIXME if fvalue is FieldLocation, and field is final,
+        // get name from value of field.
+        boolean isImportedInstance;
+        boolean externalAccess = fname.startsWith(Declaration.PRIVATE_PREFIX);
+        boolean isFinal = (fld.getModifiers() & Access.FINAL) != 0;
+        if ((isImportedInstance = fname.endsWith("$instance")))
+            fdname = fname;
+        else if (isFinal && ftype == Compilation.typeModuleMethod
+                 && fvalue instanceof Named /* should always be true */)
+            fdname = ((Named) fvalue).getSymbol();
+        else {
+            // FIXME move this to demangleName
+            if (externalAccess)
+                fname = fname.substring(Declaration.PRIVATE_PREFIX.length());
+            fdname = Compilation.demangleName(fname, true).intern();
+        }
+        try {
+            SourceName sourceName = fld.getAnnotation(SourceName.class);
+            if (sourceName != null) {
+                fdname = Symbol.valueOf(sourceName.name(), sourceName.uri(), sourceName.prefix());
+            }
+        } catch (Exception ex) {
+        }
+        if (fdname instanceof String)  {
+            String uri = mod.getNamespaceUri();
+            String sname = (String) fdname;
+            Symbol sym;
+            if (uri == null)
+                fdname = SimpleSymbol.valueOf(sname);
+            else
+                fdname = Symbol.make(uri, sname);
+        }
+        Type dtype;
+        if (isAlias)
+            dtype = Type.objectType;
+        else {
+            String annotType = null;
+            try {
+                SourceType sourceType = fld.getAnnotation(SourceType.class);
+                if (sourceType != null)
+                    annotType = sourceType.value();
+            } catch (Throwable ex) {
+            }
+            dtype = decodeType(ftype, annotType, null);
+        }
+        Declaration fdecl = mod.addDeclaration(fdname, dtype);
+        boolean isStatic = (fld.getModifiers() & Access.STATIC) != 0;
+        if (isAlias) {
+            fdecl.setIndirectBinding(true);
+            if (ftype instanceof ClassType
+                && ((ClassType) ftype).isSubclass("gnu.mapping.DynamicLocation"))
+                fdecl.setFlag(Declaration.IS_DYNAMIC);
+        }
+        else if (isFinal && ftype instanceof ClassType) {
+            if (ftype.isSubtype(Compilation.typeProcedure))
+                fdecl.setProcedureDecl(true);
+            else if (((ClassType) ftype).isSubclass("gnu.mapping.Namespace"))
+                fdecl.setFlag(Declaration.IS_NAMESPACE_PREFIX);
+        }
+        if (isStatic)
+            fdecl.setFlag(Declaration.STATIC_SPECIFIED);
+        fdecl.field = fld; 
+        if (isFinal && ! isAlias) // FIXME? ok for location?
+            fdecl.setFlag(Declaration.IS_CONSTANT);
+        if (isImportedInstance)
+            fdecl.setFlag(Declaration.MODULE_REFERENCE);
+        fdecl.setSimple(false);
+        if (externalAccess)
+            fdecl.setFlag(Declaration.EXTERNAL_ACCESS|Declaration.PRIVATE);
+        return fdecl;
+    }
 
   public static final int VALUE_NAMESPACE = 1<<0;
   public static final int FUNCTION_NAMESPACE = 1<<1;
