@@ -2,10 +2,13 @@ package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.*;
 import gnu.mapping.Location; // As opposed to gnu.bytecode.Location
-import gnu.text.*;
-import java.io.*;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
 import gnu.kawa.reflect.StaticFieldLocation;
 import gnu.kawa.reflect.FieldLocation;
+import gnu.text.SourceMessages;
+import gnu.text.SyntaxException;
+import java.io.*;
 import java.net.URL;
 
 /**
@@ -27,8 +30,8 @@ public class ModuleExp extends LambdaExp
   public static final int IMMEDIATE = LAZY_DECLARATIONS << 1;
 
   /** True of a read-eval-print interface where each module is only partial.
-   * Conceptually, each statement is a fragent of a imagined super-module
-   * for the whole interaction, thugh current there is no super-module object.
+   * Conceptually, each statement is a fragment of a imagined super-module
+   * for the whole interaction, though currently there is no super-module object.
    * IMMEDIATE is also set in this case.
    */
   public static final int INTERACTIVE = IMMEDIATE << 1;
@@ -36,10 +39,16 @@ public class ModuleExp extends LambdaExp
   /** Using explicit class (e.g. define-simple-class) for module class. */
   public static final int USE_DEFINED_CLASS = INTERACTIVE << 1;
 
+  public static final int HAS_SUB_MODULE = USE_DEFINED_CLASS << 1;
+
   public ModuleExp ()
   {
   }
 
+    public static ModuleExp valueOf(ClassType type) {
+        return ModuleInfo.find(type).getModuleExp();
+    }
+
   /** Used to control which .zip file dumps are generated. */
   public static String dumpZipPrefix;
 
@@ -52,11 +61,12 @@ public class ModuleExp extends LambdaExp
     throws SyntaxException
   {
     ModuleExp mexp = comp.getModule();
+    ModuleInfo minfo = mexp.info;
     SourceMessages messages = comp.getMessages();
     try
       {
 
-        comp.minfo.loadByStages(Compilation.COMPILED);
+        minfo.loadByStages(Compilation.COMPILED);
 
 	if (messages.seenErrors())
 	  return null;
@@ -135,7 +145,6 @@ public class ModuleExp extends LambdaExp
               context.addClass(cclass);
           }
 
-        ModuleInfo minfo = comp.minfo;
         minfo.setModuleClass(clas);
         comp.cleanupAfterCompilation();
         int ndeps = minfo.numDependencies;
@@ -145,7 +154,7 @@ public class ModuleExp extends LambdaExp
             ModuleInfo dep = minfo.dependencies[idep];
             Class dclass = dep.getModuleClassRaw();
             if (dclass == null)
-              dclass = evalToClass(dep.comp, null);
+              dclass = evalToClass(dep.getCompilation(), null);
             comp.loader.addClass(dclass);
           }
 
@@ -159,7 +168,7 @@ public class ModuleExp extends LambdaExp
       {
 	throw new WrappedException("class not found in lambda eval", ex);
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
         messages.error('f', "internal compile error - caught "+ex, ex);
         throw new SyntaxException(messages);
@@ -184,7 +193,7 @@ public class ModuleExp extends LambdaExp
                     // while using kawart - i.e. the Kawa "runtime" jar.
                     Class.forName("gnu.expr.TryExp");
                     haveCompiler = 1;
-                } catch (Throwable ex) {
+                } catch (Exception ex) {
                     haveCompiler = -1;
                 }
             }
@@ -221,7 +230,6 @@ public class ModuleExp extends LambdaExp
     throws SyntaxException
   {
     ModuleExp mexp = comp.getModule();
-    mexp.info = comp.minfo;
     Environment orig_env = Environment.setSaveCurrent(env);
     Compilation orig_comp = Compilation.setSaveCurrent(comp);
     SourceMessages messages = comp.getMessages();
@@ -230,7 +238,7 @@ public class ModuleExp extends LambdaExp
     try
       {
         comp.process(Compilation.RESOLVED);
-        comp.minfo.loadByStages(Compilation.WALKED);
+        comp.getMinfo().loadByStages(Compilation.WALKED);
 
         if (msg != null ? messages.checkErrors(msg, 20) : messages.seenErrors())
           return null;
@@ -257,7 +265,7 @@ public class ModuleExp extends LambdaExp
                     savedLoader = thread.getContextClassLoader();
                     thread.setContextClassLoader(clas.getClassLoader());
                   }
-                catch (Throwable ex)
+                catch (Exception ex)
                   {
                     thread = null;
                   }
@@ -449,7 +457,7 @@ public class ModuleExp extends LambdaExp
 	if (decl.field != null)
 	  continue;
 	Expression value = decl.getValue();
-	if (((decl.isSimple() && ! decl.isPublic()))
+        if (((decl.isSimple() && decl.isModuleLocal()))
             // Kludge - needed for macros - see Savannah bug #13601.
             && ! decl.isNamespaceDecl()
             && ! (decl.getFlag(Declaration.IS_CONSTANT)
@@ -519,7 +527,7 @@ public class ModuleExp extends LambdaExp
     return decls;
   }
 
-  /** Return the class this module.
+  /** Return the class for this module.
    * If not set yet, sets it now, based on the source file name.
    */
   public ClassType classFor (Compilation comp)
@@ -529,10 +537,10 @@ public class ModuleExp extends LambdaExp
     String mname = getName();
     String className = getFileName();
     Path path = null;
-    if (comp.getModule() == this && comp.minfo != null
-        && comp.minfo.className != null)
+    if (comp.getModule() == this && info != null
+        && info.className != null)
       // If explicitly set, perhaps using command-line flags.
-      className = comp.minfo.className;
+      className = info.className;
     else
       {
         if (mname != null)
