@@ -4,7 +4,7 @@
 package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.*;
-import gnu.text.*;
+import gnu.kawa.io.Path;
 import java.util.*;
 import gnu.kawa.util.AbstractWeakHashTable;
 
@@ -28,19 +28,21 @@ public class ModuleInfo
   String uri;
 
   ModuleExp exp;
-  Compilation comp;
+  private Compilation comp;
 
   public Compilation getCompilation () { return comp; }
 
   public void setCompilation (Compilation comp)
   {
-    comp.minfo = this;
     this.comp = comp;
+    if (comp == null)
+        return;
     ModuleExp mod = comp.mainLambda;
+    mod.info = this;
     this.exp = mod;
-    if (mod != null)
+    String fileName;
+    if (mod != null && (fileName = mod.getFileName()) != null)
       {
-        String fileName = mod.getFileName();
         this.sourcePath = fileName;
         Path abs = absPath(fileName);
         this.sourceAbsPath = abs;
@@ -142,8 +144,6 @@ public class ModuleInfo
     ModuleExp m = exp;
     if (m == null)
       {
-        if (comp != null)
-          return comp.mainLambda;
         ClassType ctype = ClassType.make(className);
         m = new ModuleExp();
         m.compiledType = ctype;
@@ -293,10 +293,14 @@ public class ModuleInfo
       return;
     comp.setState(state+1);
     int ndeps = numDependencies;
+    int depWanted = ((Compilation.writeImplicitClasses
+                      && wantedState >= Compilation.CLASS_WRITTEN)
+                     ? Compilation.COMPILED
+                     : wantedState);
     for (int idep = 0;  idep < ndeps;  idep++)
       {
         ModuleInfo dep = dependencies[idep];
-        dep.loadByStages(wantedState);
+        dep.loadByStages(depWanted);
       }
     state = getState();
     if (state >= wantedState) // Most likely? if ERROR_SEEN.
