@@ -1,8 +1,9 @@
 package gnu.expr;
 import java.net.*;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.URLPath;
 import gnu.mapping.WrappedException;
 import gnu.bytecode.ClassType;
-import gnu.text.*;
 
 /** A database of known modules as represented by {@link ModuleInfo}.
  * Currently there is only a single global instance of {@code ModuleManager};
@@ -58,7 +59,7 @@ public class ModuleManager
     }
 
   public static final long LAST_MODIFIED_CACHE_TIME = 1000;
-  /** Number millseconds before we re-check file's modified time. */
+  /** Number of milliseconds before we re-check file's modified time. */
   public long lastModifiedCacheTime = LAST_MODIFIED_CACHE_TIME;
 
   /** List of all modules managed by this ModuleManager. */
@@ -78,9 +79,7 @@ public class ModuleManager
     Path sourceAbsPath = ModuleInfo.absPath(fileName);
     ModuleInfo info = findWithSourcePath(sourceAbsPath, fileName);
     info.setClassName(ctype.getName());
-    info.exp = mexp;
-    comp.minfo = info;
-    info.comp = comp;
+    info.setCompilation(comp);
     return info;
   }
 
@@ -97,6 +96,13 @@ public class ModuleManager
     modules[numModules++] = info;
   }
 
+    public ModuleInfo createWithClassName(String className) {
+        ModuleInfo info = new ModuleInfo();
+        info.setClassName(className);
+        add(info);
+        return info;
+    }
+
   public synchronized ModuleInfo searchWithClassName (String className)
   {
     for (int i = numModules;  --i >= 0; )
@@ -128,7 +134,7 @@ public class ModuleManager
       {
         return findWithClass(ClassType.getContextClass(className));
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
         throw WrappedException.wrapIfNeeded(ex);
       }
@@ -213,7 +219,7 @@ public class ModuleManager
             info.sourceAbsPath = sourceAbsPath;
             info.sourceAbsPathname = sourceAbsPath.toString();
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             return;
           }
