@@ -1,6 +1,11 @@
 package gnu.expr;
 import gnu.bytecode.*;
 
+/** An expression that evaluated to an instance of an anonymous class.
+ * It's conceptually questionable that this inherits from ClassExp
+ * - it should perhaps inherit from ApplyExp.
+ */
+
 public class ObjectExp extends ClassExp
 {
   public ObjectExp ()
@@ -17,7 +22,6 @@ public class ObjectExp extends ClassExp
 
   public void compile (Compilation comp, Target target)
   {
-    compileMembers(comp);
     CodeAttr code = comp.getCode();
     code.emitNew(compiledType);
     code.emitDup(1);
