@@ -2,6 +2,7 @@ package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.*;
 import gnu.text.SourceLocator;
+import gnu.kawa.io.OutPort;
 import gnu.kawa.reflect.MakeAnnotation;
 
 /**
@@ -19,6 +20,7 @@ public class QuoteExp extends Expression
 
   public static final int EXPLICITLY_TYPED = Expression.NEXT_AVAIL_FLAG;
   public static final int SHARED_CONSTANT = EXPLICITLY_TYPED << 1;
+  public static final int IS_KEYWORD = EXPLICITLY_TYPED << 2;
 
   public final gnu.bytecode.Type getRawType() { return type; }
   protected final gnu.bytecode.Type calculateType()
@@ -35,11 +37,11 @@ public class QuoteExp extends Expression
       return Type.make(value.getClass());
   }
 
-  public void setType (Type type)
-  {
-    this.type = type;
-    setFlag(EXPLICITLY_TYPED);
-  }
+    @Override
+    public void setType(Type type) {
+        super.setType(type);
+        setFlag(EXPLICITLY_TYPED);
+    }
 
   public boolean isExplicitlyTyped ()
   {
@@ -51,7 +53,7 @@ public class QuoteExp extends Expression
     return getFlag(SHARED_CONSTANT);
   }
 
-  static public QuoteExp undefined_exp =  makeShared(Special.undefined);
+  static public QuoteExp undefined_exp = makeShared(Special.undefined);
   static public QuoteExp abstractExp = makeShared(Special.abstractSpecial);
   static public QuoteExp nativeExp = makeShared(Special.nativeSpecial);
   static public QuoteExp voidExp = makeShared(Values.empty, Type.voidType);
@@ -113,22 +115,24 @@ public class QuoteExp extends Expression
     ctx.writeValue(value);
   }
 
-  public void compile (Compilation comp, Target target)
-  {
-    if (type == null || type == Type.pointer_type
-        || target instanceof IgnoreTarget
-        || (type instanceof ObjectType
-            && type.isInstance(value))
-        || (type instanceof PrimType
-            && target.getType() == ((PrimType) type).boxedType()))
-      comp.compileConstant(value, target);
-    else
-      {
-        Type vtype = type.isVoid() ? Type.objectType : type;
-        comp.compileConstant(value, StackTarget.getInstance(vtype));
-        target.compileFromStack(comp, vtype);
-      }
-  }
+    public void compile (Compilation comp, Target target) {
+        Type targetType;
+        if (type == null || type == Type.pointer_type
+            || target instanceof IgnoreTarget
+            || (type instanceof ObjectType
+                && type.isInstance(value))
+            || (type instanceof PrimType
+                // In this case the value is assumed to be an instance
+                // of ((PrimType) type).boxedType().
+                && ((targetType = target.getType()) == Type.objectType
+                    || targetType == ((PrimType) type).boxedType()))) {
+            comp.compileConstant(value, target);
+        } else {
+            Type vtype = type.isVoid() ? Type.objectType : type;
+            comp.compileConstant(value, StackTarget.getInstance(vtype));
+            target.compileFromStack(comp, vtype);
+        }
+    }
  
   public Expression deepCopy (gnu.kawa.util.IdentityHashTable mapper)
   {
@@ -152,7 +156,9 @@ public class QuoteExp extends Expression
 				 + " which is a "+fval.getClass().getName()));
     Procedure proc = (Procedure) fval;
     int nargs = exp.getArgCount();
-    String msg = WrongArguments.checkArgCount(proc, nargs);
+    int spliceCount = exp.spliceCount();
+    String msg = WrongArguments.checkArgCount(proc, nargs-spliceCount,
+                                              spliceCount>0);
     if (msg != null)
       return visitor.noteError(msg);
     Expression inlined = visitor.maybeInline(exp, required, proc);
@@ -187,14 +193,9 @@ public class QuoteExp extends Expression
             comp.error('e', msg, args[0]);
           }
       }
-    if (comp.inlineOk(proc))
+    if (comp.inlineOk(proc) && exp.isSimple()
+        && ! ApplyExp.isInlineable(proc))
       {
-	if (ApplyExp.asInlineable(proc) != null)
-          {
-            if (exp.getFunction() == this)
-              return exp;
-            return new ApplyExp(this, exp.getArgs()).setLine(exp);
-          }
 	PrimProcedure mproc
 	  = PrimProcedure.getMethodFor(proc, decl, exp.args,
 				       comp.getLanguage());
