@@ -180,15 +180,39 @@ implements gnu.mapping.Named
   public void defineKey(Object keySpec, Object binding)
   {
     EKeymap keymap = this;
-    if (keySpec instanceof Sequence && ! (keySpec instanceof LList))
+    if ((keySpec instanceof Sequence
+         /* #ifdef use:java.lang.CharSequence */
+         || keySpec instanceof CharSequence
+         /* #else */
+         // || keySpec instanceof String
+         /* #endif */
+         ) && ! (keySpec instanceof LList))
       {
         // Handle key sequence.
-	Sequence value = (Sequence) keySpec;
-        boolean hackMeta = keySpec instanceof FString;
-        int len = value.size();
+	Sequence seq;
+        int len;
+        String str;
+        /* #ifdef use:java.lang.CharSequence */
+        boolean hackMeta = keySpec instanceof CharSequence;
+        /* #else */
+        // boolean hackMeta = keySpec instanceof String || keySpec instanceof CharSeq;
+        /* #endif */
+        if (keySpec instanceof Sequence)
+          {
+            seq = (Sequence) keySpec;
+            len = seq.size();
+            str = null;
+          }
+        else
+          {
+            seq = null;
+            str = keySpec.toString();
+            len = str.length();
+          }
         for (int i = 0;  i < len; )
           {
-            Object keyValue = value.get(i);
+            Object keyValue = (seq != null ? seq.get(i)
+                               : Convert.toObject(str.charAt(i)));
             boolean sawMeta = false;
             i++;
 	    int key = asKeyStroke(keyValue);
@@ -337,11 +361,11 @@ implements gnu.mapping.Named
     while (key instanceof Pair)
       {
 	Pair pair = (Pair) key;
-	if (pair.cdr == LList.Empty)
-	  key = pair.car;
+	if (pair.getCdr() == LList.Empty)
+	  key = pair.getCar();
 	else
 	  {
-	    Object car = pair.car;
+	    Object car = pair.getCar();
 	    if (car instanceof Symbol)
 	      car = ((Symbol) car).getName();
 	    if (car == "control")
@@ -352,7 +376,7 @@ implements gnu.mapping.Named
 	      m |= SHIFT_MASK;
 	    if (car == "alt")
 	      m |= ALT_MASK;
-	    key = pair.cdr;
+	    key = pair.getCdr();
 	  }
       }
     if (key instanceof Char)
@@ -426,7 +450,7 @@ implements gnu.mapping.Named
     if ((key & (PRESSED << 16)) == 0)
       { // It's a KEY_TYPED (keyChar) event.
         char ch = (char) key;
-        return (ch < ' ' && ch != metaKey) || ch >= 127;
+        return (mods & ~SHIFT_MASK) != 0 || (ch < ' ' && ch != metaKey) || ch >= 127;
       }
     else
       { // It's a KEY_PRESSED (keyCODE) event.
@@ -519,7 +543,7 @@ implements gnu.mapping.Named
   {
     EKeymap keymap = this;
     int nKeys = nPrefix + (key != 0 ? 1 : 0);
-    boolean pendingMeta = false;
+    boolean pendingMeta = false; // FIXME this is never set
     if (nKeys == 0)
      throw new Error("no keys");
     for (int i = 0;  ; )
