@@ -8,71 +8,10 @@ import gnu.kawa.lispexpr.LangObjType;
 import gnu.lists.LList;
 import kawa.standard.Scheme;
 import kawa.standard.SchemeCompilation;
+import static gnu.expr.InlineCalls.LenientExpectedType;
 
-public class CompileMisc implements Inlineable
+public class CompileMisc
 {
-  static final int CONVERT = 2;
-  static final int NOT = 3;
-  static final int EQ = 4;
-  static final int NUMBER_COMPARE = 5;
-  static final int NUMBER_PREDICATE = 6;
-  int code;
-  Procedure proc;
-
-  public CompileMisc(Procedure proc, int code)
-  {
-    this.proc = proc;
-    this.code = code;
-  }
-
-  public static CompileMisc forConvert(Object proc)
-  {
-    return new CompileMisc((Procedure) proc, CONVERT);
-  }
-
-  public static CompileMisc forNot(Object proc)
-  {
-    return new CompileMisc((Procedure) proc, NOT);
-  }
-
-  public static CompileMisc forEq(Object proc)
-  {
-    return new CompileMisc((Procedure) proc, EQ);
-  }
-
-  public static CompileMisc forNumberCompare(Object proc)
-  {
-    return new CompileMisc((Procedure) proc, NUMBER_COMPARE);
-  }
-
-  public static CompileMisc forNumberPredicate(Object proc)
-  {
-    return new CompileMisc((Procedure) proc, NUMBER_PREDICATE);
-  }
-
-  public void compile (ApplyExp exp, Compilation comp, Target target)
-  {
-    switch (code)
-      {
-      case CONVERT:
-        compileConvert((Convert) proc, exp, comp, target);
-        return;
-      case NOT:
-        compileNot((Not) proc, exp, comp, target);
-        return;
-      case EQ:
-        compileEq(exp.getArgs(), comp, target, ((IsEq) proc).language);
-        return;
-      case NUMBER_COMPARE:
-        compileNumberCompare((NumberCompare) proc, exp, comp, target);
-        return;
-      case NUMBER_PREDICATE:
-        compileNumberPredicate((NumberPredicate) proc, exp, comp, target);
-        return;
-      default: throw new Error();
-      }
-  }
-
   public static Expression validateApplyConstantFunction0
   (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
@@ -80,7 +19,7 @@ public class CompileMisc implements Inlineable
     int nargs = exp.getArgCount();
     if (nargs != 0 && visitor != null)
       {
-	String message = WrongArguments.checkArgCount(proc, nargs);
+        String message = WrongArguments.checkArgCount(proc, nargs, false);
 	return visitor.noteError(message);
       }
     return ((ConstantFunction0) proc).constant;
@@ -91,6 +30,7 @@ public class CompileMisc implements Inlineable
   {
     Compilation comp = visitor.getCompilation();
     Language language = comp.getLanguage();
+    Convert cproc = (Convert) proc;
     Expression[] args = exp.getArgs();
     if (args.length == 2)
       {
@@ -99,7 +39,11 @@ public class CompileMisc implements Inlineable
         if (type instanceof Type)
           {
             args[0] = new QuoteExp(type);
-            args[1] = visitor.visit(args[1], type);
+            Type xtype = cproc.lenient ? LenientExpectedType.make(type)
+                : type;
+            if (! args[1].getFlag(Expression.VALIDATED))
+                args[1] = ExpVisitor.visit(visitor, args[1], xtype);
+            args[1] = visitor.checkType(args[1], xtype);
             CompileReflect.checkKnownClass(type, comp);
             exp.setType(type);
             if (args[1].getType() == type)
@@ -193,11 +137,10 @@ public class CompileMisc implements Inlineable
     for (int i = 0;  i < alen;  i++)
       {
 	Expression arg = args[i];
-        Object key;
-	if (arg instanceof QuoteExp
-            && (key = ((QuoteExp) arg).getValue()) instanceof Keyword)
+        Keyword key = arg.checkLiteralKeyword();
+	if (key != null)
 	  {
-	    String keyword = ((Keyword) key).getName();
+	    String keyword = key.getName();
 	    Expression next = args[++i];
 	    if (keyword == "name")
               {
@@ -224,11 +167,10 @@ public class CompileMisc implements Inlineable
         for (int i = 0;  i < alen;  i++)
           {
             Expression arg = args[i];
-            Object key;
-            if (arg instanceof QuoteExp
-                && (key = ((QuoteExp) arg).getValue()) instanceof Keyword)
+            Keyword key = arg.checkLiteralKeyword();
+            if (key != null)
               {
-                String keyword = ((Keyword) key).getName();
+                String keyword = key.getName();
                 Expression next = args[++i];
                 if (keyword == "name")
                   lexp.setName(name);
@@ -246,8 +188,11 @@ public class CompileMisc implements Inlineable
   public static Expression validateApplyValuesMap
   (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
   {
-    exp.visitArgs(visitor);
     LambdaExp lexp = ValuesMap.canInline(exp, (ValuesMap) proc);
+    // FIXME If required is an OccurrenceType, then we want to validate
+    // lexp.body with the same OccurrenceType.  This is tricky - best to
+    // do a tree rewrite here, instead of in ValuesMap#compile.
+    exp.visitArgs(visitor);
     if (lexp != null)
       {
 	lexp.setInlineOnly(true);
@@ -260,91 +205,112 @@ public class CompileMisc implements Inlineable
   static gnu.bytecode.ClassType typeType;
   static gnu.bytecode.Method coerceMethod;
 
-  public static void compileConvert (Convert proc, ApplyExp exp, Compilation comp, Target target)
-  {
-    Expression[] args = exp.getArgs();
-    if (args.length != 2)
-      throw new Error ("wrong number of arguments to "+proc.getName());
-    CodeAttr code = comp.getCode();
-    Type type = Scheme.getTypeValue(args[0]);
-    if (type == Type.neverReturnsType)
-      {
-        args[1].compile(comp, Target.Ignore);
-        PrimProcedure.compileReachedUnexpected(code);
-      }
-    else if (type != null)
-      {
-        args[1].compile(comp, Target.pushValue(type));
-	if (code.reachableHere())
-	  target.compileFromStack(comp, type);
-      }
-    else
-      {
-	if (typeType == null)
-	  {
-	    typeType = ClassType.make("gnu.bytecode.Type");
-          }
-        if (coerceMethod == null)
-          {
-	    coerceMethod = typeType.addMethod("coerceFromObject",
-					      Compilation.apply1args,
-					      Type.pointer_type,
-					      gnu.bytecode.Access.PUBLIC);
-	  }
-	args[0].compile(comp, LangObjType.typeClassType);
-	args[1].compile(comp, Target.pushObject);
-	code.emitInvokeVirtual(coerceMethod);
-	target.compileFromStack(comp, Type.pointer_type);
-      }
-  }
+    public static boolean compileConvert (ApplyExp exp, Compilation comp,
+                                          Target target, Procedure procedure) {
+        Convert proc = (Convert) procedure;
+        Expression[] args = exp.getArgs();
+        if (args.length != 2 || ! exp.isSimple())
+            return false;
+        CodeAttr code = comp.getCode();
+        Type type = Scheme.getTypeValue(args[0]);
+        if (type == Type.neverReturnsType) {
+            args[1].compile(comp, Target.Ignore);
+            PrimProcedure.compileReachedUnexpected(code);
+        } else if (type != null) {
+            args[1].compile(comp, Target.pushValue(type));
+            if (code.reachableHere() && ! type.isVoid()) {
+                target.compileFromStack(comp, type);
+            }
+        } else {
+            if (typeType == null) {
+                typeType = ClassType.make("gnu.bytecode.Type");
+            }
+            if (coerceMethod == null) {
+                coerceMethod = typeType.addMethod("coerceFromObject",
+                                                  Compilation.apply1args,
+                                                  Type.pointer_type,
+                                                  gnu.bytecode.Access.PUBLIC);
+            }
+            args[0].compile(comp, LangObjType.typeClassType);
+            args[1].compile(comp, Target.pushObject);
+            code.emitInvokeVirtual(coerceMethod);
+            target.compileFromStack(comp, Type.pointer_type);
+        }
+        return true;
+    }
 
-  public void compileNot (Not proc, ApplyExp exp, Compilation comp, Target target)
-  {
-    Expression arg = exp.getArgs()[0];
-    Language language = proc.language;
-    if (target instanceof ConditionalTarget)
-      {
-	ConditionalTarget ctarget = (ConditionalTarget) target;
-	ConditionalTarget sub_target
-	  = new ConditionalTarget(ctarget.ifFalse, ctarget.ifTrue, language);
-	sub_target.trueBranchComesFirst = ! ctarget.trueBranchComesFirst;
-	arg.compile(comp, sub_target);
-	return;
-      }
-    CodeAttr code = comp.getCode();
-    Type type = target.getType();
-    if (target instanceof StackTarget && type.getSignature().charAt(0) == 'Z')
-      {
-	arg.compile(comp, target);
-	code.emitNot(target.getType());
-      }
-    else
-      {
-        QuoteExp trueExp = QuoteExp.getInstance(language.booleanObject(true));
-        QuoteExp falseExp = QuoteExp.getInstance(language.booleanObject(false));
-	IfExp.compile(arg, falseExp, trueExp, comp, target);
-      }
-  }
+    public static boolean compileNot(ApplyExp exp, Compilation comp,
+                                     Target target, Procedure procedure) {
+        if (! exp.isSimple())
+            return false;
+        Not proc = (Not) procedure;
+        Expression arg = exp.getArgs()[0];
+        Language language = proc.language;
+        if (target instanceof ConditionalTarget) {
+            ConditionalTarget ctarget = (ConditionalTarget) target;
+            ConditionalTarget sub_target
+                = new ConditionalTarget(ctarget.ifFalse, ctarget.ifTrue,
+                                        language);
+            sub_target.trueBranchComesFirst = ! ctarget.trueBranchComesFirst;
+            arg.compile(comp, sub_target);
+            return true;
+        }
+        CodeAttr code = comp.getCode();
+        Type type = target.getType();
+        if (target instanceof StackTarget
+                && type.getSignature().charAt(0) == 'Z') {
+            arg.compile(comp, target);
+            code.emitNot(target.getType());
+        } else {
+            QuoteExp trueExp
+                = QuoteExp.getInstance(language.booleanObject(true));
+            QuoteExp falseExp
+                = QuoteExp.getInstance(language.booleanObject(false));
+            IfExp.compile(arg, falseExp, trueExp, comp, target);
+        }
+        return true;
+    }
 
-  public static void compileEq (Expression[] args, Compilation comp,
-			      Target target, Language language)
-  {
-    CodeAttr code = comp.getCode();
-    args[0].compile(comp, Target.pushObject);
-    args[1].compile(comp, Target.pushObject);
-    if (target instanceof ConditionalTarget)
-      {
-	ConditionalTarget ctarget = (ConditionalTarget) target;
-	if (ctarget.trueBranchComesFirst)
-	  code.emitGotoIfNE(ctarget.ifFalse);
-	else
-	  code.emitGotoIfEq(ctarget.ifTrue);
-	ctarget.emitGotoFirstBranch(code);
-      }
+    public static boolean compileEq(ApplyExp exp, Compilation comp,
+                                    Target target, Procedure proc) {
+        if (! exp.isSimple())
+            return false;
+        Expression[] args = exp.getArgs();
+        Language language = ((IsEq) proc).language;
+        CodeAttr code = comp.getCode();
+        Expression arg0 = args[0];
+        Expression arg1 = args[1];
+        if (arg0==QuoteExp.nullExp) {
+            Expression tmp = arg1; arg1 = arg0; arg0 = tmp;
+        }
+        arg0.compile(comp, Target.pushObject);
+        boolean isNull = arg1==QuoteExp.nullExp;
+        if (! isNull)
+            arg1.compile(comp, Target.pushObject);
+
+        if (target instanceof ConditionalTarget) {
+            ConditionalTarget ctarget = (ConditionalTarget) target;
+            if (ctarget.trueBranchComesFirst) {
+                if (isNull)
+                    code.emitGotoIfNonNull(ctarget.ifFalse);
+                else
+                    code.emitGotoIfNE(ctarget.ifFalse);
+            } else {
+                if (isNull)
+                    code.emitGotoIfNull(ctarget.ifTrue);
+                else
+                    code.emitGotoIfEq(ctarget.ifTrue);
+            }
+            ctarget.emitGotoFirstBranch(code);
+        }
     else
       {
 	Type type;
-	code.emitIfEq();
+        if (isNull)
+            code.emitIfNull();
+        else {
+            code.emitIfEq();
+        }
 	if (target.getType() instanceof ClassType)
 	  {
 	    Object trueValue = language.booleanObject(true);
@@ -367,9 +333,13 @@ public class CompileMisc implements Inlineable
 	code.emitFi();
 	target.compileFromStack(comp, type);
       }
+      return true;
   }
 
-    public static void compileNumberCompare(NumberCompare proc, ApplyExp exp, Compilation comp, Target target) {
+    public static boolean compileNumberCompare(ApplyExp exp, Compilation comp, Target target, Procedure procedure) {
+        NumberCompare proc = (NumberCompare) procedure;
+        if (! exp.isSimple())
+            return false;
         CodeAttr code = comp.getCode();
         Expression[] args = exp.getArgs();
         if (args.length == 2) {
@@ -384,7 +354,7 @@ public class CompileMisc implements Inlineable
                 if (! (target instanceof ConditionalTarget)) {
                     IfExp.compile(exp, QuoteExp.trueExp, QuoteExp.falseExp,
                                   comp, target);
-                    return;
+                    return true;
                 }
                 int mask = proc.flags;
                 if (mask == NumberCompare.TRUE_IF_NEQ)
@@ -465,7 +435,7 @@ public class CompileMisc implements Inlineable
                     code.emitGotoIfCompare2(label1, opcode);
                 }
                 ctarget.emitGotoFirstBranch(code);
-                return;
+                return true;
             }
         }
 
@@ -492,9 +462,9 @@ public class CompileMisc implements Inlineable
                 : compclass.getDeclaredMethod(mname+"$V", 4);
             new ApplyExp(meth, args).setLine(exp)
                 .compile(comp, target);
-            return;
+            return true;
         }
-        ApplyExp.compile(exp, comp, target);
+        return false;
     }
 
   static int classifyForNumCompare (Expression exp)
@@ -514,32 +484,35 @@ public class CompileMisc implements Inlineable
     return kind;
   }
 
-  public static void compileNumberPredicate (NumberPredicate proc, ApplyExp exp, Compilation comp, Target target)
-  {
-    Expression[] args = exp.getArgs();
-    int op = proc.op;
-    if (args.length == 1 && (op == NumberPredicate.ODD || op == NumberPredicate.EVEN))
-      {
-	Expression arg0 = args[0];
-        int kind = Arithmetic.classifyType(arg0.getType());
-        if (kind <= Arithmetic.INTNUM_CODE)
-          {
-            PrimType wtype = Type.intType;
-            Target wtarget = StackTarget.getInstance(wtype);
-            CodeAttr code = comp.getCode();
-            if (op == NumberPredicate.EVEN)
-              code.emitPushInt(1);
-            arg0.compile(comp, wtarget);
-            code.emitPushInt(1);
-            code.emitAnd();
-            if (op == NumberPredicate.EVEN)
-              code.emitSub(Type.intType);
-            target.compileFromStack(comp, Type.booleanType);
-            return;
-          }
-      }
-   ApplyExp.compile(exp, comp, target);
-  }
+    public static boolean compileNumPredicate(ApplyExp exp, Compilation comp,
+                                              Target target,
+                                              Procedure procedure) {
+        NumberPredicate proc = (NumberPredicate) procedure;
+        if (! exp.isSimple())
+            return false;
+        Expression[] args = exp.getArgs();
+        int op = proc.op;
+        if (args.length == 1
+            && (op == NumberPredicate.ODD || op == NumberPredicate.EVEN)) {
+            Expression arg0 = args[0];
+            int kind = Arithmetic.classifyType(arg0.getType());
+            if (kind <= Arithmetic.INTNUM_CODE) {
+                PrimType wtype = Type.intType;
+                Target wtarget = StackTarget.getInstance(wtype);
+                CodeAttr code = comp.getCode();
+                if (op == NumberPredicate.EVEN)
+                    code.emitPushInt(1);
+                arg0.compile(comp, wtarget);
+                code.emitPushInt(1);
+                code.emitAnd();
+                if (op == NumberPredicate.EVEN)
+                    code.emitSub(Type.intType);
+                target.compileFromStack(comp, Type.booleanType);
+                return true;
+            }
+        }
+        return false;
+    }
 
   public static Expression validateApplyCallCC
   (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc)
@@ -553,6 +526,9 @@ public class CompileMisc implements Inlineable
         Declaration contDecl = lexp.firstDecl();
         if (! contDecl.getFlag(Declaration.TYPE_SPECIFIED))
           contDecl.setType(typeContinuation);
+        LambdaExp.maybeSetReturnType(lexp, required);
+        // FIXME (future): (after visiting lexp), do:
+        // exp.setType(lexp.body.getType() UNION continuation-arguments);
       }
     exp.visitArgs(visitor);
     return exp;
@@ -573,6 +549,7 @@ public class CompileMisc implements Inlineable
     final Declaration param = lambda.firstDecl();
     if (param.isSimple() && ! param.getCanRead() && ! param.getCanWrite())
       {
+        param.setCanCall(false);
         CompileTimeContinuation contProxy = new CompileTimeContinuation();
         Type rtype = target instanceof StackTarget ? target.getType() : null;
         boolean runFinallyBlocks
@@ -581,7 +558,7 @@ public class CompileMisc implements Inlineable
         contProxy.exitableBlock = bl;
         contProxy.blockTarget = target;
         param.setValue(new QuoteExp(contProxy));
-        lambda.body.compile(comp, target);
+        (new ApplyExp(lambda, QuoteExp.nullExp)).compile(comp, target);
         code.endExitableBlock();
         return;
       }
@@ -594,7 +571,7 @@ public class CompileMisc implements Inlineable
     comp.loadCallContext();
     code.emitInvokeSpecial(typeContinuation.getDeclaredMethod("<init>", 1));
     code.emitStore(contVar);
-    code.emitTryStart(false, target instanceof IgnoreTarget || target instanceof ConsumerTarget ? null : Type.objectType);
+    code.emitTryStart(false, target instanceof IgnoreTarget || target instanceof ConsumerTarget ? null : target.getType().getRawType());
     ApplyExp app = new ApplyExp(lambda, new ReferenceExp(contDecl) );
     app.compile(comp, target);
     // Emit: cont.invoked = true
@@ -644,6 +621,79 @@ public class CompileMisc implements Inlineable
     return null;
   }
 
+    public static Expression validateApplyWithExceptionHandler
+        (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc) {
+        Expression[] args = exp.getArgs();
+        Expression handler = visitor.visit(args[0], null);
+        args[0] = handler;
+        Expression thunk = args[1];
+        if (thunk instanceof LambdaExp) {
+            LambdaExp lthunk = (LambdaExp) thunk;
+            if (lthunk.min_args == 0 && lthunk.max_args == 0) {
+                LambdaExp.maybeSetReturnType(lthunk, required);
+                thunk = visitor.visit(lthunk, required);
+                args[1] = thunk;
+
+                /* Rewrite to:
+                     (let ((link (HandlerLink:push handler)))
+                      (try-catch
+                              (let ((v (thunk)))
+                               (link:pop)
+                               v)
+                       (ex java.lang.Throwable
+                        (primitive-throw (link:handle ex)))))
+                * This allows the thunk to be inlined.
+                */
+                Compilation comp = visitor.getCompilation();
+                comp.letStart();
+                ClassType handlerLinkType =
+                    ClassType.make("kawa.lib.HandlerLink");
+                Method pushMethod =
+                    handlerLinkType.getDeclaredMethod("push", 1);
+                Method popMethod =
+                    handlerLinkType.getDeclaredMethod("pop", 0);
+                Declaration linkDecl =
+                    comp.letVariable(null, handlerLinkType,
+                                     new ApplyExp(pushMethod, handler));
+                comp.letEnter();
+                Expression tryClause;
+                Type bodyType = lthunk.getReturnType();
+                Expression bodyCall = new ApplyExp(thunk);
+                Expression popHandler =
+                    new ApplyExp(popMethod, new ReferenceExp(linkDecl));
+                if (bodyType.isVoid()) {
+                    tryClause = new BeginExp(bodyCall, popHandler);
+                } else {
+                    comp.letStart();
+                    Declaration resultDecl =
+                        comp.letVariable(null, bodyType, bodyCall);
+                    comp.letEnter();
+                    tryClause =
+                        comp.letDone
+                        (new BeginExp(popHandler,
+                                      new ReferenceExp(resultDecl)));
+                }
+                TryExp texp = new TryExp(tryClause, null);
+                Declaration exDecl =
+                    new Declaration(null, Type.javalangThrowableType);
+
+                Expression doHandle =
+                    new ApplyExp(handlerLinkType
+                                 .getDeclaredMethod("handle", 1),
+                                 new ReferenceExp(linkDecl),
+                                 new ReferenceExp(exDecl));
+                texp.addCatchClause(exDecl,
+                                    new ApplyExp(Throw.primitiveThrow,
+                                                 doHandle));
+                return visitor.visit(comp.letDone(texp), required);
+            }
+        }
+        thunk = visitor.visit(thunk, null);
+        args[1] = thunk;
+        return exp;
+    }
+
+
   /** An ExpVisitor class to check if callcc exits through a try-finally. */
   static class ExitThroughFinallyChecker extends ExpVisitor<Expression,TryExp>
   {
