@@ -152,7 +152,18 @@ public class LispPackage extends Namespace
    */
   public static LispPackage valueOf (String name)
   {
-    return (LispPackage) Namespace.valueOf(name);
+    if (name == null)
+      name = "";
+    
+    synchronized (nsTable) {
+      Namespace ns = (Namespace) nsTable.get(name);
+      if (ns != null)
+        return (LispPackage) ns;
+      ns = new LispPackage();
+      ns.setName(name.intern());
+      Namespace.nsTable.put(name, ns);
+      return (LispPackage) ns;
+    }
   }
   
   public static Namespace valueOfNoCreate (String name)
@@ -160,26 +171,20 @@ public class LispPackage extends Namespace
     return (LispPackage) Namespace.valueOfNoCreate(name);
   }
   
-  public Values findSymbol (Object name)
+  public Values.Values2 findSymbol (Object name)
   {
     String sname = name.toString();
 
     Symbol sym = exported.lookup(sname);
     if (sym != null)
     {
-      return new Values(new Object[]
-      {
-        sym, CommonLisp.externalKeyword
-      });
+      return Values.values2(sym, CommonLisp.externalKeyword);
     }
     
     sym = lookupInternal(sname, sname.hashCode());
     if (sym != null)
     {
-      return new Values(new Object[]
-      {
-        sym, CommonLisp.internalKeyword
-      });
+      return Values.values2(sym, CommonLisp.internalKeyword);
     }
     
     // It's not an exported or an imported symbol, let's check the inheritance
@@ -194,19 +199,13 @@ public class LispPackage extends Namespace
 
       if (sym != null)
       {
-        return new Values(new Object[]
-        {
-          sym, CommonLisp.inheritedKeyword
-        });
+        return Values.values2(sym, CommonLisp.inheritedKeyword);
       }
 
       U = U.nextImported;
     }
 
-    return new Values(new Object[]
-    {
-      CommonLisp.FALSE, CommonLisp.FALSE
-    });
+    return Values.values2(CommonLisp.FALSE, CommonLisp.FALSE);
   }
   
   /** Export a list of symbols from a package, checking for conflicts.
