@@ -4,27 +4,34 @@ import gnu.mapping.*;
 import gnu.lists.*;
 import gnu.math.*;
 import gnu.expr.*;
+import gnu.kawa.io.BinaryInPort;
+import gnu.kawa.io.InPort;
 import gnu.kawa.util.GeneralHashTable;
+import java.util.regex.*;
 
 /** A Lexer for reading S-expressions in generic Lisp-like syntax.
  * This class may have outlived its usefulness: It's mostly just a
- * wrapper around a LineBufferedReader plus a helper token-buffer.
+ * wrapper around an InPort plus a helper token-buffer.
  * The functionality should be moved to ReadTable, though it is
  * unclear what to do about the tokenBuffer.
  */
 
 public class LispReader extends Lexer
 {
-  public LispReader(LineBufferedReader port)
+  public LispReader(InPort port)
   {
     super(port);
   }
 
-  public LispReader(LineBufferedReader port, SourceMessages messages)
+  public LispReader(InPort port, SourceMessages messages)
   {
     super(port, messages);
   }
 
+    boolean returnMutablePairs;
+    /** Set whether returned pairs are mutable or not (the default). */
+    public void setReturnMutablePairs(boolean v) { returnMutablePairs = v; }
+
   GeneralHashTable<Integer,Object> sharedStructureTable;
 
     /** Bind value to index in sharingStructuretable.
@@ -48,39 +55,57 @@ public class LispReader extends Lexer
 	return value;
     }
 
-  /** Read a #|...|#-style comment (which may contain other nested comments).
-    * Assumes the initial "#|" has already been read.
-    */
-  final public void readNestedComment (char c1, char c2)
-       throws java.io.IOException, SyntaxException
-  {
-    int commentNesting = 1;
-    int startLine = port.getLineNumber();
-    int startColumn = port.getColumnNumber();
-    do
-      {
-	int c = read ();
-	if (c == '|')
-	  {
-	    c = read();
-	    if (c == c1)
-	      commentNesting--;
-	  }
-	else if (c == c1)
-	  {
-	    c = read();
-	    if (c == c2)
-	      commentNesting++;
-	  }
-	if (c < 0)
-	  {
-            eofError("unexpected end-of-file in " + c1 + c2
-                     + " comment starting here",
-                     startLine + 1, startColumn - 1);
-	    return;
-	  }
-      } while (commentNesting > 0);
-  }
+    /** Read a #|...|#-style comment (which may contain other nested comments).
+     * Assumes the initial "#|" has already been read.
+     */
+    final public void readNestedComment (char c1, char c2)
+        throws java.io.IOException, SyntaxException {
+        int commentNesting = 1;
+        int startLine = port.getLineNumber();
+        int startColumn = port.getColumnNumber();
+        StringBuilder buf = null;
+        if (port instanceof BinaryInPort && (startLine == 0 || startLine == 1))
+            buf = new StringBuilder();
+        do {
+            int c = read ();
+            if (buf != null)
+                buf.append((char) c);
+            if (c == '|') {
+                c = read();
+                if (buf != null)
+                    buf.append((char) c);
+                if (c == c1)
+                    commentNesting--;
+            } else if (c == c1) {
+                c = read();
+                if (c == c2)
+                    commentNesting++;
+            }
+            if (c < 0) {
+                eofError("unexpected end-of-file in " + c1 + c2
+                         + " comment starting here",
+                         startLine + 1, startColumn - 1);
+                return;
+            }
+        } while (commentNesting > 0);
+        if (buf != null)
+            checkEncodingSpec(buf.toString());
+    }
+
+    public void checkEncodingSpec(String line) {
+        Matcher m = Pattern.compile("coding[:=]\\s*([-a-zA-Z0-9]+)")
+            .matcher(line);
+        if (m.find()) {
+            String enc = m.group(1);
+            try {
+                ((BinaryInPort) getPort()).setCharset(enc);
+            } catch (java.nio.charset.UnsupportedCharsetException ex) {
+                error('e', "unrecognized encoding name "+enc);
+            } catch (Exception ex) {
+                error('e', "cannot set encoding name here");
+            }
+        }
+    }
 
     boolean inQuasiSyntax;
 
@@ -288,8 +313,11 @@ public class LispReader extends Lexer
 	String str = new String(tokenBuffer, startPos, len - 1);
 	result = Keyword.make(str.intern());
       }
-    else
+    else {
+      if (len == 1 && tokenBuffer[startPos] == '.' && !seenEscapes)
+          error("invalid use of '.' token");
       result = rtable.makeSymbol(new String(tokenBuffer, startPos, len));
+    }
     tokenBufferLength = startPos;
     return result;
   }
@@ -464,6 +492,8 @@ public class LispReader extends Lexer
             return false;
         if (ch == ',')
             return true;
+        if (ch == '@')
+            return true; // To support deprecated (TYPE:@ EXP)
         int kind = rtable.lookup(ch).getKind();
         return kind == ReadTable.CONSTITUENT
             || kind == ReadTable.NON_TERMINATING_MACRO
@@ -483,24 +513,30 @@ public class LispReader extends Lexer
             if (ch == '[' && rtable.defaultBracketMode == -2) {
                 port.read();
                 Object lst = ReaderParens.readList(this, null, ch, 1, ']', -1);
-                value = PairWithPosition.make(value, lst,
-                                              port.getName(), line+1, column+1);
-                value = PairWithPosition.make(LispLanguage.bracket_apply_sym, value,
-                                              port.getName(), line+1, column+1);
+                value = makePair(value, lst, line, column);
+                value = makePair(LispLanguage.bracket_apply_sym, value,
+                                              line, column);
             } else if (ch == rtable.postfixLookupOperator) {
                 // A kludge to map PreOpWord to ($lookup$ Pre 'Word).
                 port.read();
                 int ch2 = port.peek();
-                if (! validPostfixLookupStart(ch2, rtable)) {
-                    unread();
-                    break;
+                Object rightOperand;
+                if (ch2 == '@') {
+                    error('w',
+                          "deprecated cast syntax TYPE:@ (use ->TYPE instead)");
+                    rightOperand = readAndHandleToken('\\', 0, rtable);
+                } else {
+                    if (! validPostfixLookupStart(ch2, rtable)) {
+                        unread();
+                        break;
+                    }
+                    ch = port.read();
+                    rightOperand = readValues(ch, rtable.lookup(ch), rtable, -1);
                 }
-                ch = port.read();
-                Object rightOperand = readValues(ch, rtable.lookup(ch), rtable, -1);
                 value = LList.list2(value,
                                     LList.list2(rtable.makeSymbol(LispLanguage.quote_str), rightOperand));
-                value = PairWithPosition.make(LispLanguage.lookup_sym, value,
-                                              port.getName(), line+1, column+1);
+                value = makePair(LispLanguage.lookup_sym, value,
+                                 line, column);
             }
             else
                 break;
@@ -566,6 +602,7 @@ public class LispReader extends Lexer
    *   '\0' yields an inact or inexact depending on the form of the literal,
    *   while ' ' is like '\0' but does not allow more exactness specifiers.
    * @param radix the number base to use or 0 if unspecified
+   *   A negative radix is an overideable default.
    * @return the number if a valid number; null or a String-valued error
    *   message if if there was some error parsing the number.
    */
@@ -585,22 +622,22 @@ public class LispReader extends Lexer
 	switch (ch)
 	  {
 	  case 'b':  case 'B':
-	    if (radix != 0)
+	    if (radix > 0)
 	      return "duplicate radix specifier";
 	    radix = 2;
 	    break;
 	  case 'o':  case 'O':
-	    if (radix != 0)
+	    if (radix > 0)
 	      return "duplicate radix specifier";
 	    radix = 8;
 	    break;
 	  case 'd':  case 'D':
-	    if (radix != 0)
+	    if (radix > 0)
 	      return "duplicate radix specifier";
 	    radix = 10;
 	    break;
 	  case 'x':  case 'X':
-	    if (radix != 0)
+	    if (radix > 0)
 	      return "duplicate radix specifier";
 	    radix = 16;
 	    break;
@@ -629,7 +666,7 @@ public class LispReader extends Lexer
 	      }
 	    if (ch == 'R' || ch == 'r')
 	      {
-		if (radix != 0)
+		if (radix > 0)
 		  return "duplicate radix specifier";
 		if (value < 2 || value > 35)
 		  return "invalid radix specifier";
@@ -644,8 +681,12 @@ public class LispReader extends Lexer
       }
     if (exactness == '\0')
       exactness = ' ';
-    if (radix == 0)
+    if (radix < 0)
+        radix = -radix;
+    else if (radix == 0)
       {
+        radix = 10;
+        /*
 	for (int i = count;  ; )
 	  {
 	    if (--i < 0)
@@ -661,6 +702,7 @@ public class LispReader extends Lexer
 		break;
 	      }
 	  }
+        */
       }
 
     boolean negative = ch == '-';
@@ -1180,7 +1222,7 @@ public class LispReader extends Lexer
   protected Pair makePair (Object car, Object cdr, int line, int column)
   {
     String pname = port.getName();
-    if (pname != null && line >= 0)
+    if (! returnMutablePairs && pname != null && line >= 0)
       return PairWithPosition.make(car, cdr,
                                    pname, line + 1, column + 1);
     else
@@ -1211,7 +1253,16 @@ public class LispReader extends Lexer
     throws java.io.IOException, SyntaxException
   {
     int startPos = reader.tokenBufferLength - previous;
-    reader.readToken(reader.read(), ReadTable.getCurrent());
+    ReadTable rtable = ReadTable.getCurrent();
+    for (;;) {
+        reader.readToken(reader.read(), rtable);
+        // '#' is a terminating-macro character so we have to add it "manually"
+        int ch = reader.peek();
+        if (ch != '#')
+            break;
+        reader.tokenBufferAppend(ch);
+        reader.skip();
+    }
     int endPos = reader.tokenBufferLength;
     if (startPos == endPos)
       {
@@ -1245,8 +1296,12 @@ public class LispReader extends Lexer
     reader.readToken(reader.read(), ReadTable.getCurrent());
     char[] tokenBuffer = reader.tokenBuffer;
     int length = reader.tokenBufferLength - startPos;
-    if (length == 1)
-      return Char.make(tokenBuffer[startPos]);
+    if (length == 1 || length == 2) {
+        ch = Character.codePointAt(tokenBuffer, startPos,
+                                   reader.tokenBufferLength);
+        if (ch > 0xFFFF || length == 1)
+            return Char.make(ch);
+    }
     String name = new String(tokenBuffer, startPos, length);
     ch = Char.nameToChar(name);
     if (ch >= 0)
@@ -1263,10 +1318,13 @@ public class LispReader extends Lexer
              if (v < 0)
                break;
              value = 16 * value + v;
-             if (value > 0x10FFFF)
-               break;
+             if (value > 0x10FFFF) {
+                 reader.error("character scalar value greater than #x10FFFF");
+                 return Char.make('?');
+             }
           }
       }
+    // FIXME remove - only used for BRL Perhaps a deprecation warning?
     ch = Character.digit(ch, 8);
     if (ch >= 0)
       {
