@@ -2,6 +2,7 @@
 // This is free software;  for terms and warranty disclaimer see ../../../COPYING.
 
 package gnu.kawa.lispexpr;
+import gnu.kawa.io.InPort;
 import gnu.mapping.*;
 import gnu.lists.*;
 import gnu.expr.*;
@@ -27,15 +28,7 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
         int startColumn = reader.getColumnNumber() - 2;
         in.tokenBufferAppend(ch);
         int next = reader.read();
-        if (XName.isNameStart(next)) {
-            for (;;) {
-                reader.tokenBufferAppend(next);
-                next = reader.read();
-                if (! XName.isNamePart(next)) {
-                    break;
-                }
-            }
-        }
+        next = scanTag(reader, next);
         Object result;
         if (next == '{' || next == '[') {
             int len = reader.tokenBufferLength - startPos - 1;
@@ -153,7 +146,7 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
                     }
                     else
                         reader.tokenBufferLength = lineStart;
-                     reader.skip();
+                    reader.skip();
                     continue;
                 } else if (next1 == '-') {
                     reader.skip();
@@ -185,6 +178,16 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
                 if (isNestableStartDelim(next))
                     braceNesting++;
                 reader.tokenBufferAppend(next);
+                if (next == ']' && delimiter == '<') {
+                    if (reader.peek() == ']') {
+                        reader.skip();
+                        reader.tokenBufferAppend(']');
+                        if (reader.peek() == '>') {
+                            reader.error('w', reader.getName(), line, column+1,
+                                         "literal ']]>' is only valid following '<![CDATA['");
+                        }
+                    }
+                }
                 next = ' ';
             }
             if (reader.tokenBufferLength > 0
@@ -264,8 +267,12 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
                     item = fhead;
                 }
                 else {
-                    String str = readName(reader, next);
-                    next = reader.peek();
+                    int startPos = reader.tokenBufferLength;
+                    next = scanTag(reader, next);
+                    String str = new String(reader.tokenBuffer, startPos,
+                                         reader.tokenBufferLength-startPos);
+                    reader.tokenBufferLength = startPos;
+                    reader.unread(next);
                     if (next == '[' || next == '{') {
                         item = readNamedLiteral(reader, rtable, str, reader.read(),
                                                 line, column);
@@ -335,7 +342,7 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
      */
     protected Pair readEnclosedExpressions(LispReader reader, ReadTable readTable, Pair last, int endDelimiter)
             throws IOException, SyntaxException {
-        LineBufferedReader port = reader.getPort();
+        InPort port = reader.getPort();
         char saveReadState = reader.pushNesting('[');
         int startLine = port.getLineNumber();
         int startColumn = port.getColumnNumber();
@@ -359,22 +366,29 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
           }
     }
 
-    protected String readName(LispReader reader, int next)
+    private int scanTag(LispReader reader, int next)
         throws IOException, SyntaxException {
-	int saveLength = reader.tokenBufferLength;
-	while (next >= 0)
-	  {
-	    char ch = (char) next;
-	    if (! XName.isNamePart(ch)) {
-                reader.unread(ch);
-                break;
+        if (XName.isNameStart(next)) {
+            for (;;) {
+                reader.tokenBufferAppend(next);
+                next = reader.read();
+                if (! XName.isNamePart(next)) {
+                    break;
+                }
             }
-	    reader.tokenBufferAppend(ch);
-	    next = reader.read();
-	  }
-        int len = reader.tokenBufferLength - saveLength;
-        reader.tokenBufferLength = saveLength;
-        return new String(reader.tokenBuffer, saveLength, len);
+        } else if (next == '`' || next == '<' || next == '>') {
+            int nextnext = reader.peek();
+            if (next == '>' && nextnext == '>') {
+                reader.tokenBufferAppend(next);
+                reader.skip();
+                nextnext = reader.peek();
+            }
+            if (nextnext == '{' || nextnext == '[') {
+                reader.tokenBufferAppend(next);
+                next = reader.read();
+            }
+        }
+        return next;
     }
 
     Object checkEntity(LispReader reader, String str)
@@ -387,14 +401,6 @@ public class ReaderExtendedLiteral extends ReaderConstituent {
         return LispLanguage.entityNamespace.getSymbol(str);
     }
 
-    /** Read entity following {@code '&'}.
-     * If result is null, it was appended to the token-buffer.
-     */
-    Object readEntity (LispReader reader, int next)
-        throws IOException, SyntaxException {
-        return checkEntity(reader, readName(reader, next));
-    }
-
     /** Read a character reference, assuming {@code "&#"} have been read. */
     void readCharRef (LispReader reader, int next)
         throws IOException, SyntaxException {
