@@ -3,7 +3,7 @@
 
 package gnu.kawa.lispexpr;
 import gnu.text.*;
-import gnu.mapping.InPort;
+import gnu.kawa.io.InPort;
 
 /** Reader table entry for '<' to treat '[' and ']' as constituents.
  * Lets us use (say) '<char[]>' as a token even if  '[' and ']' are parens.
@@ -21,7 +21,7 @@ public class ReaderTypespec extends ReadTableEntry
     throws java.io.IOException, SyntaxException
   {
     int startPos = in.tokenBufferLength;
-    LineBufferedReader port = in.getPort();
+    InPort port = in.getPort();
     ReadTable rtable = ReadTable.getCurrent();
     char saveReadState = '\0';
     in.tokenBufferAppend(ch);
