@@ -393,7 +393,8 @@
 			 ((#\f #\F) (f digs exp #f))
 			 ((#\g #\G) (g digs exp))
 			 ((#\k) (k digs exp ""))
-			 ((#\K) (k digs exp " "))))
+			 ((#\K) (k digs exp " "))
+                         (else "<bad printf code>")))
 		      (append (format-real signed? sgn digs exp)
 			      (apply format-real #t rest)
 			      '("i"))))
