@@ -42,15 +42,6 @@ public abstract class ByteVector extends SimpleVector
     return data[index];
   }
 
-  public boolean consumeNext (int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeInt(intAtBuffer(index));
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
@@ -117,6 +108,10 @@ public abstract class ByteVector extends SimpleVector
     this.size = size;
   }
 
+    public InputStream getInputStream() {
+        return new ByteArrayInputStream(data, 0, size);
+    }
+
     public int readFrom(int start, int count, InputStream in)
         throws IOException {
         return in.read(data, start, count);
