@@ -108,11 +108,10 @@ public class F64Vector extends SimpleVector
     data[index] = value;
   }
 
-  public final Object setBuffer(int index, Object value)
+  @Override
+  public final void setBuffer(int index, Object value)
   {
-    Object old = Convert.toObject(data[index]);
     data[index] = Convert.toDouble(value);
-    return old;
   }
 
   /*
@@ -135,15 +134,6 @@ public class F64Vector extends SimpleVector
 
   public String getTag() { return "f64"; }
 
-  public boolean consumeNext (int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeDouble(data[index]);
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
