@@ -114,12 +114,11 @@ public class FVector<E> extends SimpleVector<E>
     return (E) data[index];
   }
 
-  public final Object setBuffer(int index, Object value)
+  @Override
+  public final void setBuffer(int index, Object value)
   {
     checkCanWrite();
-    Object old = data[index];
     data[index] = value;
-    return old;
   }
 
   protected void clearBuffer(int start, int count)
@@ -183,15 +182,6 @@ public class FVector<E> extends SimpleVector<E>
         Arrays.fill(data, start, end, new_value);
     }
 
-  public boolean consumeNext(int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeObject(data[index]);
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
