@@ -6,7 +6,11 @@ import java.io.*;
 
 /** Simple adjustable-length vector of signed 16-bit integers (shorts). */
 
-public class S16Vector extends SimpleVector implements Externalizable
+public class S16Vector extends SimpleVector
+  implements Externalizable
+  /* #ifdef JAVA2 */
+  , Comparable
+  /* #endif */
 {
   short[] data;
   protected static short[] empty = new short[0];
@@ -92,11 +96,10 @@ public class S16Vector extends SimpleVector implements Externalizable
     return Convert.toObject(data[index]);
   }
 
-  public Object setBuffer(int index, Object value)
+  @Override
+  public void setBuffer(int index, Object value)
   {
-    short old = data[index];
     data[index] = Convert.toShort(value);
-    return Convert.toObject(old);
   }
 
   public final void setShortAt(int index, short value)
@@ -124,15 +127,6 @@ public class S16Vector extends SimpleVector implements Externalizable
 
   public String getTag() { return "s16"; }
 
-  public boolean consumeNext (int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeInt(data[index]);
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
@@ -145,6 +139,11 @@ public class S16Vector extends SimpleVector implements Externalizable
       out.writeInt(data[i]);
   }
 
+  public int compareTo(Object obj)
+  {
+    return compareToInt(this, (S16Vector) obj);
+  }
+
   /**
    * @serialData Write 'size' (using writeInt),
    *   followed by 'size' elements in order (using writeShort).
