@@ -20,7 +20,7 @@ public abstract class SimpleVector<E> extends AbstractSequence<E>
 {
   /** The (current) number of elements.
    * Must always have size() >= 0 && size() <= getBufferLength(). */
-  public int size;
+  protected int size;
 
   public final int size() { return size; }
 
@@ -101,11 +101,6 @@ public abstract class SimpleVector<E> extends AbstractSequence<E>
     return ipos == -1 ? size : ipos >>> 1;
   }
 
-  public int createPos(int index, boolean isAfter)
-  {
-    return (index << 1) | (isAfter ? 1 : 0);
-  }
-
   public int nextPos (int ipos)
   {
     if (ipos == -1)
@@ -113,7 +108,7 @@ public abstract class SimpleVector<E> extends AbstractSequence<E>
     int index = ipos >>> 1;
     if (index == size)
       return 0;
-    return  (index << 1) + 3;
+    return (index << 1) + 3;
   }
 
   /*
@@ -183,7 +178,7 @@ public abstract class SimpleVector<E> extends AbstractSequence<E>
     return old;
   }
 
-  protected abstract E setBuffer(int index, E value);
+  protected abstract void setBuffer(int index, E value);
 
     /* #ifdef JAVA8 */
     // @Override
@@ -439,15 +434,6 @@ public abstract class SimpleVector<E> extends AbstractSequence<E>
     consumePosRange(start << 1, (start + length) << 1, out);
   }
 
-  public boolean consumeNext(int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeObject(getBuffer(index));
-    return true;
-  }
-
   public void consumePosRange(int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
