@@ -1,7 +1,9 @@
 package kawa;
 import gnu.expr.*;
 import gnu.mapping.*;
-import gnu.text.*;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.FilePath;
+import gnu.kawa.io.TtyInPort;
 
 public class TelnetRepl extends Procedure0
 {
