@@ -1,7 +1,8 @@
 package kawa.lang;
 import gnu.mapping.*;
-import gnu.expr.*;
 import java.io.*;
+import gnu.text.Printable;
+import gnu.lists.Consumer;
 
 /**
  * A pattern that matches anything.
@@ -13,9 +14,9 @@ public class AnyPat extends Pattern implements Printable, Externalizable
 
   public static AnyPat make () { return new AnyPat (); }
 
-  public void print(java.io.PrintWriter ps)
+  public void print (Consumer out)
   {
-    ps.print ("#<match any>");
+    out.write("#<match any>");
   }
 
   public boolean match (Object obj, Object[] vars, int start_vars)
