@@ -3,8 +3,8 @@ import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.lists.*;
 import gnu.bytecode.Type;
-import gnu.kawa.functions.Convert;
 import gnu.kawa.lispexpr.LangObjType;
+import gnu.kawa.lispexpr.LispLanguage;
 import java.util.ArrayList;
 
 /**
@@ -20,6 +20,7 @@ public class Lambda extends Syntax
 
   public static final Keyword nameKeyword = Keyword.make("name");
 
+  // This should technically have the type of Scheme.booleanType.
   public Expression defaultDefault = QuoteExp.falseExp;
 
   public void setKeywords(Object optional, Object rest, Object key)
@@ -490,7 +491,9 @@ public class Lambda extends Syntax
           }
 	else
 	  {
-	    tr.error('w', "unknown procedure property "+attrName);
+            Expression attrExpr = tr.rewrite_car(pair2, syntax1);
+            attrName = ((Keyword) attrName).asSymbol();
+            lexp.setProperty(attrName, attrExpr);
 	  }
 	body = pair2.getCdr();
       }
@@ -551,7 +554,9 @@ public class Lambda extends Syntax
         if (texp instanceof LangExp)
           {
             Pair typeSpecPair = (Pair) ((LangExp) texp).getLangValue(); 
-            tr.exp2Type(typeSpecPair, cur, null/*FIXME*/);
+            Type t = tr.exp2Type(typeSpecPair, cur, null/*FIXME*/);
+            if (t != null)
+                cur.setType(t);
           }
 	prev = cur;
 
@@ -577,19 +582,21 @@ public class Lambda extends Syntax
     LambdaExp saveLambda = tr.curLambda;
     tr.curLambda = lexp;
     Type rtype = lexp.returnType;
-    if (lexp.body instanceof LangExp)
-      {
-        Object[] tform = (Object[]) ((LangExp) lexp.body).getLangValue();
-        Expression texp = tr.rewrite_car((Pair) tform[0],
-                                         (SyntaxForm) tform[1]);
-        rtype = tr.getLanguage().getTypeFor(texp);
-      }
+    Object[] tform = lexp.body instanceof LangExp
+        ? (Object[]) ((LangExp) lexp.body).getLangValue()
+        : null;
     lexp.body = auxillaryRewrite(body, tr);
     tr.curLambda = saveLambda;
     Expression[] exps;
     int len;
     Object val;
-    if (lexp.body instanceof BeginExp
+    if (tform != null)
+      {
+        Expression texp = tr.rewrite_car((Pair) tform[0],
+                                         (SyntaxForm) tform[1]);
+        lexp.setCoercedReturnValue(texp, tr.getLanguage());
+      }
+    else if (lexp.body instanceof BeginExp
         && (len = (exps = ((BeginExp) lexp.body).getExpressions()).length) > 1
         && (exps[0] instanceof ReferenceExp
             || ((val = exps[0].valueIfConstant()) instanceof Type
@@ -631,7 +638,13 @@ public class Lambda extends Syntax
 
   public static boolean isAnnotationSymbol (Object key)
   {
-    if (key instanceof SimpleSymbol)
+    if (key instanceof Pair)
+      {
+        Pair keyp = (Pair) key;
+        if (keyp.getCar() == LispLanguage.splice_sym)
+          return true;
+      }
+    if (key instanceof SimpleSymbol) // Deprecated: Symbol starting with '@'
       {
         String name = ((SimpleSymbol) key).getName();
         if (name.length() > 1 && name.charAt(0) == '@')
