@@ -106,7 +106,8 @@ public class Macro extends Syntax implements Printable, Externalizable
   public Macro(Object name, Procedure expander)
   {
     super(name);
-    this.expander = new QuoteExp(expander);
+    this.expander = expander instanceof Expression ? expander
+        : new QuoteExp(expander);
   }
 
   public Macro(Object name)
@@ -114,16 +115,9 @@ public class Macro extends Syntax implements Printable, Externalizable
     super(name);
   }
 
-  /* FIXME redundant */
-  public gnu.expr.Expression rewriteForm (Pair form, Translator tr)
-  {
-    return tr.rewrite(expand(form, tr));
-  }
-
-  public gnu.expr.Expression rewriteForm (Object form, Translator tr)
-  {
-    return tr.rewrite(expand(form, tr));
-  }
+    public gnu.expr.Expression rewriteForm(Pair form, Translator tr) {
+        return tr.rewrite(expand(form, tr), 'N');
+    }
 
   public String toString()
   {
@@ -168,7 +162,7 @@ public class Macro extends Syntax implements Printable, Externalizable
               {
                 System.err.println("expand "+this+" expander:"+exp);
                 System.err.flush();
-                OutPort dout = OutPort.errDefault();
+                gnu.kawa.io.OutPort dout = gnu.kawa.io.OutPort.errDefault();
 		dout.flush();
                 ((Expression) exp).print(dout);
                 dout.println(']');
