@@ -14,6 +14,7 @@ import gnu.kawa.lispexpr.*;
 import java.util.*;
 import gnu.kawa.functions.GetNamedPart;
 import gnu.kawa.functions.CompileNamedPart;
+import gnu.kawa.functions.MakeSplice;
 import gnu.kawa.functions.MultiplyOp;
 import gnu.kawa.xml.XmlNamespace;
 import gnu.math.DFloNum;
@@ -55,7 +56,7 @@ public class Translator extends Compilation
    * This is (more-or-less) the mark specified by the syntax-case
    * specification (in r6rs-lib), applied to the output of a transformer.
    * However, instead of "applying" a mark to the transformer output,
-   * we remember in the TemlateScope an object unique to the application.
+   * we remember in the TemplateScope an object unique to the application.
    */
   Object currentMacroMark = null;
 
@@ -66,8 +67,6 @@ public class Translator extends Compilation
   /** A stack of aliases pushed by <code>pushRenamedAlias</code>. */
   private Stack<Declaration> renamedAliasStack;
 
-  public Stack formStack = new Stack();
-  public int firstForm;
   public Object pendingForm;
 
   public LambdaExp curMethodLambda;
@@ -88,13 +87,20 @@ public class Translator extends Compilation
 
   private static Expression errorExp = new ErrorExp ("unknown syntax error");
 
-  public Translator (Language language, SourceMessages messages, NameLookup lexical)
-  {
-    super(language, messages, lexical);
-    this.env = Environment.getCurrent();
-  }
+    public Translator(Language language, SourceMessages messages,
+                      NameLookup lexical, Environment env) {
+        super(language, messages, lexical);
+        this.env = env;
+    }
 
-  public final Environment getGlobalEnvironment() { return env; }
+    public Translator(Language language, SourceMessages messages,
+                      NameLookup lexical) {
+        super(language, messages, lexical);
+        this.env = Environment.getCurrent();
+    }
+
+    @Override
+    public final Environment getGlobalEnvironment() { return env; }
 
   public Expression parse (Object input)
   {
@@ -171,6 +177,10 @@ public class Translator extends Compilation
     return null;
   }
 
+    public final boolean keywordsAreSelfEvaluating() {
+        return ((LispLanguage) getLanguage()).keywordsAreSelfEvaluating();
+    }
+
   public final boolean selfEvaluatingSymbol (Object obj)
   {
     return ((LispLanguage) getLanguage()).selfEvaluatingSymbol(obj);
@@ -286,6 +296,10 @@ public class Translator extends Compilation
               macroContext = ((TemplateScope) current_scope).macroContext;
             obj = dval.eval(env);
           }
+        catch (Error ex)
+          {
+            throw ex;
+          }
         catch (Throwable ex)
           {
             ex.printStackTrace();
@@ -302,7 +316,17 @@ public class Translator extends Compilation
 
   public Expression rewrite_pair (Pair p, boolean function)
   {
-    Expression func = rewrite_car (p, true);
+    Object p_car = p.getCar();
+    Expression func;
+    boolean useHelper = true;
+    if (p_car instanceof Pair
+        && ((Pair) p_car).getCar() == LispLanguage.splice_sym) {
+        func = gnu.kawa.reflect.MakeAnnotation.makeAnnotationMaker
+            (rewrite_car((Pair) ((Pair) p_car).getCdr(), false));
+        useHelper = false;
+    }
+    else
+        func = rewrite_car (p, true);
     Object proc = null;
     if (func instanceof QuoteExp)
       {
@@ -353,6 +377,21 @@ public class Translator extends Compilation
 	  {
             Declaration saveContext = macroContext;
             Syntax syntax = check_if_Syntax (decl);
+            if (syntax == null)
+            {
+                Expression dval = Declaration.followAliases(decl).getValue();
+                Object val = dval == null ? null : dval.valueIfConstant();
+                SimpleSymbol eqmacSymbol = Symbol.valueOf("equivalent-syntax");
+                Object syn;
+                if (dval instanceof LambdaExp)
+                    syn = ((LambdaExp) dval).getProperty(eqmacSymbol, null);
+                else if (val instanceof Procedure)
+                    syn = ((Procedure) val).getProperty(eqmacSymbol, null);
+                else
+                    syn = null;
+                if (syn instanceof Procedure)
+                    syntax = new Macro(null, (Procedure) syn);
+            }
             if (syntax != null)
               {
                 Expression e = apply_rewrite (syntax, p);
@@ -366,18 +405,29 @@ public class Translator extends Compilation
 	  func.setFlag(ReferenceExp.PREFER_BINDING2);
       }
 
+    boolean isNamedPartDecl = func instanceof ReferenceExp
+        && (((ReferenceExp) func).getBinding()==getNamedPartDecl);
+    if (isNamedPartDecl)
+      useHelper = false;
+
     Object cdr = p.getCdr();
     int cdr_length = listLength(cdr);
 
-    if (cdr_length == -1)
-      return syntaxError("circular list is not allowed after "+p.getCar());
     if (cdr_length < 0)
-      return syntaxError("dotted list ["+cdr+"] is not allowed after "+p.getCar());
+      return syntaxError
+          ("improper list (circular or dotted) is not allowed here");
+    Expression applyFunction =  useHelper ? applyFunction(func) : null;
 
-    boolean mapKeywordsToAttributes = false;
     Stack vec = new Stack();
+    if (applyFunction != null) {
+        vec.add(func);
+        func = applyFunction;
+    }
 
     ScopeExp save_scope = current_scope;
+    int first_keyword = -1;
+    int last_keyword = -1;
+    int firstSpliceArg = -1;
     for (int i = 0; i < cdr_length;)
       {
 	if (cdr instanceof SyntaxForm)
@@ -389,45 +439,59 @@ public class Translator extends Compilation
               lexical.pushSaveTopLevelRedefs();
 	    setCurrentScope(sf.getScope());
 	  }
+        Object save_pos = pushPositionOf(cdr);
 	Pair cdr_pair = (Pair) cdr;
-	Expression arg = rewrite_car (cdr_pair, false);
+        Object cdr_car = cdr_pair.getCar();
+        Expression arg;
+        if (cdr_car instanceof Keyword) {
+            if (first_keyword < 0)
+                first_keyword = i;
+            else if (keywordsAreSelfEvaluating())
+                ;
+            else if (i == last_keyword + 1 || i + 1 == cdr_length)
+                error('w', "missing keyword value");
+            else if (i != last_keyword + 2)
+                error('w', "keyword separated from other keyword arguments");
+            last_keyword = i;
+            arg = QuoteExp.getInstance(cdr_car, this);
+            arg.setFlag(QuoteExp.IS_KEYWORD);
+        }
+        else {
+            if (cdr_car instanceof Pair
+                && ((Pair) cdr_car).getCar() == LispLanguage.splice_sym) {
+                arg = rewrite_car((Pair) ((Pair) cdr_car).getCdr(), false);
+                arg = new ApplyExp(MakeSplice.quoteInstance, arg);
+                if (firstSpliceArg < 0)
+                    firstSpliceArg = i + (applyFunction != null ? 1 : 0);
+            }
+            else 
+                arg = rewrite_car (cdr_pair, false);
+        }
         i++;
 
-        if (mapKeywordsToAttributes)
-          {
-            if ((i & 1) == 0) // Previous iteration was a keyword
-              {
-                Expression[] aargs = new Expression[2];
-                aargs[0] = (Expression) vec.pop();
-                aargs[1] = arg;
-                arg = new ApplyExp(gnu.kawa.xml.MakeAttribute.makeAttribute, aargs);
-              }
-            else
-              {
-                Object value;
-                if (arg instanceof QuoteExp
-                    && (value = ((QuoteExp) arg).getValue()) instanceof Keyword
-                    && i < cdr_length)
-                  arg = new QuoteExp(((Keyword) value).asSymbol());
-                else
-                  mapKeywordsToAttributes = false;
-              }
-          }
-
         vec.addElement(arg);
 	cdr = cdr_pair.getCdr();
+        popPositionOf(save_pos);
       }
+
+
     Expression[] args = new Expression[vec.size()];
     vec.copyInto(args);
 
     if (save_scope != current_scope)
       setPopCurrentScope(save_scope);
 
-    if (func instanceof ReferenceExp
-        && (((ReferenceExp) func).getBinding()==getNamedPartDecl))
+    if (isNamedPartDecl)
         return rewrite_lookup(args[0], args[1], function);
 
-    return makeApply(func, args);
+    ApplyExp app = new ApplyExp(func, args);
+    app.firstSpliceArg = firstSpliceArg;
+    if (first_keyword >= 0)
+      {
+        app.numKeywordArgs = (last_keyword - first_keyword) / 2 + 1;
+        app.firstKeywordArgIndex = first_keyword + (applyFunction != null ? 2 : 1);
+      }
+    return app;
   }
 
     public Expression rewrite_lookup(Expression part1, Expression part2, boolean function) {
@@ -438,10 +502,11 @@ public class Translator extends Compilation
         return CompileNamedPart.makeExp(part1, part2);
     }
 
-  public ApplyExp makeApply (Expression func, Expression[] args)
-  { 
-    return new ApplyExp(func, args);
-  }
+    /** A language-dependent "apply" function for generic application.
+     */
+    public Expression applyFunction(Expression func) {
+        return null;
+    }
 
   public Namespace namespaceResolvePrefix (Expression context)
   {
@@ -519,7 +584,7 @@ public class Translator extends Compilation
    * Returns Integer.MIN_VALUE for cyclic lists.
    * For impure lists returns the negative of one more than
    * the number of pairs before the "dot".
-   * Similar to LList.listLength, but descends into SyntaxForm. */
+   * Similar to LList.listLength, but handles SyntaxForm more efficiently. */
   public static int listLength(Object obj)
   {
     // Based on list-length implementation in
@@ -569,6 +634,18 @@ public class Translator extends Compilation
 	    setPopCurrentScope(save_scope);
 	  }
       }
+    else if (exp instanceof ValuesFromLList)
+      {
+          // Optimization of following case.
+          // More importantly, we make use of the line number information.
+          for (Object vs = ((ValuesFromLList) exp).values;
+               vs != LList.Empty; )
+          {
+              Pair p = (Pair) vs;
+              pushForm(rewrite_car(p, false));
+              vs = p.getCdr();
+          }
+      }
     else if (exp instanceof Values)
       {
 	Object[] vals = ((Values) exp).getValues();
@@ -576,7 +653,7 @@ public class Translator extends Compilation
 	  rewriteInBody(vals[i]);
       }
     else
-      formStack.add(rewrite(exp, false));
+      pushForm(rewrite(exp, false));
   }
 
     public Object namespaceResolve(Object name) {
@@ -676,17 +753,17 @@ public class Translator extends Compilation
             dname = null;
             scope = null;
           }
-        for (;scope != null; scope = scope.outer)
+        for (;scope != null; scope = scope.getOuter())
           {
             if (scope instanceof LambdaExp
-                && scope.outer instanceof ClassExp // redundant? FIXME
+                && scope.getOuter() instanceof ClassExp // redundant? FIXME
                 && ((LambdaExp) scope).isClassMethod()
                 && mode != 'M')
               {
-                if (decl_nesting >= ScopeExp.nesting(scope.outer))
+                if (decl_nesting >= ScopeExp.nesting(scope.getOuter()))
                   break;
                 LambdaExp caller = (LambdaExp) scope;
-                ClassExp cexp = (ClassExp) scope.outer;
+                ClassExp cexp = (ClassExp) scope.getOuter();
                 ClassType ctype = (ClassType) cexp.getClassType();
                 // BUG: lookupMember doesn't work if ctype
                 // is a class that hasn't been compiled yet,
@@ -712,7 +789,7 @@ public class Translator extends Compilation
                 Expression part1;
                 // FIXME We're throwing away 'part', which is wasteful.
                 if (contextStatic)
-                  part1 = new ReferenceExp(((ClassExp) caller.outer).nameDecl);
+                  part1 = new ReferenceExp(((ClassExp) caller.getOuter()).nameDecl);
                 else
                   part1 = new ThisExp(caller.firstDecl());
                 return CompileNamedPart.makeExp(part1,
@@ -754,9 +831,9 @@ public class Translator extends Compilation
                   {
                     if (scope == null)
                       throw new Error("internal error: missing "+decl);
-                    if (scope.outer == decl.context) // I.e. same class.
+                    if (scope.getOuter() == decl.context) // I.e. same class.
                       break;
-                    scope = scope.outer;
+                    scope = scope.getOuter();
                   }
                 cdecl = scope.firstDecl();
               }
@@ -808,7 +885,7 @@ public class Translator extends Compilation
                     else
                       decl = null;
                   }
-                catch (Throwable ex)
+                catch (Exception ex)
                   {
                     error('e',
                           "exception loading '" + exp
@@ -863,7 +940,12 @@ public class Translator extends Compilation
     else if (exp == Special.nativeSpecial)
       return QuoteExp.nativeExp;
     else
-      return QuoteExp.getInstance(Quote.quote(exp, this), this);
+      {
+        if (exp instanceof Keyword && ! keywordsAreSelfEvaluating())
+          error('w', "keyword should be quoted if not in argument position");
+
+        return QuoteExp.getInstance(Quote.quote(exp, this), this);
+      }
   }
 
   /** 
@@ -912,7 +994,7 @@ public class Translator extends Compilation
 
     char ch0 = name.charAt(0);
 
-    if (ch0 == '@')
+    if (ch0 == '@') // Deprecated - reader now returns ($splice$ ATYPE).
     {
       String rest = name.substring(1);
       Expression classRef = tr.rewrite(Symbol.valueOf(rest));
@@ -1154,7 +1236,7 @@ public class Translator extends Compilation
     } catch (NoClassDefFoundError ex)
     {
       tr.error('w', "error loading class " + cname + " - " + ex.getMessage() + " not found");
-    } catch (Throwable ex)
+    } catch (Exception ex)
     {
     }
     return null;
@@ -1189,13 +1271,17 @@ public class Translator extends Compilation
   /** Note current line number position from a PairWithPosition.
    * Return an object to pass to popPositionOf.
    */
-  public Object pushPositionOf(Object pair)
+  public Object pushPositionOf(Object pos)
   {
-    if (pair instanceof SyntaxForm)
-      pair = ((SyntaxForm) pair).getDatum();
-    if (! (pair instanceof PairWithPosition))
+    if (pos instanceof SyntaxForm)
+      pos = ((SyntaxForm) pos).getDatum();
+    PairWithPosition pair;
+    if (pos instanceof PairWithPosition)
+        pair = (PairWithPosition) pos;
+    else if (pos instanceof SourceLocator)
+        pair = new PairWithPosition((SourceLocator) pos, null, null);
+    else
       return null;
-    PairWithPosition ppair = (PairWithPosition) pair;
     Object saved;
     if (positionPair == null
 	|| positionPair.getFileName() != getFileName()
@@ -1206,8 +1292,8 @@ public class Translator extends Compilation
       }
     else
       saved = positionPair;
-    setLine(pair);
-    positionPair = ppair;
+    setLine(pos);
+    positionPair = pair;
     return saved;
   }
 
@@ -1230,6 +1316,11 @@ public class Translator extends Compilation
         popPositionOf(save);
     }
 
+    public void errorIfNonEmpty(Object form) {
+        if (form != LList.Empty)
+            error('e', "invalid improper (dotted) list");
+    }
+
   /** Set the line position of the argument to the current position. */
 
   public void setLineOf (Expression exp)
@@ -1267,6 +1358,10 @@ public class Translator extends Compilation
                 else if (t instanceof Type)
                   type = (Type) t;
               }
+            catch (Error ex)
+              {
+                throw ex;
+              }
             catch (Throwable ex)
               {
               }
@@ -1319,23 +1414,14 @@ public class Translator extends Compilation
       return SyntaxForms.fromDatumIfNeeded(form, syntax);
   }
 
-  public Object popForms (int first)
+  /** Pop from formStack all forms that come after beforeFirst.
+   */
+  public Values popForms(Pair beforeFirst)
   {
-    int last = formStack.size();
-    if (last == first)
+    Object tail = formStack.popTail(beforeFirst);
+    if (tail == LList.Empty)
       return Values.empty;
-    Object r;
-    if (last == first + 1)
-      r = formStack.elementAt(first);
-    else
-      {
-	Values vals = new Values();
-	for (int i = first; i < last;  i++)
-	  vals.writeObject(formStack.elementAt(i));
-	r = vals;
-      }
-    formStack.setSize(first);
-    return r;
+    return new ValuesFromLList((LList) tail);
   }
 
   public void scanForm (Object st, ScopeExp defs)
@@ -1346,9 +1432,9 @@ public class Translator extends Compilation
 	ScopeExp save_scope = setPushCurrentScope(sf.getScope());
 	try
 	  {
-	    int first = formStack.size();
+	    Pair beforeFirst = formStack.last;
 	    scanForm(sf.getDatum(), defs);
-	    formStack.add(wrapSyntax(popForms(first), sf));
+	    pushForm(wrapSyntax(popForms(beforeFirst), sf));
 	    return;
 	  }
 	finally
@@ -1360,6 +1446,18 @@ public class Translator extends Compilation
       {
 	if (st == Values.empty)
 	  st = QuoteExp.voidExp; // From #!void
+        else if (st instanceof ValuesFromLList)
+        {
+          for (Object vs = ((ValuesFromLList) st).values;
+               vs != LList.Empty; )
+            {
+              Pair p = (Pair) vs;
+              Object save = pushPositionOf(p);
+              scanForm(p.getCar(), defs);
+              popPositionOf(save);
+              vs = p.getCdr();
+            }
+        }
 	else
 	  {
 	    Object[] vals = ((Values) st).getValues();
@@ -1404,7 +1502,7 @@ public class Translator extends Compilation
                         if (obj instanceof Syntax)
                           syntax = (Syntax) obj;
                       }
-                    catch (Throwable ex)
+                    catch (Exception ex)
                       {
                         obj = null;
                       }
@@ -1412,7 +1510,6 @@ public class Translator extends Compilation
                 else
                   obj = namespaceResolve(part1, part2);
               }
-            
             if (obj instanceof Symbol && ! selfEvaluatingSymbol(obj))
               {
                 Expression func = rewrite(obj, 'M');
@@ -1432,7 +1529,8 @@ public class Translator extends Compilation
             // Recognize deferred begin created in scanBody for pendingForms.
             // A seemingly-cleaner (obj instanceof Syntax) causes problems
             // with some Syntax forms, such as define.
-            else if (obj == kawa.standard.begin.begin)
+            else if (obj == kawa.standard.begin.begin
+                     || obj == kawa.standard.define_library.define_library_scan)
               syntax = (Syntax) obj;
           }
         finally
@@ -1459,7 +1557,7 @@ public class Translator extends Compilation
 	      }
 	  }
       }
-    formStack.add(st);
+    pushForm(st);
   }
 
   /** Recursive helper method for rewrite_body.
@@ -1483,7 +1581,8 @@ public class Translator extends Compilation
 	    ScopeExp save_scope = setPushCurrentScope(sf.getScope());
 	    try
 	      {
-		int first = formStack.size();
+		Pair first = formStack.last;
+
 		LList f = scanBody(sf.getDatum(), defs, makeList);
                 if (makeList)
                   {
@@ -1493,7 +1592,7 @@ public class Translator extends Compilation
                     lastPair.setCdrBackdoor(f);
                     return list;
                   }
-		formStack.add(wrapSyntax(popForms(first), sf));
+		pushForm(wrapSyntax(popForms(first), sf));
 		return null;
 	      }
 	    finally
@@ -1504,8 +1603,10 @@ public class Translator extends Compilation
 	else if (body instanceof Pair)
 	  {
 	    Pair pair = (Pair) body;
-	    int first = formStack.size();
+	    Pair first = formStack.last;
+            Object savePos = pushPositionOf(pair);
 	    scanForm(pair.getCar(), defs);
+            popPositionOf(savePos);
             if (getState() == Compilation.PROLOG_PARSED)
               {
                 // We've seen a require form during the initial pass when
@@ -1514,30 +1615,26 @@ public class Translator extends Compilation
                 if (pair.getCar() != pendingForm)
                   pair = makePair(pair, pendingForm, pair.getCdr());
                 pendingForm = new Pair(kawa.standard.begin.begin, pair);
-                if (list != LList.Empty)
-                  formStack.add(new Pair(kawa.standard.begin.begin, list));
+                if (makeList)
+                  formStack.pushAll(list);
                 return LList.Empty;
               }
-	    int fsize = formStack.size();
 	    if (makeList)
 	      {
-		for (int i = first;  i < fsize;  i++)
-		  {
-		    Pair npair
-		      = makePair(pair, formStack.elementAt(i), LList.Empty);
-		    if (lastPair == null)
-		      list = npair;
-		    else
-		      lastPair.setCdrBackdoor(npair);
-		    lastPair = npair;
-		  }
-		formStack.setSize(first);
+                Pair last = formStack.lastPair();
+                LList nlist = (LList) formStack.popTail(first);
+                if (lastPair == null)
+		  list = nlist;
+		else
+		  lastPair.setCdrBackdoor(nlist);
+                if (last != first)
+                  lastPair = last;
 	      }
 	    body = pair.getCdr();
 	  }
 	else
 	  {
-	    formStack.add(syntaxError ("body is not a proper list"));
+	    pushForm(syntaxError("body is not a proper list"));
 	    break;
 	  }
       }
@@ -1561,14 +1658,16 @@ public class Translator extends Compilation
     // This is confusing, at the least.  FIXME.
     Object saved = pushPositionOf(exp);
     LetExp defs = new LetExp();
-    int first = formStack.size();
-    defs.outer = current_scope;
+    int renamedAliasOldSize =
+        renamedAliasStack == null ? 0 : renamedAliasStack.size();
+    Pair first = formStack.last;
+    defs.setOuter(current_scope);
     current_scope = defs;
     try
       {
         LList list = scanBody(exp, defs, true);
 	if (list.isEmpty())
-	  formStack.add(syntaxError ("body with no expressions"));
+	  pushForm(syntaxError("body with no expressions"));
         int ndecls = 0;
         for (Declaration decl = defs.firstDecl(); decl != null; decl = decl.nextDecl())
           {
@@ -1579,6 +1678,9 @@ public class Translator extends Compilation
               }
           }
         rewriteBody(list);
+        int renamedAliasNewSize =
+            renamedAliasStack == null ? 0 : renamedAliasStack.size();
+        popRenamedAlias((renamedAliasNewSize - renamedAliasOldSize) >> 1);
 	Expression body = makeBody(first, null);
 	setLineOf(body);
 	if (ndecls == 0)
@@ -1613,22 +1715,22 @@ public class Translator extends Compilation
   }
 
   /** Combine a list of zero or more expression forms into a "body". */
-  protected Expression makeBody(int first, ScopeExp scope)
+  protected Expression makeBody(Pair head, ScopeExp scope)
   {
-    int nforms = formStack.size() - first;
+    Object tail = formStack.popTail(head);
+    int nforms = LList.length(tail);
     if (nforms == 0)
-      return QuoteExp.voidExp; 
-    else if (nforms == 1)
+      return QuoteExp.voidExp;
+    Pair first = (Pair) tail;
+     if (nforms == 1)
       {
-	return (Expression) formStack.pop();
+	return (Expression) first.getCar();
       }
     else
       {
 	Expression[] exps = new Expression[nforms];
-	for (int i = 0; i < nforms; i++)
-	  exps[i] = (Expression) formStack.elementAt(first + i);
-	formStack.setSize(first);
-	if (scope instanceof ModuleExp)
+        first.toArray(exps);
+        if (scope instanceof ModuleExp)
 	  return new ApplyExp(gnu.kawa.functions.AppendValues.appendValues,
 			      exps);
 	else
@@ -1740,47 +1842,36 @@ public class Translator extends Compilation
         mexp.setFlag(false, ModuleExp.USE_DEFINED_CLASS);
   }
 
-  static void vectorReverse (Vector vec, int start, int count)
-  {
-    // See http://www.azillionmonkeys.com/qed/case8.html
-    int j = count / 2;
-    int last = start + count - 1;
-    for (int i = 0; i < j; i++)
-      {
-        Object tmp = vec.elementAt(start + i);
-        vec.setElementAt(vec.elementAt(last - i), start+i);
-        vec.setElementAt(tmp, last - i);
-      }
-  }
-
   public void resolveModule(ModuleExp mexp)
   {
+    setLine(null, -1, -1);
     int numPending = pendingImports == null ? 0 : pendingImports.size();
     for (int i = 0;  i < numPending;  )
       {
         ModuleInfo info = (ModuleInfo) pendingImports.elementAt(i++);
         ScopeExp defs = (ScopeExp) pendingImports.elementAt(i++);
         Expression posExp = (Expression) pendingImports.elementAt(i++);
-        Integer savedSize = (Integer) pendingImports.elementAt(i++);
+        Pair beforeGoal = (Pair) pendingImports.elementAt(i++);
         if (mexp == defs)
           {
             // process(BODY_PARSED);
             Expression savePos = new ReferenceExp((Object) null);
             savePos.setLine(this);
             setLine(posExp);
-            int beforeSize = formStack.size();
+            Pair beforeImports = formStack.last;
             kawa.standard.require.importDefinitions(null, info, null,
                                                     formStack, defs, this);
-            int desiredPosition = savedSize.intValue();
-            if (savedSize != beforeSize)
+            if (beforeGoal != beforeImports
+                && beforeImports != formStack.last)
               {
-                // Move the forms resulting from the import to desiredPosition:
-                int curSize = formStack.size();
-                int count = curSize - desiredPosition;
-                // See http://www.azillionmonkeys.com/qed/case8.html
-                vectorReverse(formStack, desiredPosition, beforeSize-desiredPosition);
-                vectorReverse(formStack, beforeSize, curSize - beforeSize);
-                vectorReverse(formStack, desiredPosition, count);
+                // Move forms derived from the import forwards in the list,
+                // just following beforeGoal.
+                Object firstGoal = beforeGoal.getCdr();
+                Object firstImports = beforeImports.getCdr();
+                beforeGoal.setCdrBackdoor(firstImports);
+                formStack.last.setCdrBackdoor(firstGoal);
+                beforeImports.setCdrBackdoor(LList.Empty);
+                formStack.last = beforeImports;
               }
             setLine(savePos);
           }
@@ -1793,11 +1884,21 @@ public class Translator extends Compilation
     Compilation save_comp = Compilation.setSaveCurrent(this);
     try
       {
-        rewriteInBody(popForms(firstForm));
+        Pair firstForm = formStack.getHead();
+        rewriteBody((LList) formStack.popTail(firstForm));
 	mexp.body = makeBody(firstForm, mexp);
         // In immediate mode need to preserve Declaration for current "session".
         if (! immediate)
 	  lexical.pop(mexp);
+
+        // Patch up renamed exports - see export.
+        for (Declaration decl = mexp.firstDecl();  decl != null;
+             decl = decl.nextDecl()) {
+            if (decl.getSymbol() == null
+                && decl.getFlag(Declaration.EXPORT_SPECIFIED)) {
+                decl.patchSymbolFromSet();
+            }
+        }
       }
     finally
       {
@@ -1880,23 +1981,114 @@ public class Translator extends Compilation
       }
   }
 
-  public Declaration define (Object name, SyntaxForm nameSyntax, ScopeExp defs)
-  {
-    boolean aliasNeeded = nameSyntax != null && nameSyntax.getScope() != currentScope();
-    Object declName = aliasNeeded ? new String(name.toString()) : name;
-    Declaration decl = defs.getDefine(declName, 'w', this);
-    if (aliasNeeded)
-      {
-	Declaration alias = makeRenamedAlias(name, decl, nameSyntax.getScope());
-        nameSyntax.getScope().addDeclaration(alias);
-      }
-    push(decl);
-    return decl;
-  }
+    public Declaration define(Object name, SyntaxForm nameSyntax,
+                              ScopeExp defs) {
+        ScopeExp scope = nameSyntax != null ? nameSyntax.getScope()
+            : currentScope();
+        boolean aliasNeeded = scope != defs;
+        Object declName = aliasNeeded
+            ? Symbol.makeUninterned(name.toString())
+            : name;
+        Declaration decl = defs.getDefine(declName, 'w', this);
+        if (aliasNeeded) {
+            Declaration alias = makeRenamedAlias(name, decl, scope);
+            if (defs instanceof LetExp)
+                pushRenamedAlias(alias);
+            else
+                scope.addDeclaration(alias);
+        }
+        push(decl);
+        return decl;
+    }
 
   static boolean isObjectSyntax (ClassType declaringClass, String fieldName)
   {
     return "objectSyntax".equals(fieldName)
       &&  "kawa.standard.object".equals(declaringClass.getName());
   }
+
+    public FormStack formStack = new FormStack(this);
+    public void pushForm(Object value) { formStack.push(value); }
+  
+    /** A list of "forms" to be further processed.
+     * It is implemented as an LList so we can save position information.
+     */
+    public static class FormStack extends Pair {
+        private Pair last = this;
+        SourceLocator sloc;
+    
+        public FormStack(SourceLocator sloc) {
+            this.sloc = sloc;
+            this.cdr = LList.Empty;
+        }
+
+        /** Return the "head" of the list.
+         * The cdr of the head is the first element.
+         */
+        public Pair getHead() { return this; }
+        public Object getFirst() { return cdr; }
+        /** The Pair whose car is the last form in the list.
+         * If the list is empty, this returns the list head.
+         */
+        @Override
+        public Pair lastPair() { return last; }
+
+        /* DEBUGGING:
+        public void dump() {
+            int i=0;
+            System.err.println("formStack len:"+LList.length(getFirst()));
+            for(Object x = getFirst(); x instanceof Pair; i++) {
+                Pair p = (Pair) x;
+                //if (! (p.getCar() instanceof SetExp))
+                    System.err.println("- #"+i+": "+p.getCar());
+                x = p.getCdr();
+            }
+        }
+        */
+
+        public Object popTail(Pair oldTail) {
+            Object r = oldTail.getCdr();
+            oldTail.setCdrBackdoor(LList.Empty);
+            last = oldTail;
+            return r;
+        }
+    
+        public void push(Object value) {
+            Pair pair = new PairWithPosition(sloc, value, LList.Empty);
+            last.setCdrBackdoor(pair);
+            last = pair;
+        }
+
+        public void pushAll(LList values) {
+            if (values == LList.Empty)
+                return;
+            last.setCdrBackdoor(values);
+            last = ((Pair) values).lastPair();
+        }
+    
+        public void pushAll(LList values, Pair valuesLast) {
+            if (values == LList.Empty)
+                return;
+            last.setCdrBackdoor(values);
+            last = valuesLast;
+        }
+    
+        public void pushAfter(Object value, Pair position) {
+            Pair pair = new PairWithPosition(sloc, value, position.getCdr());
+            position.setCdrBackdoor(pair);
+            if(last == position)
+                last = pair;
+        }
+    }
+
+    /** An implementationof Values using a linked list.
+     */
+    public static class ValuesFromLList extends Values.FromList<Object> {
+        public LList values;
+
+        public ValuesFromLList(LList values) {
+            super(values);
+            this.values = values;
+        }
+    }
 }
