@@ -2,6 +2,12 @@
 (require <kawa.lib.std_syntax>)
 (require <kawa.lib.syntax>)
 (require <kawa.lib.misc>)
+(define-alias Double java.lang.Double)
+(define-alias IntNum gnu.math.IntNum)
+(define-alias Numeric gnu.math.Numeric)
+(define-alias RatNum gnu.math.RatNum)
+(define-alias RealNum gnu.math.RealNum)
+(define-alias LangObjType gnu.kawa.lispexpr.LangObjType)
 
 (define-private (java.lang.real? x) ::boolean
   (and (java.lang.Number? x)
@@ -32,19 +38,41 @@
 	       (java.lang.Byte? x)
 	       (java.math.BigInteger? x)
 	       (java.math.BigDecimal? x)))))
+
 (define (integer? x) :: <boolean>
   (or (instance? x <gnu.math.IntNum>)
-      (and (instance? x <gnu.math.DFloNum>)
-	   (= (java.lang.Math:IEEEremainder
-	       (gnu.math.DFloNum:doubleValue x)
-	       1.0)
-	      0.0))
+      (and (instance? x <java.lang.Number>)
+           (cond ((or (java.lang.Long? x)
+                      (java.lang.Integer? x)
+                      (java.lang.Short? x)
+                      (java.lang.Byte? x)
+                      (java.math.BigInteger? x))
+                  #t)
+                 ((or (gnu.math.DFloNum? x)
+                      (java.lang.Float? x)
+                      (java.lang.Double? x))
+                  (= (java.lang.Math:IEEEremainder
+                      (java.lang.Number:doubleValue x)
+                      1.0)
+                     0.0))
+                 ((java.math.BigDecimal? x)
+                  (try-catch
+                   (begin
+                     ((->java.math.BigDecimal x):toBigIntegerExact)
+                     #t)
+                   (ex java.lang.ArithmeticException #f)))
+                 (else
+                  #f)))))
+
+(define (exact-integer? x) :: <boolean>
+  (or (instance? x <gnu.math.IntNum>)
       (and (instance? x <java.lang.Number>)
 	   (or (instance? x <java.lang.Long>)
 	       (instance? x <java.lang.Integer>)
 	       (instance? x <java.lang.Short>)
 	       (instance? x <java.lang.Byte>)
 	       (instance? x <java.math.BigInteger>)))))
+
 (define (real-valued? x) ::boolean
   (and (complex? x) (zero? (imag-part x)) (real? (real-part x))))
 (define (rational-valued? x) ::boolean
@@ -124,6 +152,16 @@
 	(else
 	 (- x))))
 
+(define (floor/ (x :: real) (y :: real))
+  (let* ((q (floor-quotient x y))
+	 (r (- x (* q y))))
+    (values q r)))
+
+(define (truncate/ (x :: real) (y :: real))
+  (let* ((q (truncate-quotient x y))
+	 (r (- x (* q y))))
+    (values q r)))
+
 (define (div-and-mod (x :: real) (y :: real))
   (let* ((q (div x y))
 	 (r (- x (* q y))))
@@ -134,29 +172,49 @@
 	 (r (- x (* q y))))
     (values q r)))
 
-(define (gcd #!rest (args ::integer[])) :: integer
-  (let ((n args:length))
-    (if (= n 0)
-	0
-	(let ((result ::integer (args 0)))
-	  (do ((i ::int 1 (+ i 1)))
-	      ((>= i n) result)
-	    (set! result (gnu.math.IntNum:gcd result (args i))))))))
+(define (gcd #!rest (args ::real[])) ::real
+  (let ((n args:length)
+        (result-inexact ::boolean #f)
+        (result ::integer 0))
+    (do ((i ::int 0 (+ i 1)))
+        ((>= i n)
+         (if result-inexact (inexact result) result))
+      (let* ((val ::real (args i))
+             (cur-inexact (inexact? val))
+             (cur ::integer
+                  (if cur-inexact
+                      (begin (set! result-inexact #t)
+                             (exact val))
+                      val)))
+        (set! result (if (= i 0) cur
+                         (gnu.math.IntNum:gcd result cur)))))))
 
-(define (lcm #!rest (args ::integer[])) :: <integer>
-  (let ((n args:length))
-    (if (= n 0)
-	1
-	(let ((result ::integer (gnu.math.IntNum:abs (args 0))))
-	  (do ((i ::int 1 (+ i 1)))
-	      ((>= i n) result)
-	    (set! result (gnu.math.IntNum:lcm result (args i))))))))
+(define (lcm #!rest (args ::real[])) ::real
+  (let ((n args:length)
+        (result-inexact ::boolean #f)
+        (result ::integer 1))
+    (do ((i ::int 0 (+ i 1)))
+        ((>= i n)
+         (if result-inexact (inexact result) result))
+      (let* ((val ::real (args i))
+             (cur-inexact (inexact? val))
+             (cur ::integer
+                  (if cur-inexact
+                      (begin (set! result-inexact #t)
+                             (exact val))
+                      val)))
+        (set! result (if (= i 0) cur
+                         (gnu.math.IntNum:lcm result cur)))))))
 
-(define (numerator (x :: <rational>)) :: <integer>
-  (x:numerator))
+(define (numerator (x ::real)) ::real
+  (if (RatNum? x)
+      ((->RatNum x):numerator)
+      (inexact ((LangObjType:coerceRatNum (exact x)):numerator))))
 
-(define (denominator (x :: <rational>)) :: <integer>
-  (x:denominator))
+(define (denominator (x ::real)) ::real
+  (if (RatNum? x)
+      ((->RatNum x):denominator)
+      (inexact ((LangObjType:coerceRatNum (exact x)):denominator))))
 
 (define (floor (x :: real)) :: real
   (x:toInt gnu.math.Numeric:FLOOR))
@@ -207,12 +265,15 @@
 	  (java.lang.Math:atan x)))
 
 (define-procedure sqrt
+#|
+  ;; Only when the "required type" is real:
+  (lambda ((x :: <double>)) :: <double>
+	  (java.lang.Math:sqrt x))
+|#
   (lambda ((num :: <quantity>)) :: <quantity>
 	  (gnu.math.Quantity:make
 	   (invoke (invoke num 'number) 'sqrt)
-	   (invoke (invoke num 'unit) 'sqrt)))
-  (lambda ((x :: <double>)) :: <double>
-	  (java.lang.Math:sqrt x)))
+	   (invoke (invoke num 'unit) 'sqrt))))
 
 (define (square x::quantity) ::quantity
   (* x x))
@@ -239,7 +300,7 @@
 (define (angle (x ::java.lang.Number)):: <real>
   (if (gnu.math.Complex? x)
       ((->gnu.math.Complex x):angle)
-      0))
+      (if (< (x:doubleValue) 0) java.lang.Math:PI 0)))
 
 (define (inexact (num :: java.lang.Number)) :: java.lang.Number
   (gnu.kawa.functions.Arithmetic:toInexact num))
@@ -318,9 +379,9 @@
   (make <gnu.lists.FString>
     (gnu.kawa.functions.Arithmetic:toString arg radix)))
 
-(define (string->number (str :: <string>) #!optional (radix :: <int> 10))
+(define (string->number (str :: <string>) #!optional (radix ::int 10))
   ::object
-  (let ((result (gnu.kawa.lispexpr.LispReader:parseNumber str radix)))
+  (let ((result (gnu.kawa.lispexpr.LispReader:parseNumber str (- radix))))
     (if (instance? result <gnu.math.Numeric>) result #f)))
 
 (define (quantity->number (q :: <quantity>)) :: <complex>
@@ -345,41 +406,24 @@
 (define (duration duration) :: <gnu.math.Duration>
   (gnu.math.Duration:parseDuration duration))
 
-#| The algorithm of exact-integer-sqrt is from chibi-scheme,
-which has:
-Copyright (c) 2009-2012 Alex Shinn
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-1. Redistributions of source code must retain the above copyright
-   notice, this list of conditions and the following disclaimer.
-2. Redistributions in binary form must reproduce the above copyright
-   notice, this list of conditions and the following disclaimer in the
-   documentation and/or other materials provided with the distribution.
-3. The name of the author may not be used to endorse or promote products
-   derived from this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
-IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
-INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
-THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-|#
-
 (define (exact-integer-sqrt (i ::integer))
   (if (< i 0)
       (primitive-throw (java.lang.IllegalArgumentException
-                        (format #f "negative argument: ~A" i)))
-      (let ((res (sqrt i)))
-        (let lp ((res ::integer (inexact->exact (truncate res))))
-          (let ((rem ::integer (- i (* res res))))
-            (if (negative? rem)
-                (lp (quotient (+ res (quotient i res)) 2))
-                (values res rem)))))))
+                        (format #f "negative argument: ~A" i))))
+  (let* ((dval ::double (i:doubleValue))
+         (init ::integer ;; initial approximation
+               (if (Double:isInfinite dval) 
+                   (IntNum:shift 1 (i:intLength))
+                   (RealNum:toExactInt (java.lang.Math:sqrt dval)
+                                       Numeric:TRUNCATE))))
+    ;; "Babylonian method"
+    (let loop ((q ::integer init)) ;; q is current approximation
+      (let ((rem (- i (* q q))))
+        ;; Continue if rem<0 or (q+1)^2<=i
+        (if (or (< rem 0)
+                ;; Because rem=i-q^2, we can simplify (q+1)^2<=i
+                ;; to q^2+2*q+1<=q^2+rem or 2*q+1<=rem
+                (<= (IntNum:shift q 1) (- rem 1)))
+            ;; Next q is average of old q and i/q
+            (loop (IntNum:shift (+ q (IntNum:quotient i q)) -1))
+            (values q rem))))))
