@@ -5,13 +5,19 @@ import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.text.SourceMessages;
 import gnu.text.SyntaxException;
-import gnu.text.Path;
 import gnu.lists.*;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import gnu.bytecode.ClassType;
 import gnu.kawa.servlet.HttpRequestContext;
+import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.WriterManager;
+import gnu.kawa.util.ExitCalled;
+import kawa.lang.SyntaxPattern;
 
 /** Start a "Read-Eval-Print-Loop" for the Kawa Scheme evaluator. */
 
@@ -170,6 +176,10 @@ public class repl extends Procedure0or1
             Environment env = Environment.getCurrent();
             Shell.runFile(fs, path, env, true, 0);
           }
+        catch (Error e)
+          {
+            throw e;
+          }
         catch (Throwable e)
           {
             return "An error occurred while loading '" + initFile +"' : " + e;
@@ -207,7 +217,7 @@ public class repl extends Procedure0or1
   }
 
   static boolean shutdownRegistered
-    = gnu.text.WriterManager.instance.registerShutdownHook();
+    = WriterManager.instance.registerShutdownHook();
 
   public static int processArgs(String[] args, int iArg, int maxArg)
   {
@@ -262,7 +272,7 @@ public class repl extends Procedure0or1
                   {
                     skipLines = Integer.parseInt(count);
                   }
-                catch (Throwable ex)
+                catch (Exception ex)
                   {
                     iArg = maxArg; // force bad_option.
                   }
@@ -389,9 +399,11 @@ public class repl extends Procedure0or1
 	    if (iArg == maxArg)
 	      bad_option (arg);
             arg = args[iArg];
-            if (arg.equals("7"))
+            if (arg.equals("8") || arg.equals("1.8"))
+              Compilation.defaultClassFileVersion = ClassType.JDK_1_8_VERSION;
+            else if (arg.equals("7") || arg.equals("1.7"))
               Compilation.defaultClassFileVersion = ClassType.JDK_1_7_VERSION;
-            if (arg.equals("6") || arg.equals("1.6"))
+            else if (arg.equals("6") || arg.equals("1.6"))
               Compilation.defaultClassFileVersion = ClassType.JDK_1_6_VERSION;
             else if (arg.equals("5") || arg.equals("1.5"))
               Compilation.defaultClassFileVersion = ClassType.JDK_1_5_VERSION;
@@ -420,6 +432,10 @@ public class repl extends Procedure0or1
 	      bad_option (arg);
 	    compilationTopname = args[iArg];
 	  }
+        else if (arg.equals ("--main"))
+          {
+	    defaultParseOptions |= Language.PARSE_EMIT_MAIN;
+          }
 	else if (arg.equals ("-C"))
 	  {
 	    ++iArg;
@@ -594,6 +610,10 @@ public class repl extends Procedure0or1
 	  {
 	    Compilation.debugPrintFinalExpr = true;
 	  }
+	else if (arg.equals("--debug-syntax-pattern-match"))
+	  {
+	    SyntaxPattern.printSyntaxPatternMatch = true;
+	  }
 	else if (arg.equals("--debug-error-prints-stack-trace"))
 	  {
             SourceMessages.debugStackTraceOnError = true;
@@ -662,7 +682,7 @@ public class repl extends Procedure0or1
 	    System.out.print("Kawa ");
 	    System.out.print(Version.getVersion());
 	    System.out.println();
-	    System.out.println("Copyright (C) 2013 Per Bothner");
+	    System.out.println("Copyright (C) 2014 Per Bothner");
 	    something_done = true;
 	  }
 	else if (arg.length () > 0 && arg.charAt(0) == '-')
@@ -752,7 +772,8 @@ public class repl extends Procedure0or1
             InPort fstream;
             try
               {
-                fstream = InPort.openFile(arg);
+                Path path = Path.valueOf(arg);
+                fstream = Shell.openFile(path.openInputStream(), path);
               }
             catch (java.io.FileNotFoundException ex)
               {
@@ -777,9 +798,8 @@ public class repl extends Procedure0or1
                                defaultParseOptions, minfo);
             infos[i-iArg] = minfo;
             comps[i-iArg] = comp;
-
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             if (! (ex instanceof SyntaxException)
                 || ((SyntaxException) ex).getMessages() != messages)
@@ -812,7 +832,7 @@ public class repl extends Procedure0or1
             if (sawErrors)
               System.exit(-1);
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             internalError(ex, comp, arg);
           }
@@ -822,7 +842,7 @@ public class repl extends Procedure0or1
   static void internalError (Throwable ex, Compilation comp, Object arg)
   {
     try { comp.getMessages().checkErrors(System.err, 50); }
-    catch (Throwable e) { }
+    catch (Exception e) { }
     StringBuffer sbuf = new StringBuffer();
     if (comp != null)
       {
@@ -847,6 +867,7 @@ public class repl extends Procedure0or1
   {
     try
       {
+        ExitCalled.push();
 	int iArg = processArgs(args, 0, args.length);
 	if (iArg < 0)
 	  return;
@@ -880,9 +901,10 @@ public class repl extends Procedure0or1
 	if (! shutdownRegistered)
 	  {
 	    // Redundant if registerShutdownHook succeeded (e.g on JDK 1.3).
-	    gnu.mapping.OutPort.runCleanups();
+	    OutPort.runCleanups();
 	  }
 	ModuleBody.exitDecrement();
+        ExitCalled.pop();
       }
   }
 
@@ -899,7 +921,7 @@ public class repl extends Procedure0or1
              .invoke(new Object[0])) == null)
           return true;
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
       }
     return false;
