@@ -9,6 +9,8 @@ import gnu.text.SourceMessages;
 import gnu.kawa.lispexpr.*;
 import gnu.lists.AbstractFormat;
 import gnu.kawa.functions.*;
+import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.InPort;
 import gnu.kawa.reflect.LazyType;
 import gnu.kawa.servlet.HttpRequestContext;
 
@@ -20,7 +22,7 @@ public class Scheme extends LispLanguage
   public static final Environment r6Environment;
   protected static final SimpleEnvironment kawaEnvironment;
 
-  public static LangPrimType booleanType;
+  public static final LangPrimType booleanType;
 
   public static final int FOLLOW_R5RS = 5;
   public static final int FOLLOW_R6RS = 6;
@@ -92,6 +94,8 @@ public class Scheme extends LispLanguage
 
     instance.initScheme();
 
+    booleanType = new LangPrimType(Type.booleanType, instance);
+            
     int withServlets = HttpRequestContext.importServletDefinitions;
     if (withServlets > 0)
       {
@@ -100,7 +104,7 @@ public class Scheme extends LispLanguage
             instance.loadClass(withServlets > 1 ? "gnu.kawa.servlet.servlets"
                                : "gnu.kawa.servlet.HTTP");
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
           }
       }
@@ -153,6 +157,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("lambda", "kawa.standard.SchemeCompilation", "lambda");
       defSntxStFld("$bracket-apply$", "gnu.kawa.lispexpr.BracketApply", "instance");
       defSntxStFld("$string$", "kawa.lib.syntax");
+      defSntxStFld("$string-with-default-format$", "kawa.lib.syntax");
       defSntxStFld("$format$", "kawa.lib.syntax");
       defSntxStFld("$sprintf$", "kawa.lib.syntax");
       defSntxStFld("define-simple-constructor", "kawa.lib.syntax");
@@ -169,7 +174,6 @@ public class Scheme extends LispLanguage
 
       //-- Section 4.1  -- complete
       defSntxStFld(LispLanguage.quote_str, "kawa.lang.Quote", "plainQuote");
-      defSntxStFld("%define", "kawa.standard.define", "defineRaw");
       defSntxStFld("define", "kawa.lib.prim_syntax");
 
       defSntxStFld("if", "kawa.lib.prim_syntax");
@@ -177,17 +181,24 @@ public class Scheme extends LispLanguage
 
       // Section 4.2  -- complete
       defSntxStFld("cond", "kawa.lib.std_syntax");
-      defSntxStFld("case", "kawa.lib.std_syntax");
+      defSntxStFld("...", "kawa.lib.std_syntax");
+      defSntxStFld("_", "kawa.lib.std_syntax");
+      defSntxStFld("=>", "kawa.lib.std_syntax");
+      defSntxStFld("else", "kawa.lib.std_syntax");
+      defSntxStFld("unquote", "kawa.lib.std_syntax");
+      defSntxStFld("unquote-splicing", "kawa.lib.std_syntax");
+      defSntxStFld("case", "kawa.lib.case_syntax");
       defSntxStFld("and", "kawa.lib.std_syntax");
       defSntxStFld("or", "kawa.lib.std_syntax");
-      defSntxStFld("%let", "kawa.standard.let", "let");
       defSntxStFld("let", "kawa.lib.std_syntax");
       defSntxStFld("let*", "kawa.lib.std_syntax");
       defSntxStFld("letrec", "kawa.lib.prim_syntax");
+      defSntxStFld("letrec*", "kawa.lib.prim_syntax", "letrec");
 
       defSntxStFld("begin", "kawa.standard.begin", "begin");
       defSntxStFld("do", "kawa.lib.std_syntax");
       defSntxStFld("lazy", "kawa.lib.std_syntax");
+      defSntxStFld("delay-force", "kawa.lib.std_syntax");
       defSntxStFld("delay", "kawa.lib.std_syntax");
       defSntxStFld("quasiquote", "kawa.lang.Quote", "quasiQuote");
 
@@ -289,6 +300,7 @@ public class Scheme extends LispLanguage
       defProcStFld("real?", "kawa.lib.numbers");
       defProcStFld("rational?", "kawa.lib.numbers");
       defProcStFld("integer?", "kawa.lib.numbers");
+      defProcStFld("exact-integer?", "kawa.lib.numbers");
       defProcStFld("exact?", "kawa.lib.numbers");
       defProcStFld("inexact?", "kawa.lib.numbers");
       defProcStFld("=", "kawa.standard.Scheme", "numEqu");
@@ -309,12 +321,19 @@ public class Scheme extends LispLanguage
       defProcStFld("/", "gnu.kawa.functions.DivideOp", "$Sl");
       defProcStFld("abs", "kawa.lib.numbers");
       defProcStFld("quotient", "gnu.kawa.functions.DivideOp", "quotient");
+      defProcStFld("truncate-quotient", "gnu.kawa.functions.DivideOp", "quotient");
       defProcStFld("remainder", "gnu.kawa.functions.DivideOp", "remainder");
+      defProcStFld("truncate-remainder", "gnu.kawa.functions.DivideOp", "remainder");
       defProcStFld("modulo", "gnu.kawa.functions.DivideOp", "modulo");
+      defProcStFld("floor-quotient",
+                   "gnu.kawa.functions.DivideOp", "floorQuotient");
+      defProcStFld("floor-remainder", "gnu.kawa.functions.DivideOp", "modulo");
       defProcStFld("div", "gnu.kawa.functions.DivideOp", "div");
       defProcStFld("mod", "gnu.kawa.functions.DivideOp", "mod");
       defProcStFld("div0", "gnu.kawa.functions.DivideOp", "div0");
       defProcStFld("mod0", "gnu.kawa.functions.DivideOp", "mod0");
+      defProcStFld("floor/", "kawa.lib.numbers");
+      defProcStFld("truncate/", "kawa.lib.numbers");
       defProcStFld("div-and-mod", "kawa.lib.numbers");
       defProcStFld("div0-and-mod0", "kawa.lib.numbers");
       defProcStFld("gcd", "kawa.lib.numbers");
@@ -402,7 +421,8 @@ public class Scheme extends LispLanguage
 
       defProcStFld("substring", "kawa.lib.strings");
       defProcStFld("string-append", "kawa.lib.strings");
-      defProcStFld("string-append/shared", "kawa.lib.strings");
+      defProcStFld("string-append!", "kawa.lib.strings");
+      defProcStFld("string-replace!", "kawa.lib.strings");
       defProcStFld("string->list", "kawa.lib.strings");
       defProcStFld("list->string", "kawa.lib.strings");
       defProcStFld("string-copy", "kawa.lib.strings");
@@ -438,6 +458,8 @@ public class Scheme extends LispLanguage
       defProcStFld("force", "kawa.lib.misc");
       defProcStFld("force*", "kawa.lib.misc");
       defProcStFld("eager", "kawa.lib.misc");
+      defProcStFld("promise?", "kawa.lib.misc");
+      defProcStFld("make-promise", "kawa.lib.misc");
       defProcStFld("promise-set-value!", "kawa.lib.misc");
       defProcStFld("promise-set-alias!", "kawa.lib.misc");
       defProcStFld("promise-set-exception!", "kawa.lib.misc");
@@ -466,8 +488,8 @@ public class Scheme extends LispLanguage
       defProcStFld("close-output-port", "kawa.lib.ports");
       defProcStFld("read", "kawa.lib.ports");
       defProcStFld("read-line", "kawa.lib.ports");
-      defProcStFld("read-char", "kawa.standard.readchar", "readChar");
-      defProcStFld("peek-char", "kawa.standard.readchar", "peekChar");
+      defProcStFld("read-char", "kawa.lib.ports");
+      defProcStFld("peek-char", "kawa.lib.ports");
       defProcStFld("eof-object?", "kawa.lib.ports");
       defProcStFld("eof-object", "kawa.lib.ports");
       defProcStFld("read-string", "kawa.lib.ports");
@@ -516,9 +538,9 @@ public class Scheme extends LispLanguage
       defProcStFld("set-input-port-prompter!", "kawa.lib.ports");
       defProcStFld("base-uri", "kawa.lib.misc");
 
-      defProcStFld("%syntax-error", "kawa.standard.syntax_error",
+      defProcStFld("syntax-error", "kawa.standard.syntax_error",
                    "syntax_error");
-      defProcStFld("syntax-error", "kawa.lib.prim_syntax");
+      defProcStFld("report-syntax-error", "kawa.lib.prim_syntax");
 
       r4Environment.setLocked();
       environ = r5Environment;
@@ -528,11 +550,13 @@ public class Scheme extends LispLanguage
 		   "callWithValues");
       defSntxStFld("let-values", "kawa.lib.syntax");
       defSntxStFld("let*-values", "kawa.lib.syntax");
+      defSntxStFld("define-values", "kawa.lib.syntax");
       defSntxStFld("case-lambda", "kawa.lib.syntax");
       defSntxStFld("receive", "kawa.lib.syntax");
-      defProcStFld("eval", "kawa.lib.std_syntax");
+      defProcStFld("eval", "kawa.lib.scheme.eval");
       defProcStFld("repl", "kawa.standard.SchemeCompilation", "repl");
       defProcStFld("scheme-report-environment", "kawa.lib.misc");
+      defProcStFld("environment", "kawa.lib.scheme.eval");
       defProcStFld("null-environment", "kawa.lib.misc");
       defProcStFld("interaction-environment", "kawa.lib.misc");
       defProcStFld("dynamic-wind", "kawa.lib.misc");
@@ -542,7 +566,9 @@ public class Scheme extends LispLanguage
 
       defProcStFld("vector-map", "kawa.lib.vectors");
       defProcStFld("vector-for-each", "kawa.lib.vectors");
-      defProcStFld("string-for-each", "gnu.kawa.slib.srfi13");
+      defProcStFld("string-map", "kawa.lib.strings");
+      defProcStFld("srfi-13-string-for-each", "kawa.lib.strings");
+      defProcStFld("string-for-each", "kawa.lib.strings");
       defProcStFld("real-valued?", "kawa.lib.numbers");
       defProcStFld("rational-valued?", "kawa.lib.numbers");
       defProcStFld("integer-valued?", "kawa.lib.numbers");
@@ -558,6 +584,7 @@ public class Scheme extends LispLanguage
  
       defSntxStFld("define-private", "kawa.lib.prim_syntax");
       defSntxStFld("define-constant", "kawa.lib.prim_syntax");
+      defSntxStFld("define-early-constant", "kawa.lib.prim_syntax");
 
       defSntxStFld("define-autoload",
                    "kawa.standard.define_autoload", "define_autoload");
@@ -566,6 +593,7 @@ public class Scheme extends LispLanguage
                    "define_autoloads_from_file");
 
       defProcStFld("exit", "kawa.lib.rnrs.programs");
+      defProcStFld("emergency-exit", "kawa.lib.rnrs.programs");
       defProcStFld("command-line", "kawa.lib.rnrs.programs");
       defAliasStFld("command-line-arguments", 
                     "gnu.expr.ApplicationMainSupport", "commandLineArguments");
@@ -591,8 +619,6 @@ public class Scheme extends LispLanguage
       defProcStFld("lognot", "gnu.kawa.functions.BitwiseOp", "not");
       defProcStFld("logop", "kawa.lib.numbers");
       defProcStFld("bitwise-bit-set?", "kawa.lib.numbers");
-      defProcStFld("logbit?", "kawa.lib.numbers",
-                   Language.mangleNameIfNeeded("bitwise-bit-set?"));
       defProcStFld("logtest", "kawa.lib.numbers");
       defProcStFld("bitwise-bit-count", "kawa.lib.numbers");
       defProcStFld("logcount", "kawa.lib.numbers");
@@ -637,9 +663,18 @@ public class Scheme extends LispLanguage
       defProcStFld("primitive-throw", "gnu.kawa.reflect.Throw", "primitiveThrow");
       defSntxStFld("try-finally", "kawa.lib.syntax");
       defSntxStFld("try-catch", "kawa.lib.prim_syntax");
-      defProcStFld("throw", "kawa.lib.misc");
-      defProcStFld("catch", "kawa.lib.system");
-      defProcStFld("error", "kawa.lib.misc");
+      defProcStFld("throw", "kawa.lib.exceptions");
+      defProcStFld("catch", "kawa.lib.exceptions");
+      defProcStFld("error", "kawa.lib.exceptions");
+      defProcStFld("error-object?", "kawa.lib.exceptions");
+      defProcStFld("error-object-message", "kawa.lib.exceptions");
+      defProcStFld("error-object-irritants", "kawa.lib.exceptions");
+      defProcStFld("file-error?", "kawa.lib.exceptions");
+      defProcStFld("read-error?", "kawa.lib.exceptions");
+      defProcStFld("raise", "kawa.lib.exceptions");
+      defProcStFld("raise-continuable", "kawa.lib.exceptions");
+      defProcStFld("with-exception-handler", "kawa.lib.exceptions");
+      defSntxStFld("guard", "kawa.lib.exceptions");
       defProcStFld("as", "gnu.kawa.functions.Convert", "as");
       defProcStFld("instance?", "kawa.standard.Scheme", "instanceOf");
       defSntxStFld("synchronized", "kawa.lib.syntax");
@@ -687,10 +722,12 @@ public class Scheme extends LispLanguage
       defProcStFld("syntax-source", "kawa.lib.std_syntax");
       defProcStFld("syntax-line", "kawa.lib.std_syntax");
       defProcStFld("syntax-column", "kawa.lib.std_syntax");
-      defSntxStFld("begin-for-syntax", "kawa.lib.std_syntax");
-      defSntxStFld("define-for-syntax", "kawa.lib.std_syntax");
-      defSntxStFld("include", "kawa.lib.misc_syntax");
-      defSntxStFld("include-relative", "kawa.lib.misc_syntax");
+      defSntxStFld("begin-for-syntax", "kawa.lib.syntax");
+      defSntxStFld("define-for-syntax", "kawa.lib.syntax");
+      defSntxStFld("include", "kawa.standard.Include", "include");
+      defSntxStFld("include-relative", "kawa.standard.Include",
+                   "includeRelative");
+      defSntxStFld("include-ci", "kawa.standard.Include", "includeCi");
 
       defProcStFld("file-exists?", "kawa.lib.files");
       defProcStFld("file-directory?", "kawa.lib.files");
@@ -708,12 +745,23 @@ public class Scheme extends LispLanguage
 
       defProcStFld("system", "kawa.lib.system");
       defProcStFld("make-process", "kawa.lib.system");
+      defProcStFld("pipe-process", "kawa.lib.system");
+      defProcStFld("run-process", "gnu.kawa.functions.RunProcess", "instance");
+      defProcStFld("process-exit-wait", "kawa.lib.system");
+      defProcStFld("process-exit-ok?", "kawa.lib.system");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol("cmd"),
+                   "kawa.lib.system", "cmd");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol("`"),
+                   "kawa.lib.system", "cmd");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol("sh"),
+                   "kawa.lib.system", "sh");
       defProcStFld("tokenize-string-to-string-array", "kawa.lib.system");
       defProcStFld("tokenize-string-using-shell", "kawa.lib.system");
       defProcStFld("command-parse", "kawa.lib.system");
       defProcStFld("process-command-line-assignments", "kawa.lib.system");
       defProcStFld("get-environment-variable", "kawa.lib.system");
       defProcStFld("get-environment-variables", "kawa.lib.system");
+      defProcStFld("current-path", "kawa.lib.ports");
       defProcStFld("current-second", "kawa.lib.system");
       defProcStFld("current-jiffy", "kawa.lib.system");
       defProcStFld("jiffies-per-second", "kawa.lib.system");
@@ -728,7 +776,7 @@ public class Scheme extends LispLanguage
       defProcStFld("record-type-name", "kawa.lib.reflection");
       defProcStFld("record-type-field-names", "kawa.lib.reflection");
       defProcStFld("record?", "kawa.lib.reflection");
-      defSntxStFld("define-record-type", "gnu.kawa.slib.DefineRecordType");
+      defSntxStFld("define-record-type", "kawa.lib.DefineRecordType");
 
       defSntxStFld("when", "kawa.lib.syntax"); //-- (when cond exp ...)
       defSntxStFld("unless", "kawa.lib.syntax"); //-- (unless cond exp ...)
@@ -736,7 +784,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("constant-fold", "kawa.standard.constant_fold",
                    "constant_fold");
       defProcStFld("make-parameter", "kawa.lib.parameters");
-      defSntxStFld("parameterize", "kawa.lib.parameters");
+      defSntxStFld("parameterize", "kawa.lib.parameterize");
 
       defProcStFld("compile-file", "kawa.lib.system");
       defProcStFld("environment-bound?", "kawa.lib.misc");
@@ -794,6 +842,8 @@ public class Scheme extends LispLanguage
       defProcStFld("keyword->string", "kawa.lib.keywords");
       defProcStFld("string->keyword", "kawa.lib.keywords");
       defSntxStFld("location", "kawa.standard.location", "location");
+      defSntxStFld("define-library", "kawa.standard.define_library",
+                   "define_library");
       defSntxStFld("define-alias", "kawa.standard.define_alias",
                    "define_alias");
       defSntxStFld("define-private-alias", "kawa.standard.define_alias",
@@ -803,7 +853,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("define-member-alias", "kawa.standard.define_member_alias",
                    "define_member_alias");
       defSntxStFld("define-enum", "gnu.kawa.slib.enums");
-      defSntxStFld("import", "kawa.lib.syntax");
+      defSntxStFld("import", "kawa.standard.ImportFromLibrary", "instance");
       defSntxStFld("require", "kawa.standard.require", "require");
       defSntxStFld("module-name", "kawa.standard.module_name",
                    "module_name");
@@ -849,6 +899,7 @@ public class Scheme extends LispLanguage
 
       defAliasStFld("bytevector", "gnu.kawa.lispexpr.LangObjType",
                     "u8vectorType");
+      defProcStFld("bytevector?", "kawa.lib.bytevectors");
       defProcStFld("make-bytevector", "kawa.lib.bytevectors");
       defProcStFld("bytevector-length", "kawa.lib.bytevectors");
       defProcStFld("bytevector-u8-ref", "kawa.lib.bytevectors");
@@ -862,18 +913,17 @@ public class Scheme extends LispLanguage
       defSntxStFld("cut", "gnu.kawa.slib.cut");
       defSntxStFld("cute", "gnu.kawa.slib.cut");
 
-      defSntxStFld("cond-expand", "kawa.lib.syntax");
-      defSntxStFld("%cond-expand", "kawa.lib.syntax");
+      defSntxStFld("cond-expand", "kawa.standard.IfFeature", "condExpand");
 
       defAliasStFld("*print-base*", "gnu.kawa.functions.DisplayFormat",
                     "outBase");
       defAliasStFld("*print-radix*", "gnu.kawa.functions.DisplayFormat",
                     "outRadix");
       defAliasStFld("*print-right-margin*",
-                    "gnu.text.PrettyWriter", "lineLengthLoc");
+                    "gnu.kawa.io.PrettyWriter", "lineLengthLoc");
       defAliasStFld("*print-miser-width*",
-                    "gnu.text.PrettyWriter", "miserWidthLoc");
-      defAliasStFld("*print-circle*", "gnu.text.PrettyWriter", "isSharing");
+                    "gnu.kawa.io.PrettyWriter", "miserWidthLoc");
+      defAliasStFld("*print-circle*", "gnu.kawa.io.PrettyWriter", "isSharing");
       defAliasStFld("*print-xml-indent*",
                     "gnu.xml.XMLPrinter", "indentLoc");
       defAliasStFld("html", "gnu.kawa.xml.XmlNamespace", "HTML");
@@ -907,6 +957,16 @@ public class Scheme extends LispLanguage
       defProcStFld("path-extension", "kawa.lib.files");
       defProcStFld("path-fragment", "kawa.lib.files");
       defProcStFld("path-query", "kawa.lib.files");
+      defProcStFld("path-bytes", "kawa.lib.files");
+      defProcStFld("path-data", "kawa.lib.files");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol("PD"),
+                   "kawa.lib.files", "PD");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol("<"),
+                   "kawa.lib.files", "PD");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol(">"),
+                   "kawa.lib.files", "set_PD");
+      defProcStFld(LispLanguage.constructNamespace.getSymbol(">>"),
+                   "kawa.lib.files", "append_PD");
 
       defProcStFld("annotation", "gnu.kawa.reflect.MakeAnnotation", "instance");
 
@@ -978,17 +1038,9 @@ public class Scheme extends LispLanguage
 	throw new RuntimeException("eval: I/O exception: "
 				   + e.toString ());
       }
-    catch (RuntimeException ex)
-      {
-	throw ex;
-      }
-    catch (Error ex)
-      {
-	throw ex;
-      }
     catch (Throwable ex)
       {
-	throw new WrappedException(ex);
+	throw WrappedException.rethrow(ex);
       }
   }
 
@@ -1002,17 +1054,9 @@ public class Scheme extends LispLanguage
       {
 	return Eval.eval (sexpr, env);
       }
-    catch (RuntimeException ex)
-      {
-	throw ex;
-      }
-    catch (Error ex)
-      {
-	throw ex;
-      }
     catch (Throwable ex)
       {
-	throw new WrappedException(ex);
+	throw WrappedException.rethrow(ex);
       }
   }
 
@@ -1051,8 +1095,6 @@ public class Scheme extends LispLanguage
     @Override
     protected synchronized HashMap<String, Type> getTypeMap() {
         if (types == null) {
-            booleanType = new LangPrimType(Type.booleanType,
-                                           Scheme.getInstance());
             types = new HashMap<String, Type>(128); // Bit more wiggle room
             types.put("boolean", booleanType);
             types.putAll(super.getTypeMap());
@@ -1065,33 +1107,24 @@ public class Scheme extends LispLanguage
         return types;
     }
 
-  public String formatType (Type type)
-  {
-    // FIXME synchronize
-    if (type instanceof LazyType)
-      {
-        LazyType ltype = (LazyType) type;
-        return formatType(ltype.getRawType())
-          +'['+formatType(ltype.getValueType())+']';
-      }
-    if (typeToStringMap == null)
-      {
-        typeToStringMap = new HashMap<Type,String>();
-        for (java.util.Map.Entry<String,Type> e : getTypeMap().entrySet())
-          {
-            String s = e.getKey();
-            Type t = e.getValue();
-            typeToStringMap.put(t, s);
-            Type it = t.getImplementationType();
-            if (it != t)
-              typeToStringMap.put(it, s);
-          }
-      }
-    String str = typeToStringMap.get(type);
-    if (str != null)
-      return str;
-    return super.formatType(type);
-  }
+    public String formatType(Type type) {
+        // FIXME synchronize
+        if (type instanceof LazyType) {
+            LazyType ltype = (LazyType) type;
+            return formatType(ltype.getRawType())
+                +'['+formatType(ltype.getValueType())+']';
+        }
+        if (typeToStringMap == null)  {
+            typeToStringMap = new HashMap<Type,String>();
+            // Invert the map returned by getTypeMap.
+            for (java.util.Map.Entry<String,Type> e : getTypeMap().entrySet())
+                typeToStringMap.put(e.getValue(), e.getKey());
+        }
+        String str = typeToStringMap.get(type);
+        if (str != null)
+            return str;
+        return super.formatType(type);
+    }
 
  /** Convert expression to a Type.
    * Allow "TYPE" or 'TYPE or <TYPE>.
@@ -1111,6 +1144,9 @@ public class Scheme extends LispLanguage
    */
   public boolean appendBodyValues () { return false; }
 
+    @Override
+    public boolean keywordsAreSelfEvaluating() { return false; }
+
   public ReadTable createReadTable ()
   {
     ReadTable tab = ReadTable.createInitial();
@@ -1133,6 +1169,8 @@ public class Scheme extends LispLanguage
       {
         tab.postfixLookupOperator = ':';
         tab.setFinalColonIsKeyword(true);
+        tab.set('@', new ReaderQuote(LispLanguage.splice_sym,
+                                     ReadTable.NON_TERMINATING_MACRO));
       }
     return tab;
   }
