@@ -14,20 +14,31 @@ import java.util.Stack;
 
 public class let extends Syntax
 {
-  public static final let let = new let();
-  static { let.setName("let"); }
+  public static final let let = new let("let", false);
+  
+  /**
+   * Used for constructs such as FLET, where we intend to set a function binding
+   * rather than an ordinary binding.
+   */
+  protected boolean settingProcedures;
+  
+  public let(String name, boolean settingProcedures) {
+    this.setName(name);
+    this.settingProcedures = settingProcedures;
+  }
 
+  @Override
   public Expression rewrite (Object obj, Translator tr)
   {
     if (! (obj instanceof Pair))
-      return tr.syntaxError ("missing let arguments");
+      return tr.syntaxError ("missing " + getName() + " arguments");
     Pair pair = (Pair) obj;
     Object bindings = pair.getCar();
     Object body = pair.getCdr();
     int decl_count = Translator.listLength(bindings);
     if (decl_count < 0)
       return tr.syntaxError("bindings not a proper list");
-      
+    
     Expression[] inits = new Expression[decl_count];
     Declaration[] decls = new Declaration[decl_count];
     // Used to check for duplicate definitions.
@@ -53,7 +64,8 @@ public class let extends Syntax
 	    bind_pair_car = syntax.getDatum();
 	  }
 	if (! (bind_pair_car instanceof Pair))
-	  return tr.syntaxError ("let binding is not a pair:"+bind_pair_car);
+	  return tr.syntaxError (getName() + 
+                                 " binding is not a pair:"+bind_pair_car);
 	Pair binding = (Pair) bind_pair_car;
 	Object name = binding.getCar();
 	TemplateScope templateScope;
@@ -67,7 +79,8 @@ public class let extends Syntax
 	  templateScope = syntax == null ? null : syntax.getScope();
         name = tr.namespaceResolve(name);
 	if (! (name instanceof Symbol))
-	  return tr.syntaxError("variable "+name+" in let binding is not a symbol: "+obj);
+	  return tr.syntaxError("variable "+name+" in " + getName() + 
+                                " binding is not a symbol: "+obj);
 	Declaration decl = new Declaration(name);
         Translator.setLine(decl, binding);
         Symbol sym = (Symbol) name;
@@ -77,6 +90,7 @@ public class let extends Syntax
         dupenv.put(sym, templateScope, decl);
         decls[i] = decl;
         decl.setFlag(Declaration.IS_SINGLE_VALUE);
+        maybeSetProcedure(decl);
 	if (templateScope != null)
 	  {
 	    Declaration alias = tr.makeRenamedAlias(decl, templateScope);
@@ -93,7 +107,7 @@ public class let extends Syntax
 	    binding_cdr = syntax.getDatum();
 	  }
 	if (! (binding_cdr instanceof Pair))
-	  return tr.syntaxError("let has no value for '"+name+"'");
+	  return tr.syntaxError(getName() + " has no value for '"+name+"'");
 	binding = (Pair) binding_cdr;
 	binding_cdr = binding.getCdr();
 	Pair init;
@@ -106,7 +120,7 @@ public class let extends Syntax
 	  {
 	    if (! (binding_cdr instanceof Pair)
 		|| (binding = (Pair) binding_cdr).getCdr() == LList.Empty)
-	      return tr.syntaxError("missing type after '::' in let");
+	      return tr.syntaxError("missing type after '::' in " + getName());
 	    binding_cdr = binding.getCdr();
 	    while (binding_cdr instanceof SyntaxForm)
 	      {
@@ -125,10 +139,11 @@ public class let extends Syntax
 	    init = (Pair) binding_cdr;
 	  }
 	else
-	  return tr.syntaxError("let binding for '"+name+"' is improper list");
+	  return tr.syntaxError(getName() + " binding for '" + getName() +
+                                "' is improper list");
         inits[i] = tr.rewrite_car (init, syntax);
 	if (init.getCdr() != LList.Empty)
-	  return tr.syntaxError("junk after declaration of "+name);
+	  return tr.syntaxError("junk after declaration of "+getName());
 	bindings = bind_pair.getCdr();
       }
 
@@ -141,7 +156,21 @@ public class let extends Syntax
     tr.letEnter();
     LetExp let = tr.letDone(tr.rewrite_body(body));
     tr.popRenamedAlias(renamedAliasesCount);
-
+    
     return let;
   }
+
+  /**
+   * Set the procedure flag of a declaration if binding a function property.
+   * 
+   * This is used for FLET .vs. LET distinction, where {@code settingProcedures}
+   * is true for FLET, and false for LET.
+   * 
+   * @param decl The declaration to possibly set the {@code PROCEDURE} flag.
+   */
+  protected void maybeSetProcedure (Declaration decl)
+  {
+    if (settingProcedures)
+      decl.setProcedureDecl(true); 
+  }
 }
