@@ -1,4 +1,6 @@
-(test-begin "libs" 255)
+;; -*- coding: utf-8 -*-
+
+(test-begin "libs" 257)
 
 (test-begin "vectors")
 (test-equal '(dah dah didah)
@@ -198,7 +200,7 @@
       (expand '(let loop () (loop))))
 (test-equal '(let ((x #!undefined)) (set x 10))
       (expand '(define x 10)))
-(test-equal '(as <java.lang.String> (quote a))
+(test-equal '(as <String> (quote a))
       (expand '(as String 'a)))
 
 (test-equal '(lambda (a b c) #f)
@@ -492,6 +494,27 @@
 
 (test-end "rnrs-lists")
 
+(test-begin "strings")
+(import (kawa string-cursors))
+
+(define str1 "a😂b😼c")
+(define sc1e::string-cursor (string-cursor-end str1))
+(define str1lst '())
+(do ((sc::string-cursor (string-cursor-start str1)
+                        (string-cursor-next str1 sc)))
+    ((string-cursor>=? sc sc1e))
+  (set! str1lst (cons (as int (string-cursor-ref str1 sc)) str1lst)))
+(test-equal '(97 128514 98 128572 99) (reverse str1lst))
+
+(define str2lst '())
+(string-cursor-for-each
+ (lambda (x) (set! str2lst (cons (char->integer x) str2lst)))
+ str1 (as string-cursor 3))
+(test-equal '(98 128572 99) (reverse str2lst))
+;; FIXME more
+
+(test-end)
+
 (test-begin "char-sets" 89)
 
 (import (srfi :14 char-sets))
@@ -587,7 +610,7 @@
                  (lambda (c) (java.lang.Character:title-case?
                               (char->integer c))) char-set:full)))
 
-; Some of these tests only succeed on Java 7, which supports
+; Some of these tests only succeed on Java 7 (or later), which supports
 ; Unicode 6. On earlier Javas, the java.lang.Character predicates will
 ; disagree with the char-set definitions.
 (define-syntax expect-fail-unless-unicode-6
