@@ -1,4 +1,4 @@
-(test-init "macros" 124)
+(test-init "macros" 127)
 
 (test 'ok 'letxx (let ((xx #f)) (cond (#t xx 'ok))))
 
@@ -292,7 +292,7 @@
 (test 2 'mzscheme-lang-12.3.5-1 (def-and-use-of-x 2))
 (test 1 'mzscheme-lang-12.3.5-2 x1)
 
-;; From FLT MzScheme Manual section 12.3.5 Macro-Gnerated Top-Level
+;; From FLT MzScheme Manual section 12.3.5 Macro-Generated Top-Level
 (define-syntax def-and-use
   (syntax-rules ()
     ((def-and-use x val)
@@ -301,6 +301,8 @@
 (set! x 2)
 (test 3 'mzscheme-lang-12.3.5-3 (def-and-use x 3))
 (set! fail-expected "mzscheme-lang-12.3.5-4 is 2 but should be 3")
+;; Note this works if def-and-use uses set! instead of define.
+;; Probably chalk this up to Kawa's top-level define being different.
 (test 3 'mzscheme-lang-12.3.5-4 x)
 
 ;; Example from Chez Scheme User's Guide by Kent Dybvig:
@@ -796,3 +798,31 @@
 
 ;; Savannah bug report #39946 "NullPointerException when using syntax->datum"
 (test '(+ 1 2) 'savannah-39946 (syntax->datum (syntax (+ 1 2))))
+
+;; This example is in the Kawa internals documentation.
+(define-syntax mac1
+  (syntax-rules ()
+    ((mac1-nest v1 init exp)
+     (let ((v1 init))
+       (let ((i 2))
+         (list exp i))))))
+(define j 10)
+(test '(11 2) 'test-mac1 (mac1 i 1 (+ i j)))
+
+;; Savannah bug report #40616 "Unhygienic syntax-rules"
+(define-syntax def-a
+  (syntax-rules ()
+    ((_) (define a 'wrong))))
+(test 'correct 'savannah-40616
+      (let ((a 'correct)) (def-a) a))
+
+(begin
+  (define xlist '())
+  (define-syntax def-b
+    (syntax-rules ()
+      ((_ val)
+       (begin (define a val)
+              (set! xlist (cons a xlist))))))
+  (def-b 12)
+  (def-b 42)
+  (test '(42 12) 'savannah-40616-2 xlist))
