@@ -1,4 +1,4 @@
-(test-begin "numbers" 1872)
+(test-begin "num" 1880)
 
 (test-equal 7 (+ 3 4))
 (test-equal 3 (+ 3))
@@ -299,6 +299,14 @@
 (test-eqv #t (exact? 5))
 (test-eqv #t (inexact? +inf.0))
 
+(test-eqv #t (integer? (java.math.BigDecimal "345")))
+(test-eqv #f (integer? (java.math.BigDecimal "345.01")))
+(test-eqv #t (integer? (java.lang.Long "345")))
+(test-eqv #t (integer? (java.lang.Double "345")))
+(test-eqv #t (exact-integer? (java.lang.Short "345")))
+(test-eqv #f (exact-integer? (java.lang.Double "345")))
+(test-eqv #f (exact-integer? (java.math.BigDecimal "345")))
+
 (test-end "numerical operations")
 
 (test-begin "logop")
@@ -310,8 +318,8 @@
 (define (logop-compare result op x y)
   (do ((i 0 (+ i 1)))
       ((or (= i 100)
-	   (not (eq? (logop-bits op (logbit? x i) (logbit? y i))
-		     (logbit? result i))))
+	   (not (eq? (logop-bits op (bitwise-bit-set? x i) (bitwise-bit-set? y i))
+		     (bitwise-bit-set? result i))))
        i)
     #t))
 
@@ -453,4 +461,7 @@
 (test-assert (< 0 1/0))
 (test-assert (> 4.5 -1/0))
 
+(define (circle-area radius) (* java.lang.Math:PI (expt radius 2)))
+(test-approximate 28.27 (circle-area 3) 0.1)
+
 (test-end)
