@@ -38,14 +38,20 @@
                                       #'(test-equal exp val)
                                       #'rest1))))))))))
 
+(define-syntax test-values
+  (syntax-rules ()
+    ((_ expect expr)
+     (test (call-with-values (lambda () expect) (lambda results results))
+       (call-with-values (lambda () expr) (lambda results results))))))
+
 ;; R7RS test suite.  Covers all procedures and syntax in the small
 ;; language except `delete-file'.  Currently assumes full-unicode
 ;; support, the full numeric tower and all standard libraries
 ;; provided.
 ;;
 ;; Uses the (chibi test) library which is written in portable R7RS.
-;; This provides test-begin, test-end and test, which could be defined
-;; as something like:
+;; This is mostly a subset of SRFI-64, providing test-begin, test-end
+;; and test, which could be defined as something like:
 ;;
 ;;   (define (test-begin . o) #f)
 ;;
@@ -193,7 +199,6 @@
                     (even? (- n 1))))))
       (even? 88)))
 
-(skip-if-kawa "letrec* not implemented"
 (test 5
     (letrec* ((p
                (lambda (x)
@@ -206,7 +211,31 @@
               (x (p 5))
               (y x))
              y))
-)
+
+;; By Jussi Piitulainen <jpiitula@ling.helsinki.fi>
+;; and John Cowan <cowan@mercury.ccil.org>:
+;; http://lists.scheme-reports.org/pipermail/scheme-reports/2013-December/003876.html
+(define (means ton)
+  (letrec*
+     ((mean
+        (lambda (f g)
+          (f (/ (sum g ton) n))))
+      (sum
+        (lambda (g ton)
+          (if (null? ton)
+            (+)
+            (if (number? ton)
+                (g ton)
+                (+ (sum g (car ton))
+                   (sum g (cdr ton)))))))
+      (n (sum (lambda (x) 1) ton)))
+    (values (mean values values)
+            (mean exp log)
+            (mean / /))))
+(let*-values (((a b c) (means '(8 5 99 1 22))))
+  (test 27 a)
+  (test 9.728 b)
+  (test 1800/497 c))
 
 (let*-values (((root rem) (exact-integer-sqrt 32)))
   (test 35 (* root rem)))
@@ -231,6 +260,10 @@
     (let*-values (((root rem) (exact-integer-sqrt (expt 2 121))))
       (list root rem)))
 
+(test '(31622776601683793319 62545769258890964239)
+    (let*-values (((root rem) (exact-integer-sqrt (expt 10 39))))
+      (list root rem)))
+
 (let*-values (((root rem) (exact-integer-sqrt (expt 2 140))))
   (test 0 rem)
   (test (expt 2 140) (square root)))
@@ -287,7 +320,6 @@
 
 (test 2 (head (tail (tail integers))))
 
-(skip-if-kawa "delay-force not implemented"
 (define (stream-filter p? s)
   (delay-force
    (if (null? (force s)) 
@@ -310,7 +342,17 @@
                       (force p)))))
   (test 6 (force p))
   (test 6 (begin (set! x 10) (force p))))
-)
+
+(test #t (promise? (delay (+ 2 2))))
+(test #t (promise? (make-promise (+ 2 2))))
+(test #t
+    (let ((x (delay (+ 2 2))))
+      (force x)
+      (promise? x)))
+(test #t
+    (let ((x (make-promise (+ 2 2))))
+      (force x)
+      (promise? x)))
 
 (define radix
   (make-parameter
@@ -328,8 +370,8 @@
 (test '(list 3 4) `(list ,(+ 1 2) 4))
 (let ((name 'a)) (test '(list a (quote a)) `(list ,name ',name)))
 (test '(a 3 4 5 6 b) `(a ,(+ 1 2) ,@(map abs '(4 -5 6)) b))
-(cond (kawa (test #(10 5 2.0 4.0 3.0 8) `#(10 5 ,(sqrt 4) ,@(map sqrt '(16 9)) 8)))
-      (else (test #(10 5 2 4 3 8) `#(10 5 ,(sqrt 4) ,@(map sqrt '(16 9)) 8))))
+(test #(10 5 4 16 9 8)
+      `#(10 5 ,(square 2) ,@(map square '(4 3)) 8))
 (test '(a `(b ,(+ 1 2) ,(foo 4 d) e) f)
     `(a `(b ,(+ 1 2) ,(foo ,(+ 1 3) d) e) f) )
 (let ((name1 'x)
@@ -403,19 +445,86 @@
            (if y)
            y))))
 
-(cond-expand
- (kawa (begin))
- (else
-(define-syntax be-like-begin
+(define-syntax be-like-begin1
   (syntax-rules ()
-    ((be-like-begin name)
+    ((be-like-begin1 name)
      (define-syntax name
        (syntax-rules ()
          ((name expr (... ...))
           (begin expr (... ...))))))))
-(be-like-begin sequence)
-(test 4 (sequence 1 2 3 4))
-))
+(be-like-begin1 sequence1)
+(test 3 (sequence1 0 1 2 3))
+
+(define-syntax be-like-begin2
+  (syntax-rules ()
+    ((be-like-begin2 name)
+     (define-syntax name
+       (... (syntax-rules ()
+              ((name expr ...)
+               (begin expr ...))))))))
+(be-like-begin1 sequence2)
+(test 4 (sequence2 1 2 3 4))
+
+(define-syntax be-like-begin3
+  (syntax-rules ()
+    ((be-like-begin3 name)
+     (define-syntax name
+       (syntax-rules dots ()
+         ((name expr dots)
+          (begin expr dots)))))))
+(be-like-begin3 sequence3)
+(test 5 (sequence3 2 3 4 5))
+
+;; Syntax pattern with ellipsis in middle of proper list.
+(define-syntax part-2
+  (syntax-rules ()
+    ((_ a b (m n) ... x y)
+     (vector (list a b) (list m ...) (list n ...) (list x y)))
+    ((_ . rest) 'error)))
+(test '#((10 43) (31 41 51) (32 42 52) (63 77))
+      (part-2 10 (+ 21 22) (31 32) (41 42) (51 52) (+ 61 2) 77))
+;; Syntax pattern with ellipsis in middle of improper list.
+(define-syntax part-2x
+  (syntax-rules ()
+    ((_ a b (m n) ... x y . rest)
+     (vector (list a b) (list m ...) (list n ...) (list x y)
+             (cons "rest:" 'rest)))
+    ((_ . rest) 'error)))
+(test '#((10 43) (31 41 51) (32 42 52) (63 77) ("rest:"))
+      (part-2x 10 (+ 21 22) (31 32) (41 42) (51 52) (+ 61 2) 77))
+(test '#((10 43) (31 41 51) (32 42 52) (63 77) ("rest:" . "tail"))
+      (part-2x 10 (+ 21 22) (31 32) (41 42) (51 52) (+ 61 2) 77 . "tail"))
+
+;; underscore
+(define-syntax count-to-2
+  (syntax-rules ()
+    ((_) 0)
+    ((_ _) 1)
+    ((_ _ _) 2)
+    ((_ . _) 'many)))
+(test '(2 0 many)
+      (list (count-to-2 a b) (count-to-2) (count-to-2 a b c d)))
+
+(define-syntax count-to-2_
+  (syntax-rules (_)
+    ((_) 0)
+    ((_ _) 1)
+    ((_ _ _) 2)
+    ((x . y) 'fail)))
+(test '(2 0 fail fail)
+    (list (count-to-2_ _ _) (count-to-2_)
+          (count-to-2_ a b) (count-to-2_ a b c d)))
+
+(define-syntax jabberwocky
+  (syntax-rules ()
+    ((_ hatter)
+     (begin
+       (define march-hare 42)
+       (define-syntax hatter
+         (syntax-rules ()
+           ((_) march-hare)))))))
+(jabberwocky mad-hatter)
+(test 42 (mad-hatter))
 
 (test 'ok (let ((=> #f)) (cond (#t => 'ok))))
 
@@ -434,7 +543,6 @@
   (define bar (lambda (a b) (+ (* a b) a)))
   (foo (+ x 3))))
 
-(skip-if-kawa "define-values not implemented"
 (test 'ok
     (let ()
       (define-values () (values))
@@ -459,7 +567,6 @@
     (let ()
       (define-values (x y . z) (values 1 2 3 4))
       (+ x y (car z) (cadr z))))
-)
 
 (test '(2 1) (let ((x 1) (y 2))
   (define-syntax swap!
@@ -572,16 +679,13 @@
 (test #t (exact? #e3.0))
 (test #t (inexact? 3.))
 
-(skip-if-kawa "exact-integer? not implemented"
 (test #t (exact-integer? 32))
 (test #f (exact-integer? 32.0))
 (test #f (exact-integer? 32/5))
-)
+
 (test #t (finite? 3))
 (test #f (finite? +inf.0))
-(skip-if-kawa "3.0+inf.0i syntax not recognized"
 (test #f (finite? 3.0+inf.0i))
-)
 (test #f (infinite? 3))
 (test #t (infinite? +inf.0))
 (test #f (infinite? +nan.0))
@@ -602,6 +706,7 @@
 (test #f (<= 1 2 1))
 (test #t (>= 2 1 1))
 (test #f (>= 1 2 1))
+(test '(#t #f) (list (<= 1 1 2) (<= 2 1 3)))
 
 ;; From R7RS 6.2.6 Numerical operations:
 ;;
@@ -696,7 +801,6 @@
 (test 7 (abs -7))
 (test 7 (abs 7))
 
-(skip-if-kawa "floor/ etc not implemented"
 (test-values (values 2 1) (floor/ 5 2))
 (test-values (values -3 1) (floor/ -5 2))
 (test-values (values -3 -1) (floor/ 5 -2))
@@ -706,7 +810,6 @@
 (test-values (values -2 1) (truncate/ 5 -2))
 (test-values (values 2 -1) (truncate/ -5 -2))
 (test-values (values 2.0 -1.0) (truncate/ -5.0 -2))
-)
 
 (test 1 (modulo 13 4))
 (test 1 (remainder 13 4))
@@ -725,20 +828,16 @@
 (test 4 (gcd 32 -36))
 (test 0 (gcd))
 (test 288 (lcm 32 -36))
-(skip-if-kawa "lcm not implemented for inexacts"
 (test 288.0 (lcm 32.0 -36))
-)
 (test 1 (lcm))
 
 (test 3 (numerator (/ 6 4)))
 (test 2 (denominator (/ 6 4)))
-(skip-if-kawa "denominator not implemented for inexacts"
 (test 2.0 (denominator (inexact (/ 6 4))))
 (test 11.0 (numerator 5.5))
 (test 2.0 (denominator 5.5))
 (test 5.0 (numerator 5.0))
 (test 1.0 (denominator 5.0))
-)
 
 (test -5.0 (floor -4.3))
 (test -4.0 (ceiling -4.3))
@@ -756,20 +855,20 @@
 (test 1/3 (rationalize (exact .3) 1/10))
 (test #i1/3 (rationalize .3 1/10))
 
-(test 1.0 (exp 0))
+(test 1.0 (inexact (exp 0))) ;; may return exact number
 (test 20.0855369231877 (exp 3))
 
-(test 0.0 (log 1))
+(test 0.0 (inexact (log 1))) ;; may return exact number
 (test 1.0 (log (exp 1)))
 (test 42.0 (log (exp 42)))
 (test 2.0 (log 100 10))
 (test 12.0 (log 4096 2))
 
-(test 0.0 (sin 0))
+(test 0.0 (inexact (sin 0))) ;; may return exact number
 (test 1.0 (sin 1.5707963267949))
-(test 1.0 (cos 0))
+(test 1.0 (inexact (cos 0))) ;; may return exact number
 (test -1.0 (cos 3.14159265358979))
-(test 0.0 (tan 0))
+(test 0.0 (inexact (tan 0))) ;; may return exact number
 (test 1.5574077246549 (tan 1))
 
 (test 0.0 (asin 0))
@@ -792,9 +891,9 @@
 (test 1764 (square 42))
 (test 4 (square 2))
 
-(test 3.0 (sqrt 9))
+(test 3.0 (inexact (sqrt 9)))
 (test 1.4142135623731 (sqrt 2))
-(test +1.0i (sqrt -1))
+(test 0.0+1.0i (inexact (sqrt -1)))
 
 (test '(2 0) (call-with-values (lambda () (exact-integer-sqrt 4)) list))
 (test '(2 1) (call-with-values (lambda () (exact-integer-sqrt 5)) list))
@@ -824,6 +923,9 @@
 
 (test 100 (string->number "100"))
 (test 256 (string->number "100" 16))
+(test 100 (string->number "#d100" 16))
+(test 256 (string->number "#x100" 10))
+(test #f (string->number "#d#x100" 16))
 (test 100.0 (string->number "1e2"))
 
 (test-end)
@@ -849,6 +951,8 @@
 (test #t (boolean=? #t #t))
 (test #t (boolean=? #f #f))
 (test #f (boolean=? #t #f))
+(test #t (boolean=? #f #f #f))
+(test #f (boolean=? #t #t #f))
 
 (test-end)
 
@@ -964,6 +1068,8 @@
 
 (test #t (symbol=? 'a 'a))
 (test #f (symbol=? 'a 'A))
+(test #t (symbol=? 'a 'a 'a))
+(test #f (symbol=? 'a 'a 'A))
 
 (test "flying-fish"     
 (symbol->string 'flying-fish))
@@ -1094,6 +1200,11 @@
 
 (test "a-c" (let ((str (string #\a #\b #\c))) (string-set! str 1 #\-) str))
 
+(test (string #\a #\x1F700 #\c)
+    (let ((s (string #\a #\b #\c)))
+      (string-set! s 1 #\x1F700)
+      s))
+
 (test #t (string=? "" ""))
 (test #t (string=? "abc" "abc" "abc"))
 (test #f (string=? "" "abc"))
@@ -1172,6 +1283,7 @@
 (test "ßa" (string-downcase "ßa"))
 (test "ssa" (string-downcase "SSA"))
 (test "İ" (string-upcase "İ"))
+(test-expect-fail (cond-expand (java-8 1) (else 0)))
 (test "i̇" (string-downcase "İ"))
 (test "i̇" (string-foldcase "İ"))
 (test "J̌" (string-upcase "ǰ"))
@@ -1181,9 +1293,11 @@
 (test "γλώσσα" (string-downcase "ΓΛΏΣΣΑ"))
 (test "γλώσσα" (string-foldcase "ΓΛΏΣΣΑ"))
 (test "ΜΈΛΟΣ" (string-upcase "μέλος"))
-(test "μέλος" (string-downcase "ΜΈΛΟΣ"))
-(test "μέλος" (string-foldcase "ΜΈΛΟΣ"))
-(test "μέλος ενός" (string-downcase "ΜΈΛΟΣ ΕΝΌΣ"))
+(test #t (and (member (string-downcase "ΜΈΛΟΣ") '("μέλος" "μέλοσ")) #t))
+(test "μέλοσ" (string-foldcase "ΜΈΛΟΣ"))
+(test #t (and (member (string-downcase "ΜΈΛΟΣ ΕΝΌΣ")
+                      '("μέλος ενός" "μέλοσ ενόσ"))
+              #t))
 
 (test "" (substring "" 0 0))
 (test "" (substring "a" 0 0))
@@ -1437,7 +1551,6 @@
       (set-cdr! (cddr ls1) ls1)
       (map * ls1 ls2)))
 
-(skip-if-kawa "string-map not implemented"
 (test "abdegh" (string-map char-foldcase "AbdEgH"))
 
 (test "IBM" (string-map
@@ -1450,7 +1563,6 @@
      (lambda (c k) (if (eqv? k #\u) (char-upcase c) (char-downcase c)))
      "studlycaps xxx"
      "ululululul"))
-)
 
 (test #(b e h) (vector-map cadr '#((a b) (d e) (g h))))
 
@@ -1551,37 +1663,149 @@
 
 (test-begin "6.11 Exceptions")
 
-(skip-if-kawa "with-exception-handler and raise-continuable not implemented"
 (test 65
     (with-exception-handler
      (lambda (con) 42)
      (lambda ()
        (+ (raise-continuable "should be a number")
           23))))
-)
 
-(skip-if-kawa "error-object?, error-object-message, error-object-irritants, guard not implemented"
 (test #t
     (error-object? (guard (exn (else exn)) (error "BOOM!" 1 2 3))))
 (test "BOOM!"
     (error-object-message (guard (exn (else exn)) (error "BOOM!" 1 2 3))))
 (test '(1 2 3)
     (error-object-irritants (guard (exn (else exn)) (error "BOOM!" 1 2 3))))
-)
 
-(skip-if-kawa "file-error? and guard not implemented"
 (test #f
     (file-error? (guard (exn (else exn)) (error "BOOM!"))))
 (test #t
     (file-error? (guard (exn (else exn)) (open-input-file " no such file "))))
-)
 
-(skip-if-kawa "read-error? and guard not implemented"
 (test #f
     (read-error? (guard (exn (else exn)) (error "BOOM!"))))
 (test #t
     (read-error? (guard (exn (else exn)) (read (open-input-string ")")))))
-)
+
+(define something-went-wrong #f)
+(define (test-exception-handler-1 v)
+  (call-with-current-continuation
+   (lambda (k)
+     (with-exception-handler
+      (lambda (x)
+        (set! something-went-wrong (list "condition: " x))
+        (k 'exception))
+      (lambda ()
+        (+ 1 (if (> v 0) (+ v 100) (raise 'an-error))))))))
+(test 106 (test-exception-handler-1 5))
+(test #f something-went-wrong)
+(test 'exception (test-exception-handler-1 -1))
+(test '("condition: " an-error) something-went-wrong)
+
+(set! something-went-wrong #f)
+(define (test-exception-handler-2 v)
+  (guard (ex (else 'caught-another-exception))
+    (with-exception-handler
+     (lambda (x)
+       (set! something-went-wrong #t)
+       (list "exception:" x))
+     (lambda ()
+       (+ 1 (if (> v 0) (+ v 100) (raise 'an-error)))))))
+(test 106 (test-exception-handler-2 5))
+(test #f something-went-wrong)
+(test 'caught-another-exception (test-exception-handler-2 -1))
+(test #t something-went-wrong)
+
+;; Based on an example from R6RS-lib section 7.1 Exceptions.
+;; R7RS section 6.11 Exceptions has a simplified version.
+(let* ((out (open-output-string))
+       (value (with-exception-handler
+               (lambda (con)
+                 (cond
+                  ((not (list? con))
+                   (raise con))
+                  ((list? con)
+                   (display (car con) out))
+                  (else
+                   (display "a warning has been issued" out)))
+                 42)
+               (lambda ()
+                 (+ (raise-continuable
+                     (list "should be a number"))
+                    23)))))
+  (test "should be a number" (get-output-string out))
+  (test 65 value))
+
+;; From SRFI-34 "Examples" section - #3
+(define (test-exception-handler-3 v out)
+  (guard (condition
+          (else
+           (display "condition: " out)
+           (write condition out)
+           (display #\! out)
+           'exception))
+         (+ 1 (if (= v 0) (raise 'an-error) (/ 10 v)))))
+(let* ((out (open-output-string))
+       (value (test-exception-handler-3 0 out)))
+  (test 'exception value)
+  (test "condition: an-error!" (get-output-string out)))
+
+(define (test-exception-handler-4 v out)
+  (call-with-current-continuation
+   (lambda (k)
+     (with-exception-handler
+      (lambda (x)
+        (display "reraised " out)
+        (write x out) (display #\! out)
+        (k 'zero))
+      (lambda ()
+        (guard (condition
+                ((positive? condition)
+                 'positive)
+                ((negative? condition)
+                 'negative))
+          (raise v)))))))
+
+;; From SRFI-34 "Examples" section - #5
+(let* ((out (open-output-string))
+       (value (test-exception-handler-4 1 out)))
+  (test "" (get-output-string out))
+  (test 'positive value))
+;; From SRFI-34 "Examples" section - #6
+(let* ((out (open-output-string))
+       (value (test-exception-handler-4 -1 out)))
+  (test "" (get-output-string out))
+  (test 'negative value))
+;; From SRFI-34 "Examples" section - #7
+(let* ((out (open-output-string))
+       (value (test-exception-handler-4 0 out)))
+  (test "reraised 0!" (get-output-string out))
+  (test 'zero value))
+
+;; From SRFI-34 "Examples" section - #8
+(test 42
+    (guard (condition
+            ((assq 'a condition) => cdr)
+            ((assq 'b condition)))
+      (raise (list (cons 'a 42)))))
+
+;; From SRFI-34 "Examples" section - #9
+(test '(b . 23)
+    (guard (condition
+            ((assq 'a condition) => cdr)
+            ((assq 'b condition)))
+      (raise (list (cons 'b 23)))))
+
+(test 'caught-d
+      (guard (condition
+              ((assq 'c condition) 'caught-c)
+              ((assq 'd condition) 'caught-d))
+             (list
+              (sqrt 8)
+              (guard (condition
+                      ((assq 'a condition) => cdr)
+                      ((assq 'b condition)))
+                     (raise (list (cons 'd 24)))))))
 
 (test-end)
 
@@ -1593,14 +1817,14 @@
     (let ((f (eval '(lambda (f x) (f x x)) (null-environment 5))))
       (f + 10)))
 
-(skip-if-kawa "evironment not implemented"
 (test 1024 (eval '(expt 2 10) (environment '(scheme base))))
-(test 0.0 (eval '(sin 0) (environment '(scheme inexact))))
-(test 1024.0 (eval '(+ (expt 2 10) (sin 0))
+;; (sin 0) may return exact number
+(test 0.0 (inexact (eval '(sin 0) (environment '(scheme inexact)))))
+;; ditto
+(test 1024.0 (eval '(+ (expt 2 10) (inexact (sin 0)))
                    (environment '(scheme base) '(scheme inexact))))
 
-
-)(test-end)
+(test-end)
 
 (test-begin "6.13 Input and output")
 
@@ -1650,6 +1874,19 @@
 (test "abc" (read-string 3 (open-input-string "abcd")))
 (test "abc" (read-string 3 (open-input-string "abc\ndef\n")))
 
+(let ((in (open-input-string (string #\x10F700 #\x10F701 #\x10F702))))
+  (let* ((c1 (read-char in))
+         (c2 (read-char in))
+         (c3 (read-char in)))
+    (test #\x10F700 c1)
+    (test #\x10F701 c2)
+    (test #\x10F702 c3)))
+
+(test (string #\x10F700)
+    (let ((out (open-output-string)))
+      (write-char #\x10F700 out)
+      (get-output-string out)))
+
 (test "abc"
     (let ((out (open-output-string)))
       (write 'abc out)
@@ -1667,10 +1904,10 @@
       (display #\c out)
       (get-output-string out)))
 
-(test "\n"
-    (let ((out (open-output-string)))
-      (newline out)
-      (get-output-string out)))
+(test #t
+      (let* ((out (open-output-string))
+             (r (begin (newline out) (get-output-string out))))
+        (or (equal? r "\n") (equal? r "\r\n"))))
 
 (test "abc def"
     (let ((out (open-output-string)))
@@ -1827,6 +2064,11 @@
 (test 'def (read (open-input-string "#| abc |# def")))
 (test 'ghi (read (open-input-string "#| abc #| def |# |# ghi")))
 (test 'ghi (read (open-input-string "#; ; abc\n def ghi")))
+(test '(abs -16) (read (open-input-string "(#;sqrt abs -16)")))
+(test '(a d) (read (open-input-string "(a #; #;b c d)")))
+(test '(a e) (read (open-input-string "(a #;(b #;c d) e)")))
+(test '(a . c) (read (open-input-string "(a . #;b c)")))
+(test '(a . b) (read (open-input-string "(a . b #;c)")))
 
 (test #\a (read (open-input-string "#\\a")))
 (test #\space (read (open-input-string "#\\space")))
@@ -1996,10 +2238,8 @@
 (test-numeric-syntax "#o-10" -8 "-8")
 (test-numeric-syntax "#b-10" -2 "-2")
 ;; Combination of prefixes
-(skip-if-kawa "combination of prefixes broken"
 (test-numeric-syntax "#e#x10" 16 "16")
 (test-numeric-syntax "#i#x10" 16.0 "16.0" "16.")
-)
 ;; (Attempted) decimal notation with base prefixes
 (test-numeric-syntax "#d1." 1.0 "1.0" "1.")
 (test-numeric-syntax "#d.1" 0.1 "0.1" ".1" "100.0e-3")
@@ -2050,17 +2290,14 @@
 (test #t (exact? (jiffies-per-second)))
 
 (test #t (list? (features)))
-(test-expect-fail 1)
 (test #t (and (memq 'r7rs (features)) #t))
 
 (test #t (file-exists? "."))
 (test #f (file-exists? " no such file "))
 
-(skip-if-kawa "file-error? and guard missing"
 (test #t (file-error?
           (guard (exn (else exn))
             (delete-file " no such file "))))
-)
 
 (test-end)
 
